@@ -1,6 +1,6 @@
 /* acconfig.h
 
-   $Id: acconfig.h,v 1.3 1998/06/21 22:10:23 miers Exp $
+   $Id: acconfig.h,v 1.6 1999/03/05 04:38:56 rwscott Exp $
    This file is in the public domain.
 
    Descriptive text for the C preprocessor macros that
@@ -23,6 +23,7 @@
 #undef _ALL_SOURCE
 #endif
 
+#undef NEWMENUS
 /* Define if using alloca.c.  */
 #undef C_ALLOCA
 
@@ -186,6 +187,7 @@
 
 /* LessTif */
 #undef READDIR_LEXICOGRAPHIC_ORDER
+#undef LESSTIF_EDITRES
 
 /* Define as the return type of signal handlers (int or void).  */
 #undef RETSIGTYPE
@@ -244,6 +246,9 @@
    instead of <sys/cpustats.h>.  */
 #undef UMAX4_3
 
+/* XDND ? */
+#undef	USE_XDND
+
 /* Define if you do not have <strings.h>, index, bzero, etc..  */
 #undef USG
 
