@@ -79,6 +79,7 @@ extern int menuFromFrameOrWindowOrTitlebar;
 extern Bool DoHandlePageing;
 extern XtAppContext app;
 extern volatile int alarmed;
+extern Bool debugging;
 
 extern char **g_argv;
 
@@ -290,7 +291,7 @@ wait_quit_timeout(ScreenInfo *sinfo, MwmWindow *win)
 /*
  * circulate a window
  */
-MwmWindow *
+static MwmWindow *
 circulate(ScreenInfo *scr, MwmWindow *tmp_win, char *action, Bool Direction)
 {
     MwmWindow *t, *selected;
@@ -416,6 +417,12 @@ maximize(ScreenInfo *scr, MwmWindow *tmp_win, int val1, int val2,
 {
     int new_width, new_height, new_x, new_y;
 
+    /* First make sure that window is de-iconified */
+    if (tmp_win->flags & ICONIFIED)
+    {
+	    if (val1 <= 0)
+		ICON_DeIconify(scr, tmp_win);
+    }
     if (tmp_win->flags & MAXIMIZED)
     {
 	tmp_win->flags &= ~MAXIMIZED;
@@ -879,6 +886,49 @@ win_list(ScreenInfo *scr, int val1, int val2)
     free(mr);
 }
 
+/*
+ * Explicit focus : move focus
+ */
+static void
+ExplicitMoveFocus(ScreenInfo *scr, MwmWindow *win, XEvent *event)
+{
+    if (Mwm.keyboard_focus_policy != XmEXPLICIT) {
+	fprintf(stderr, "ExplicitMoveFocus(_,%p,_) pointer mode\n", win);
+	return;
+    }
+
+    fprintf(stderr, "ExplicitMoveFocus(_, %p, _)\n", win);
+    if (win) {
+	/* FIX ME move code from circulate() in here to make it work */
+    }
+}
+
+/***********************************************************************
+ *
+ *  Procedure:
+ *	(Un)normalize a window.
+ *
+ ***********************************************************************/
+static void
+normalize(ScreenInfo *scr, MwmWindow *tmp_win, int val1, int val2,
+	 int val1_unit, int val2_unit)
+{
+   /* Deiconify if iconified */
+   if (tmp_win->flags & ICONIFIED)
+   {
+      if (val1 <= 0)
+         ICON_DeIconify(scr, tmp_win);
+   } else
+      /* DeMaximize if maximized */
+      if (tmp_win->flags & MAXIMIZED) {
+         /* Taken from maximize() */
+         tmp_win->flags &= ~MAXIMIZED;
+         DEC_ConfigureDecorations(scr, tmp_win, tmp_win->orig_x, tmp_win->orig_y,
+               tmp_win->orig_wd, tmp_win->orig_ht, True);
+         DEC_DrawDecorations(scr, tmp_win, True, True, True, None);
+      }
+}
+
 /*
  * execute a mwm built in function
  */
@@ -894,6 +944,14 @@ FUNC_Execute(ScreenInfo *scr, int func, char *action, Window in_w,
     int warp_x = 0, warp_y = 0;
     Pixel TextColor, BackColor;
     Pixmap BackPixmap;
+    if (debugging) {
+       fprintf(stderr,
+             "FUNC_Execute: func %d %s context %s menu %s (%s)\n",
+             func, _MwmPrintF(func),
+             _MwmPrintC(context), 
+             menu ? menu->name : "(none)",
+             action);
+    }
 
     /* Defer Execution may wish to alter this value */
     w = in_w;
@@ -968,7 +1026,6 @@ FUNC_Execute(ScreenInfo *scr, int func, char *action, Window in_w,
     case F_EXEC:
 	{
 	    char *shell;
-	    extern char *getenv();
 
 	    if ((shell = getenv(MWM_SHELL_NAME)) == NULL)
 	    {
@@ -1049,11 +1106,14 @@ FUNC_Execute(ScreenInfo *scr, int func, char *action, Window in_w,
 	break;
 
     case F_MAXIMIZE:
-	if (tmp_win == NULL)
+	if (tmp_win == NULL) {
+           fprintf(stderr, "Could not maximize - tmp_win is 0\n");
 	    break;
+        }
 
 	if (function_allowed(func, tmp_win) == 0)
 	{
+           fprintf(stderr, "Could not maximize - op not allowed\n");
 	    XBell(dpy, scr->screen);
 	    break;
 	}
@@ -1127,15 +1187,23 @@ FUNC_Execute(ScreenInfo *scr, int func, char *action, Window in_w,
 	break;
 
     case F_NEXT_KEY:
+#if 1
+	ExplicitMoveFocus(scr, tmp_win, eventp);
+#else
 	fprintf(stderr, "NEXT_KEY NOT SUPPORTED YET\n");
+#endif
 	break;
 
     case F_NORMALIZE:
-	fprintf(stderr, "NORMALIZE NOT SUPPORTED YET\n");
+	if (tmp_win == NULL)
+	    break;
+	normalize(scr, tmp_win, val1, val2, val1_unit, val2_unit);
 	break;
 
     case F_NORM_AND_RAISE:
-	fprintf(stderr, "NORM_AND_RAISE NOT SUPPORTED YET\n");
+	normalize(scr, tmp_win, val1, val2, val1_unit, val2_unit);
+	if (tmp_win)
+	    WIN_Raise(scr, MISC_RootOfTree(tmp_win));
 	break;
 
     case F_PACK_ICONS:
@@ -1237,10 +1305,10 @@ FUNC_Execute(ScreenInfo *scr, int func, char *action, Window in_w,
 	break;
 
     case F_RESTART:
-	if ((Mwm.show_feedback & MWM_FEEDBACK_RESTART) && !action)
+       if ((Mwm.show_feedback & MWM_FEEDBACK_RESTART) && (!action || !*action))
 	    restart(scr);
 	else {
-	  if (!action) {
+         if (!action || !*action) {
 	    MWM_Done(1, g_argv[0]);
 	  }  else {
 	    MWM_Done(1, action);
