@@ -1,6 +1,6 @@
 /**
  *
- * $Id: MacrosI.h,v 1.31 1998/09/17 10:23:24 kbwj Exp $
+ * $Id: MacrosI.h,v 1.40 1999/12/23 04:25:19 rwscott Exp $
  * 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -25,6 +25,9 @@
 #ifndef MACROS_I_H
 #define MACROS_I_H
 
+#include <Xm/LabelGP.h>
+#include <Xm/LabelP.h>
+
 /*
  * object shorthand
  */
@@ -90,6 +93,8 @@
 #define	Shell_GrabKind(x) \
     (((ShellWidget)(x))->shell.grab_kind)
 
+#define Shell_AllowResize(x) \
+    (((ShellWidget)(x))->shell.allow_shell_resize)
 
 /*
  * one that Motif should have but doesn't
@@ -491,6 +496,9 @@
 #define AB_ArmTimeStamp(w) \
     (((XmArrowButtonWidget)(w))->arrowbutton.armTimeStamp)
 
+#define AB_DetailShadowThickness(w) \
+    (((XmArrowButtonWidget)(w))->arrowbutton.detail_shadow_thickness)
+
 /*
  * ARROW BUTTON GADGET ACCESS MACROS
  */
@@ -527,6 +535,9 @@
 #define ABG_ArmTimeStamp(w) \
     (((XmArrowButtonGadget)(w))->arrowbutton.armTimeStamp)
 
+#define ABG_DetailShadowThickness(w) \
+    (((XmArrowButtonGadget)(w))->arrowbutton.detail_shadow_thickness)
+
 /*
  * BULLETINBOARD WIDGET ACCESS MACROS
  */
@@ -1234,7 +1245,7 @@
 
 /* constraint_macros */
 #define FormC_Atta(w, i) \
-    (((XmFormConstraintRec *)((w)->core.constraints))->form.atta[(i)])
+    (((XmFormConstraintRec *)((w)->core.constraints))->form.att[(i)])
 
 #define FormC_NextSib(w) \
     (((XmFormConstraintRec *)((w)->core.constraints))->form.next_sibling)
@@ -1252,16 +1263,16 @@
     (((XmFormConstraintRec *)((w)->core.constraints))->form.preferred_height)
 
 #define FormC_X(w) \
-    (((XmFormConstraintRec *)((w)->core.constraints))->form.x)
+    (((XmFormConstraintRec *)((w)->core.constraints))->form.att[0].value)
 
 #define FormC_Y(w) \
-    (((XmFormConstraintRec *)((w)->core.constraints))->form.y)
+    (((XmFormConstraintRec *)((w)->core.constraints))->form.att[2].value)
 
 #define FormC_W(x) \
-    (((XmFormConstraintRec *)((x)->core.constraints))->form.w)
+    (((XmFormConstraintRec *)((x)->core.constraints))->form.att[1].value)
 
 #define FormC_H(w) \
-    (((XmFormConstraintRec *)((w)->core.constraints))->form.h)
+    (((XmFormConstraintRec *)((w)->core.constraints))->form.att[3].value)
 
 #define FormC_WFS(w) \
     (((XmFormConstraintRec *)((w)->core.constraints))->form.width_from_side)
@@ -1271,7 +1282,7 @@
 
 /* different look */
 #define FCP_Atta(c, i) \
-    (((XmFormConstraintRec *)(c))->form.atta[(i)])
+    (((XmFormConstraintRec *)(c))->form.att[(i)])
 
 #define FCP_NextSib(c) \
     (((XmFormConstraintRec *)(c))->form.next_sibling)
@@ -1289,22 +1300,24 @@
     (((XmFormConstraintRec *)(c))->form.preferred_height)
 
 #define FCP_X(c) \
-    (((XmFormConstraintRec *)(c))->form.x)
+    (((XmFormConstraintRec *)(c))->form.att[0].value)
 
 #define FCP_Y(c) \
-    (((XmFormConstraintRec *)(c))->form.y)
+    (((XmFormConstraintRec *)(c))->form.att[2].value)
 
 #define FCP_W(c) \
-    (((XmFormConstraintRec *)(c))->form.w)
+    (((XmFormConstraintRec *)(c))->form.att[1].value)
 
 #define FCP_H(c) \
-    (((XmFormConstraintRec *)(c))->form.h)
+    (((XmFormConstraintRec *)(c))->form.att[3].value)
 
+#if 0
 #define FCP_WidthFromSide(c) \
     (((XmFormConstraintRec *)(c))->form.width_from_side)
 
 #define FCP_HeightFromSide(c) \
     (((XmFormConstraintRec *)(c))->form.height_from_side)
+#endif
 
 /*
  * FILE SELECTIONBOX WIDGET ACCESS MACROS
@@ -1990,6 +2003,9 @@
 #define RCClass_MenuProcs(c) \
     (((XmRowColumnClassRec *)(c))->row_column_class.menuProcedures)
 
+#define RCClass_MenuTraverse(c, dir) \
+    (((XmRowColumnClassRec *)(XtClass(c)))->row_column_class.menuProcedures)(XmMENU_TRAVERSAL, c, dir)
+
 #define RCClass_ArmAndActivate(c) \
     (((XmRowColumnClassRec *)(c))->row_column_class.armAndActivate)
 
@@ -2014,6 +2030,7 @@
 #define RC_MenuShellPopdown(w,e,v) (((XmRowColumnClassRec *)(XtClass(XtParent(w))))->row_column_class.menuProcedures)(XmMENU_SHELL_POPDOWN, w, e, v)
 #define RC_MenuSubmenu(w) (((XmRowColumnClassRec *)(XtClass(XtParent(w))))->row_column_class.menuProcedures)(XmMENU_SUBMENU, w)
 #define RC_MenuCascading(w,e) (((XmRowColumnClassRec *)(XtClass(XtParent(w))))->row_column_class.menuProcedures)(XmMENU_CASCADING, w, e)
+#define RC_MenuMenuCallback(w,cbs) (((XmRowColumnClassRec *)(XtClass(XtParent(w))))->row_column_class.menuProcedures)(XmMENU_CALLBACK, w, cbs)
 
 /*
  * SASH WIDGET ACCESS MACROS
