@@ -1,6 +1,6 @@
 /**
  *
- * $Id: XmI.h,v 1.20 1998/10/21 22:46:33 danny Exp $
+ * $Id: XmI.h,v 1.30 1999/09/11 04:28:39 gritton Exp $
  * 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -45,6 +45,9 @@ extern "C" {
 #include <Xm/PrimitiveP.h>
 #include <Xm/GadgetP.h>
 #include <Xm/DrawP.h>
+#include <Xm/RowColumnP.h>
+#include <Xm/SelectioB.h>
+#include <Xm/Text.h>
 #include <XmI/MacrosI.h>
 
 /*
@@ -138,23 +141,23 @@ void _XmError(Widget w, char *message, ...);
  */
 typedef struct
 {
-    /* NOTE: The next three lines ABSOLUTELY MUST MATCH the first
+    /* NOTE: The next four lines ABSOLUTELY MUST MATCH the first
      * three lines in XmMWValues!!!!!!! */
     Boolean   ShowVSB, ShowHSB, HasHSB, HasVSB;
     Position  HsbX, HsbY, VsbX, VsbY, ClipX, ClipY, WorkX, WorkY;
     int       HsbW, HsbH, VsbW, VsbH, ClipW, ClipH, WorkW, WorkH;
-    int       SwW, SwH;
+    int       SwY, SwW, SwH;
 }
 XmSWValues;
 
 typedef struct
 {
-    /* NOTE: The next three lines ABSOLUTELY MUST MATCH the first
+    /* NOTE: The next four lines ABSOLUTELY MUST MATCH the first
      * three lines in XmSWValues!!!!! */
     Boolean   ShowVSB, ShowHSB, HasHSB, HasVSB;
     Position  HsbX, HsbY, VsbX, VsbY, ClipX, ClipY, WorkX, WorkY;
     int       HsbW, HsbH, VsbW, VsbH, ClipW, ClipH, WorkW, WorkH;
-    int       MwW, MwH;
+    int       SwY, MwW, MwH;
     int       mbw, mbh, cww, cwh, mww, mwh, www, wwh;
     int       s1w, s1h, s2w, s2h, s3w, s3h;
     Position  mbx, mby, cwx, cwy, mwx, mwy, wwx, wwy;
@@ -176,15 +179,15 @@ XmFontListEntry _XmFontListEntryFromTag(XmFontList fontlist, char *tag);
 
 /* for vendor */
 extern void _XmInitProtocols(Widget w);
+extern void _XmDestroyProtocols(Widget w);
+#ifdef LESSTIF_EDITRES
+extern void _XmNSEEditResCheckMessages(Widget w, XtPointer data,
+				       XEvent *event, Boolean *cont);
+#endif
 
 /* Things for Label/LabelG */
 
 extern void _XmLabelGetPixmapSize(Widget w, Pixmap Pix, Dimension *width, Dimension *height);
-extern void _XmLabelAccTextSize(Widget w);
-extern Boolean _XmLabelShowsAccelerators(Widget w);
-extern Boolean _XmLabelShowsMnemonic(Widget w);
-extern void _XmExportLabelString(Widget w, int offset, XtArgVal *value);
-extern XmImportOperator _XmImportLabelString(Widget w, int offset, XtArgVal *value);
 
 /* For buttons */
 #define ACTIVATE_DELAY	100
@@ -236,6 +239,7 @@ extern Boolean _XmGetInPMMode(Widget w);
 #if 0
 extern void _XmFixOptionMenu(Widget, Boolean);
 #endif
+extern void _XmRCSetKidGeo(XmRCKidGeometry kg, Widget instigator);
 
 /* used as the operation parameter for _XmMenuFocus. */
 enum {
@@ -274,13 +278,18 @@ extern void _XmManagerInstallMnemonic(Widget m, Widget w, KeySym mn);
 extern void _XmManagerUninstallAccelerator(Widget m, Widget w);
 extern void _XmManagerUninstallMnemonic(Widget m, Widget w);
 
-/* for ScrollBar */
-extern Position _XmScrollBarValueToPos(Widget sw, int value);
-extern Dimension _XmScrollBarSliderPixSize(Widget sw);
-
 /* for ScrolledW */
-extern void _XmScrolledWindowPreferredSize(Widget w, XtWidgetGeometry *request,
-				      XtWidgetGeometry *answer);
+/* T. Straumann: */
+extern void _XmScrolledWPreferredSize(Widget w, Widget instigator,
+				      XtWidgetGeometry *instigator_geom, XmSWValues * vals);
+extern void _XmScrolledWLayout(Widget w, Widget instigator,
+				      XtWidgetGeometry *instigator_geom, XmSWValues * vals);
+extern void _XmScrolledWConfigureChildren(Widget w, Widget instigator,
+				      XtWidgetGeometry *instigator_geom, XmSWValues * vals);
+
+/* from SelectionBox.c */
+extern Boolean _XmSelectionBoxMatch(XmSelectionBoxWidget w);
+
 
 /* for TearOff */
 extern void _XmPushButtonSetTranslation(Widget, int);
@@ -288,6 +297,10 @@ extern void _XmPushButtonSetTranslation(Widget, int);
 /* TearOff */
 extern void _XmTearOffInitiate(Widget w, XEvent *event);
 
+/* Text */
+void _XmChangeVSB(XmTextWidget w, XmTextPosition pos);
+void _XmRedisplayHBar(XmTextWidget w, int offset);
+
 /* Traversal */
 extern void _XmSetFocusResetFlag(Widget w, Boolean value);
 extern Boolean _XmGetFocusResetFlag(Widget w);
@@ -325,6 +338,11 @@ extern void _XmAddBackgroundToColorCache(Screen *screen, Colormap color_map,
 					 int rgb_fallback, XrmValue *val);
 extern void _XmInvalidateColorCache(Boolean default_only);
 
+/* XmString */
+
+extern void _XmStringBaselines(XmFontList fontlist, _XmString string,
+			       Position y, Dimension *baselines);
+
 /*
  * Behaviour control for XmFormLayout, XmRowColumnLayout.
  * This is their second parameter.
