@@ -1,6 +1,6 @@
 
 /* 
- * $Id: lookup.c,v 1.2 1997/09/08 23:21:41 miers Exp $
+ * $Id: lookup.c,v 1.3 1999/05/28 20:43:20 danny Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -24,6 +24,9 @@
  *  Original author:  Geoffrey W. Ritchey
  *                    codesmit@southwind.net
  **/
+
+#include <LTconfig.h>
+
 #include "Mrm/MrmPublic.h"
 
 #include <Xm/Xm.h>
@@ -271,7 +274,7 @@ ArgNamesType ArgNames[] =
     {"XmNbottomPosition", XmNbottomPosition, CSG},
     {"XmNbottomShadowColor", XmNbottomShadowColor, CSG},
     {"XmNbottomShadowPixmap", XmNbottomShadowPixmap, CSG},
-    {"XmNbottomWidget", XmNbottomWidget, 0},
+    {"XmNbottomWidget", XmNbottomWidget, /* 0 */ CSG},
     {"XmNbrowseSelectionCallback", XmNbrowseSelectionCallback, C},
     {"XmNbuttonAcceleratorText", XmNbuttonAcceleratorText, C},
     {"XmNbuttonAccelerators", XmNbuttonAccelerators, C},
@@ -548,7 +551,7 @@ ArgNamesType ArgNames[] =
     {"XmNleftAttachment", XmNleftAttachment, CSG},
     {"XmNleftOffset", XmNleftOffset, CSG},
     {"XmNleftPosition", XmNleftPosition, CSG},
-    {"XmNleftWidget", XmNleftWidget, 0},
+    {"XmNleftWidget", XmNleftWidget, /* 0 */ CSG},
     {"XmNlightThreshold", XmNlightThreshold, C},
     {"XmNlistItemCount", XmNlistItemCount, CSG},
     {"XmNlistItems", XmNlistItems, CSG},
@@ -724,7 +727,7 @@ ArgNamesType ArgNames[] =
     {"XmNrightAttachment", XmNrightAttachment, CSG},
     {"XmNrightOffset", XmNrightOffset, CSG},
     {"XmNrightPosition", XmNrightPosition, CSG},
-    {"XmNrightWidget", XmNrightWidget, 0},
+    {"XmNrightWidget", XmNrightWidget, /* 0 */ CSG},
     {"XmNrowColumnType", XmNrowColumnType, CG},
     {"XmNrows", XmNrows, CSG},
     {"XmNrubberPositioning", XmNrubberPositioning, CSG},
@@ -861,7 +864,7 @@ ArgNamesType ArgNames[] =
     {"XmNtopPosition", XmNtopPosition, CSG},
     {"XmNtopShadowColor", XmNtopShadowColor, CSG},
     {"XmNtopShadowPixmap", XmNtopShadowPixmap, CSG},
-    {"XmNtopWidget", XmNtopWidget, 0},
+    {"XmNtopWidget", XmNtopWidget, /* 0 */ CSG},
     {"XmNtransferProc", XmNtransferProc, CSG},
     {"XmNtransferStatus", XmNtransferStatus, CSG},
     {"XmNtransient", XmNtransient, CSG},
