@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/GrabShell.c,v 1.13 1998/09/24 15:37:37 pgw Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/GrabShell.c,v 1.17 1999/12/21 22:14:18 amai Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/GrabShell.c,v 1.13 1998/09/24 15:37:37 pgw Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/GrabShell.c,v 1.17 1999/12/21 22:14:18 amai Exp $";
 
 
 #include <LTconfig.h>
@@ -38,6 +38,7 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/GrabShe
 
 #include <XmI/MessagesI.h>
 #include <XmI/DebugUtil.h>
+#include <XmI/PopupUtil.h>
 
 
 /*
@@ -76,9 +77,12 @@ static void CreateTopShadowGC(Widget w);
 static void Destroy(Widget w);
 static void DoLayout(Widget w);
 static void DrawBorder(Widget w, XEvent *xev, Region region);
+static void ForceExpose(Widget w);
+static Widget GetFirstManagedChild(Widget w);
 static XtGeometryResult GeometryManager(Widget w, 
    XtWidgetGeometry *request, 
    XtWidgetGeometry *reply);
+static int IgnoreError(Display *d, XErrorEvent *e);
 static void Initialize(Widget request, 
    Widget c_new,
    ArgList args,
@@ -99,6 +103,17 @@ static void _XmGrabShellFocus(Widget w, int operation, Time _time);
 static void _XmGrabShellGrab(Widget w, Time _time);
 static void _XmGrabShellUngrab(Widget w, Time _time);
 
+#if 0
+
+typedef int (*XErrorHandler) (      /* WARNING, this type not in Xlib spec */
+#if NeedFunctionPrototypes
+    Display*            /* display */,
+    XErrorEvent*        /* error_event */
+#endif
+);
+
+#endif
+
 enum {
   XmGRAB_SHELL_FOCUS_SAVE=0,
   XmGRAB_SHELL_FOCUS_RESTORE,
@@ -248,7 +263,11 @@ XmGrabShellClassRec xmGrabShellClassRec = {
    },
    /* Composite class part */
    {
+#if 0
+      XtInheritGeometryManager, 	/* geometry manager */
+#else
       GeometryManager,        /* geometry manager */
+#endif
       ChangeManaged,          /* ChangeManaged */
       XtInheritInsertChild,   /* insert_child */ 
       XtInheritDeleteChild,   /* delete_child */ 
@@ -368,20 +387,49 @@ BtnUp(
 static void 
 ChangeManaged(Widget w)
 {
+   Widget child;
+   XtWidgetGeometry pref;
+   XtGeometryResult res;
+   Dimension adjustment;
 
    DEBUGOUT(XdbDebug(__FILE__, w,
                      "ChangeManaged: dimensions are %d %d\n",
                       XtWidth(w), XtHeight(w)));
 
-   DoLayout(w);
+   /* Shells only support one child. */
+   child = GetFirstManagedChild(w); 
 
-   if(!XtIsManaged(w))
+   if(!child)
    {
-
+      /* This will happen if being realized and there are no managed child */
       DEBUGOUT(XdbDebug(__FILE__, w, 
-                        "ChangeManaged: Not managed yet.\n"));
+                        "ChangeManaged: No managed child bye.\n"));
       return;
+   } 
+  
+   /* Get the preferred size of the child */
+   XtQueryGeometry(child, NULL, &pref);
+   DEBUGOUT(XdbDebug(__FILE__, child,
+                     "get preferred size of parent from child %s\n",
+                     XdbWidgetGeometry2String(&pref)));
+
+   /* adjust for shadow thickness */
+   adjustment = GS_ShadowThickness(w); 
+   pref.width = pref.width == 0 ? 1 : pref.width + (2*adjustment);
+   pref.height = pref.height == 0 ? 1 : pref.height + (2*adjustment);
+   /* change the width and height */
+   pref.request_mode = CWWidth | CWHeight;
+
+   /* Make the geometry request */
+   res = _XmMakeGeometryRequest(w, &pref);
+   if (res == XtGeometryNo)
+   {
+       DEBUGOUT(XdbDebug(__FILE__, w,
+                        "XtGeometryNo returned... THIS SHOULD NOT HAPPEN\n"));
    }
+
+   /* Lay me out */
+   DoLayout(w);
 }
 
 /*
@@ -487,45 +535,37 @@ static void
 DoLayout(Widget w)
 {
    Widget child = NULL;
+   Position child_x, child_y;
+   Dimension child_width, child_height;
+   Dimension adjustment;
 
    DEBUGOUT(XdbDebug(__FILE__, w, "DoLayout ...\n"));
 
-   /* 
-      REMARK: 
-      Using MGR_* macros does a cast to a structure we are not using.  
-      However, the pointer ends up pointing to what we want since we
-      are accessing the composite structure.  See the macros MGR_NumChildren
-      and MGR_Children in MacrosI.h if you don't understand this.  
-      Does anyone see a problem with this? 
-      END:
-   */
-   if(MGR_NumChildren(w) > 0)
-   {
-      child = MGR_Children(w)[0]; /* take first child */
-   }
+   child = GetFirstManagedChild(w);
 
-   /* 
-      REMARK: 
-      Use dimensions of first child to determine width and height.
-      If dimensions of child are 0x0 then set to 1x1.
-      END:
-   */
    if(child != NULL)
    {
-      int adjustment;
+      DEBUGOUT(XdbDebug(__FILE__, w,
+                        "DoLayout: dimensions before change are %u %u\n",
+                        XtWidth(w), XtHeight(w)));
 
       adjustment = GS_ShadowThickness(w); 
 
-      XtWidth(w) = XtWidth(child) == 0 ? 1 : XtWidth(child) + (2*adjustment);
-      XtHeight(w) = XtHeight(child) == 0 ? 1 : XtHeight(child) + (2*adjustment);
-
-      DEBUGOUT(XdbDebug(__FILE__, w,
-                        "DoLayout: dimensions changed to %d %d\n",
-                        XtWidth(w), XtHeight(w)));
-
       /* change childs x,y */
-      XtX(child) = adjustment;
-      XtY(child) = adjustment;
+      child_x = adjustment;
+      child_y = adjustment;
+
+      /* change child width,height */
+      child_width = XtWidth(w) - 2*adjustment; 
+      child_height = XtHeight(w) - 2*adjustment;
+
+      /* configure child object */
+      XmeConfigureObject(child,
+                         child_x,
+                         child_y,
+                         child_width,
+                         child_height,
+                         XtBorderWidth(child));
    }
 }
 
@@ -557,6 +597,65 @@ DrawBorder(Widget w, XEvent *xev, Region region)
    Unused(region);
 }
 
+/*
+  FUNCTION: ForceExpose
+  SYNOPSIS: static void ForceExpose(Widget w)
+  DESCRIPTION:
+  This function forces an expose on the widget.
+  END:
+*/
+static void
+ForceExpose(Widget w)
+{
+   Display *dpy;
+
+   dpy = XtDisplayOfObject(w);
+
+   XFlush(dpy);
+
+   DrawBorder(w, NULL, NULL);
+}
+
+/*
+  FUNCTION: GetFirstManagedChild
+  SYNOPSIS: statis Widget GetFirstManageChild(Widget composite)
+  DESCRIPTION:
+  This function returns the first managed child widget or NULL.
+  END:
+*/ 
+static Widget
+GetFirstManagedChild(Widget w)
+{
+   int i;
+   Widget child = NULL;
+
+   /* 
+      REMARK: 
+      Using MGR_* macros does a cast to a structure we are not using.  
+      However, the pointer ends up pointing to what we want since we
+      are accessing the composite structure.  See the macros MGR_NumChildren
+      and MGR_Children in MacrosI.h if you don't understand this.  
+      Does anyone see a problem with this? 
+      END:
+   */
+
+   for(i = 0; i < MGR_NumChildren(w); i++) 
+   {
+      if(XtIsManaged(MGR_Children(w)[i])) 
+      {
+         /*
+            REMARK:
+            Any subclass of the shell that manages more then one
+            child is broken.
+            END:
+         */
+         child = MGR_Children(w)[i]; /* first managed child */
+         break;          
+      }
+   }
+   return child;
+}
+
 /*
   FUNCTION: GeometryManager
   SYNOPSIS: static XtGeometryResult(Widget, XtWidgetGeometry *, XtWidgetGeometry *)
@@ -580,11 +679,17 @@ GeometryManager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
                      ? "True"
                      : "False"));
 
-   if ((request->request_mode & (CWWidth | CWHeight)) == 0)
+   if (Shell_AllowResize(me) == FALSE && XtIsRealized(w))
    {
-       return XtGeometryYes;
+       /* No changes allowed after being realized. */
+       return XtGeometryNo;
    }
 
+   if (request->request_mode & (CWX | CWY))
+   {
+       /* Can not change my x or y */
+       return XtGeometryNo;
+   }
 
    DEBUGOUT(XdbDebug(__FILE__, me,
                      "geometry_manager: %s\n",
@@ -596,13 +701,21 @@ GeometryManager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 
    if (res == XtGeometryNo)
    {
-      DEBUGOUT(XdbDebug(__FILE__, w,
+       DEBUGOUT(XdbDebug(__FILE__, w,
                         "XtGeometryNo returned... THIS SHOULD NOT HAPPEN\n"));
    }
    *reply = wants;
 
-   XtWidth(w) = wants.width;
-   XtHeight(w) = wants.height;
+   if (!(request->request_mode & XtCWQueryOnly))
+   {
+       /* if not a query then set the values */
+       XtWidth(w) = wants.width;
+       XtHeight(w) = wants.height;
+       if (request->request_mode & CWBorderWidth)
+       {
+          XtX(w) = XtY(w) = -request->border_width;
+       }
+   }
 
    DEBUGOUT(XdbDebug(__FILE__, w,
                      "geometry_manager: size %dx%d => Yes\n",
@@ -611,6 +724,42 @@ GeometryManager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
    return XtGeometryYes;
 }
 
+/*
+  FUNCTION: IgnoreError
+  SYNOPSIS: 
+  DESCRIPTION: static int IgnoreError(Display *d, XErrorEvent *e)
+  This function is used to ignore X errors.  Typically, this will occur
+  when doing grabs.  I wonder if ignoring errors is bad idea.  Any one
+  have any thoughts on the issue.
+  END:
+*/
+static int 
+IgnoreError(Display *d, XErrorEvent *e)
+{
+
+   DEBUGOUT(XdbDebug(__FILE__, NULL, "In IgnoreError\n"));
+   DEBUGOUT(XdbDebug(__FILE__, NULL, 
+                     "type: %d\n"
+                     "display: %p\n"
+                     "resourceid: %p\n"
+                     "serial: %ul\n"
+                     "error_code: %u\n"
+                     "request_code: %u\n"
+                     "minor_code: %u\n",
+                     e->type,
+                     e->display,
+                     e->resourceid,
+                     e->serial,
+                     (unsigned int) e->error_code,
+                     (unsigned int) e->request_code,
+                     (unsigned int) e->minor_code));
+
+   Unused(d);
+   Unused(e);
+
+   return 0;
+}
+
 /*
   FUNCTION: Initialize
   SYNOPSIS: static void Initialize(Widget,Widget,ArgList,Cardinal *)
@@ -652,6 +801,7 @@ Initialize(Widget request,
                      (XtEventHandler) MapNotifyHandler, 
                      (XtPointer) NULL);
 
+#if 0
 #define FAKE_NONZERO
 #ifdef  FAKE_NONZERO
    if (XtWidth(c_new) == 0 || XtHeight(c_new) == 0)
@@ -663,6 +813,7 @@ Initialize(Widget request,
       XtWidth(c_new) = XtHeight(c_new) = 1;
    }
 #endif
+#endif
 }
 
 /*
@@ -678,6 +829,7 @@ static void
 MapNotifyHandler(Widget w, XtPointer cdata, XEvent *xev, Boolean *pb)
 {
    int event_mode;
+   XErrorHandler old_handler;
 
    *pb = TRUE;
 
@@ -702,15 +854,38 @@ MapNotifyHandler(Widget w, XtPointer cdata, XEvent *xev, Boolean *pb)
    /* Save post time. */
    GS_PostTime(w) = XtLastTimestampProcessed(XtDisplay(w));
 
+   ForceExpose(w);
+
+#if 1
+#if 1
    _XmGrabShellGrab(w, GS_PostTime(w));
+#else
+   _XmGrabShellGrab(w, CurrentTime);
+#endif
+#endif
 
    /* Only needed if using GrabModeSync..I think */
    event_mode = (GS_GrabStyle(w) == GrabModeSync) ? SyncBoth : AsyncBoth;
-   _XmAllowEvents(w, event_mode, GS_PostTime(w));
 
+#if 1
+   _XmAllowEvents(w, event_mode, GS_PostTime(w));
+#else
+   _XmAllowEvents(w, event_mode, CurrentTime);
+#endif
+
+   /* The time parameter is ignored on a XmGRAB_SHELL_FOCUS_SAVE */
    _XmGrabShellFocus(w, XmGRAB_SHELL_FOCUS_SAVE, GS_PostTime(w));
+
+   /* Setup ignore X error routine */
+   old_handler = XSetErrorHandler(IgnoreError);
+
    _XmGrabShellFocus(w, XmGRAB_SHELL_FOCUS_SET, CurrentTime);
 
+   /* Should I sync here? */
+   XSync(XtDisplay(w),False);
+
+   /* Set error handler back to the old error handler */
+   XSetErrorHandler(old_handler);
 }
 
 /*
@@ -729,6 +904,8 @@ Popdown(
    String * params,
    Cardinal * num_params)
 {
+   XErrorHandler old_handler;
+
    DEBUGOUT(XdbDebug(__FILE__, w, "Popdown\n"));
 
    /* Check to see if mapped yet */
@@ -745,17 +922,23 @@ Popdown(
    _XmGrabShellUngrab(w, GS_UnpostTime(w));
 
    /* popdown the shell */
-   XtPopdown(w);
+   _XmPopdown(w);
 
    /* Unlock map notify handler */
    GS_Mapped(w) = False;
 
+   /* Setup ignore X error routine */
+   old_handler = XSetErrorHandler(IgnoreError);
+
    /* restore focus */
    _XmGrabShellFocus(w, XmGRAB_SHELL_FOCUS_RESTORE, GS_UnpostTime(w));
 
    /* Should I sync here? */
    XSync(XtDisplay(w),False);
 
+   /* Set error handler back to the old error handler */
+   XSetErrorHandler(old_handler);
+
    Unused(xev);
    Unused(params);
    Unused(num_params);
@@ -772,14 +955,18 @@ Popdown(
 static void 
 Resize(Widget w)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "Resize ...\n"));
+   DEBUGOUT(XdbDebug(__FILE__, w, "%s:Resize %ix%i\n",
+        __FILE__,
+        XtWidth(w), XtHeight(w)));
 
+   /* FIX ME: Should resize regardless if realized.  However,
+              this sort of fixes a geometry problem with the scrolled
+              window.  Remove the if realized check when scrolled
+              window geometry is working correctly.  */ 
    if(XtIsRealized(w))
    {  /* widgets must be resized */
       DoLayout(w);
    }
-
-   (*superclass->core_class.resize)(w);
 }
 
 /* 
