@@ -1,6 +1,7 @@
+#define	LARGE_PAGE
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Notebook.c,v 1.14 1998/09/24 15:37:38 pgw Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Notebook.c,v 1.26 1999/12/29 18:24:44 danny Exp $
  *
  * Copyright (C) 1997-1998 Free Software Foundation, Inc.
  *
@@ -22,10 +23,27 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Notebook.c,v 1.14 1998/09/24 15:37:38 pgw Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Notebook.c,v 1.26 1999/12/29 18:24:44 danny Exp $";
+
 
 /*
- * Also recognise 1.2 widgets
+ * This is a widget whose main purpose is to do geometry management for its
+ *	children. Fortunately it is rather simple geometry management, unlike
+ *	some of the other Motif widgets.
+ *
+ * Peculiarities that you should know about :
+ * - Children are not unmanaged to render them invisible. They are moved out
+ *   of sight by moving them to position -10000,-10000.
+ * - Layout of children is performed in several places: in Layout() and in
+ *   _XmNotebookShowChild().
+ * - Size of the Notebook and its children is determined by PreferredSize()
+ *   which does an algorithm similar to the one in Layout(). Changes in one
+ *   of these functions probably need to be reflected in the other as well.
+ */
+/*
+ * What's TRAIT_WORKAROUND ?
+ *
+ * It serves to also recognise 1.2 widgets.
  * This is needed to make Notebook work right until all 1.2 widgets
  * have a 2.0 equivalent which holds all the right traits.
  *
@@ -40,6 +58,7 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Noteboo
 #include <XmI/DebugUtil.h>
 
 #include <Xm/NotebookP.h>
+#include <XmI/XmXpm.h>
 
 /*
  * Traits used by Notebook
@@ -126,9 +145,55 @@ static XmScrollFrameTraitRec _XmNBScrollFrameTraitRec = {
 	/* addNavigator */	_XmNBTraitAddNavigator,
 	/* removeNavigator */	_XmNBTraitRemoveNavigator
 };
+
 /*
  * Resources for the Notebook class
  */
+
+#define spiral_width 36
+#define spiral_height 17
+static char spiral_bits[] = {
+ 0x00,0x10,0x00,0x00,0xf0,0x00,0x10,0x00,0x00,0xf0,0x00,0x10,0x00,0x00,0xf0,
+ 0x00,0xff,0x01,0x00,0xf0,0xe0,0xff,0x0f,0x00,0xf0,0xf8,0xff,0x3f,0x00,0xf0,
+ 0xfc,0xff,0x7f,0x00,0xf0,0xfe,0x11,0xf8,0x03,0xf0,0x1f,0x10,0xc0,0x05,0xf0,
+ 0x0f,0x10,0xa0,0x09,0xf0,0x0f,0x10,0x20,0x09,0xf0,0x0e,0x10,0xa0,0x09,0xf0,
+ 0x1c,0x10,0xc0,0x04,0xf0,0x38,0x10,0x80,0x03,0xf0,0xe0,0x11,0x00,0x00,0xf0,
+ 0x00,0x1f,0x00,0x00,0xf0,0x00,0x10,0x00,0x00,0xf0};
+
+
+
+/* XPM */
+static char *spiral[] = {
+/* width height num_colors chars_per_pixel */
+"    36    17        5            1",
+/* colors */
+". c #000000",
+"# c #989898",
+"a c #c0c4c0",
+"b c #c8ccc8",
+"c c #f8fcf8",
+/* pixels */
+"cccccccccccc.aaaaaaaaaaaaaaaaaaaaaaa",
+"cccccccccccc.aaaaaaaaaaaaaaaaaaaaaaa",
+"cccccccccccc.aaaaaaaaaaaaaaaaaaaaaaa",
+"ccccccccbbbbbbbbbaaaaaaaaaaaaaaaaaaa",
+"cccccbbb.........bbbaaaaaaaaaaaaaaaa",
+"cccbb...............bbaaaaaaaaaaaaaa",
+"ccb......##########...baaaaaaaaaaaaa",
+"cb....###ccc.aaaaaa###.b##aaaaaaaaaa",
+"b...c#cccccc.aaaaaaaaa#.bcbaaaaaaaaa",
+"b..c#ccccccc.aaaaaaaa#c#bccbaaaaaaaa",
+"b..#cccccccc.aaaaaaaa#cc#ccbaaaaaaaa",
+"cb.#cccccccc.aaaaaaaa#cb#ccbaaaaaaaa",
+"ccb.#ccccccc.aaaaaaaaab#ccbaaaaaaaaa",
+"cccbb#cccccc.aaaaaaaaaabbbaaaaaaaaaa",
+"cccccb###ccc.aaaaaaaaaaaaaaaaaaaaaaa",
+"ccccccccb###.aaaaaaaaaaaaaaaaaaaaaaa",
+"cccccccccccc.aaaaaaaaaaaaaaaaaaaaaaa"
+};
+
+
+
 #define Offset(field) XtOffsetOf(XmNotebookRec, notebook.field)
 static XtResource resources[] =
 {
@@ -172,14 +237,14 @@ static XtResource resources[] =
 	XmNbindingWidth, XmCBindingWidth,
 	XmRDimension,
 	sizeof(Dimension), Offset(binding_width),
-	XmRImmediate, (XtPointer)25
+       XmRImmediate, (XtPointer) spiral_width
     },
-#if 0
+#if 1
     {
 	XmNcurrentPageNumber, XmCCurrentPageNumber,
 	XmRInt,
 	sizeof(int), Offset(current_page_number),
-	XmRImmediate, (XtPointer)0	/* FIX ME should be dynamic */
+	XmRImmediate, (XtPointer)1	/* FIX ME should be dynamic */
     },
 #endif
     {
@@ -312,7 +377,7 @@ static XtResource ConstraintResources[] = {
 };
 static XtActionsRec actions[] =
 {
-    {"TraverseTab", NULL},
+    {"TraverseTab", TraverseTab},
 };
 
 /* FIX ME - these should go in other files */
@@ -486,7 +551,7 @@ ClassPartInitialize(WidgetClass widget_class)
  * Show the child indicated, don't show any other XmPAGE child.
  */
 static void
-_XmNotebookShowChild(Widget nb, Widget child)
+_XmNotebookShowChild(Widget nb, Widget child, Widget status)
 {
 	int	i;
 
@@ -497,30 +562,56 @@ _XmNotebookShowChild(Widget nb, Widget child)
 
 	for (i=0; i<MGR_NumChildren(nb); i++) {
 		Widget c = MGR_Children(nb)[i];
+		if (NBC_ChildType(c) == XmPAGE || NBC_ChildType(c) == XmSTATUS_AREA) {
+			if (NBC_Active(c)) {
+				NBC_Active(c) = False;
 
-		if (NBC_ChildType(c) != XmPAGE)
-			continue;
+				/* Move out of sight */
+				_XmMoveObject(c, -10000, -10000);	/* FIX ME */
 
-		if (c == child && NBC_Active(c))
-			return;
-
-		if (NBC_Active(c)) {
-			NBC_Active(c) = False;
-
-			/* Move out of sight */
-			_XmMoveObject(c, -10000, -10000);	/* FIX ME */
-
-			DEBUGOUT(XdbDebug2(__FILE__, nb, c,
-				"Move child out of sight\n"));
+				DEBUGOUT(XdbDebug2(__FILE__, nb, c,
+					"Move child out of sight\n"));
+			}
 		}
 	}
 
-	NBC_Active(child) = True;
-	_XmMoveObject(child, NB_BindingWidth(nb) + NB_MarginWidth(nb), 0);
+	if (child) {
+		NBC_Active(child) = True;
+		_XmMoveObject(child, NB_BindingWidth(nb) + NB_MarginWidth(nb), 0);
+
+		_XmConfigureObject(child,
+			/* x */	NB_BindingWidth(nb) + NB_MarginWidth(nb),
+			/* y */	0,
+			/* w */	NB_PageWidth(nb),
+			/* h */	NB_PageHeight(nb),
+			/* bw */XtBorderWidth(child));
+
+		DEBUGOUT(XdbDebug2(__FILE__, nb, child, "Move child to %d %d sz %d %d\n",
+			/* x */	NB_BindingWidth(nb) + NB_MarginWidth(nb),
+			/* y */	0,
+			/* w */	NB_PageWidth(nb),
+			/* h */	NB_PageHeight(nb)));
+	}
+
+	if (status) {
+		NBC_Active(status) = True;
+		_XmMoveObject(status, NB_BindingWidth(nb) + NB_MarginWidth(nb), 0);
+
+		_XmConfigureObject(status,
+			/* x */	NB_BindingWidth(nb) + NB_MarginWidth(nb),
+			/* y */	NB_PageHeight(nb),
+			/* w */	NB_StatusWidth(nb),
+			/* h */	NB_StatusHeight(nb),
+			/* bw */XtBorderWidth(child));
+
+		DEBUGOUT(XdbDebug2(__FILE__, nb, status, "Move status to %d %d sz %d %d\n",
+			/* x */	NB_BindingWidth(nb) + NB_MarginWidth(nb),
+			/* y */	NB_PageHeight(nb),
+			/* w */	NB_StatusWidth(nb),
+			/* h */	NB_StatusHeight(nb)));
+	}
 
 	NB_ConstraintChild(nb) = child;
-
-	DEBUGOUT(XdbDebug2(__FILE__, nb, child, "Move child in sight\n"));
 }
 
 static void
@@ -528,11 +619,13 @@ _XmNotebookShowPage(Widget w, int page, int reason, XEvent *evp)
 {
 	int				i;
 	XmNotebookCallbackStruct	cbs;
-	Widget				cl = NULL;
+	Widget				cl = NULL, status = 0;
 	XmNavigatorTrait		t;
 	XmNavigatorDataRec		nd;
 	XmTwoDIntRec			ndmin, ndmax, ndval;
 
+	NB_CurrentPageNumber(w) = page;
+
 	/*
 	 * Be sure to pick up the last page with this page number.
 	 * You can read that in the M*tif specs, if you keep in mind
@@ -543,6 +636,16 @@ _XmNotebookShowPage(Widget w, int page, int reason, XEvent *evp)
 
 	    if (NBC_ChildType(c) == XmPAGE && NBC_PageNumber(c) == page) {
 		cl = c;
+#if 0
+		DEBUGOUT(XdbDebug2(__FILE__, w, cl, "page child found\n"));
+#endif
+	    }
+
+	    if (NBC_ChildType(c) == XmSTATUS_AREA && NBC_PageNumber(c) == page) {
+		status = c;
+#if 0
+		DEBUGOUT(XdbDebug2(__FILE__, w, status, "Status child found\n"));
+#endif
 	    }
 	}
 
@@ -563,21 +666,17 @@ _XmNotebookShowPage(Widget w, int page, int reason, XEvent *evp)
 	cbs.prev_page_number = -1;			/* FIX ME */
 	cbs.prev_page_widget = NULL;			/* FIX ME */
 
-	XtCallCallbackList(w, NB_PageChangeCallback(w), (XtPointer)&cbs);
+	if (reason)	/* FIX ME */
+		XtCallCallbackList(w, NB_PageChangeCallback(w), (XtPointer)&cbs);
 
 	cl = cbs.page_widget;
 
 	/*
-	 * Show the XmPAGE widget
+	 * Show the XmPAGE widget and the STATUS_AREA.
 	 */
 	DEBUGOUT(XdbDebug2(__FILE__, w, cl,
 		"_XmNotebookShowPage nr %d\n", page));
-	_XmNotebookShowChild(w, cl);
-
-	/*
-	 * Show the right XmSTATUS_AREA
-	 */
-		/* FIX ME */
+	_XmNotebookShowChild(w, cl, status);
 
 	/*
 	 * Use trait method to update XmPAGE_SCROLLER
@@ -609,7 +708,6 @@ Initialize(Widget request, Widget nw,
 		      "Notebook initialize: w/h %d %d\n",
 		      XtWidth(nw), XtHeight(nw)));
 
-    NB_DynamicLastPageNum(nw) = False;
     NB_ScrollerChild(nw) = NULL;
 
     XtSetArg(al[0], XmNnotebookChildType, XmMINOR_TAB);
@@ -636,6 +734,9 @@ Initialize(Widget request, Widget nw,
     NB_BackgroundGc(nw) = NULL;
     NB_SpiralPixmap(nw) = XmUNSPECIFIED_PIXMAP;
 
+    NB_PageWidth(nw) = 0;
+    NB_PageHeight(nw) = 0;
+
     _XmNBChangeGC(nw);
 
 #if 1
@@ -643,6 +744,48 @@ Initialize(Widget request, Widget nw,
     XtWidth(nw) = 100;
     XtHeight(nw) = 100;
 #endif
+
+    /*
+     * The spec says this should happen in Realize() but that's a really odd
+     *	and inconvenient place.
+     */
+    if (NB_Scroller(nw) == NULL) {
+	Arg	al[2];
+
+	NB_Scroller(nw) = XtCreateWidget("PageScroller",
+		xmSpinBoxWidgetClass, nw, NULL, 0);
+
+	/* FIX ME should we add a child to it ? */
+	XtSetArg(al[0], XmNspinBoxChildType, XmNUMERIC);
+	XtSetArg(al[1], XmNcolumns, 5);
+
+	(void) XtCreateManagedWidget("TextField",
+		xmTextFieldWidgetClass, NB_Scroller(nw), al, 2);
+
+	XtManageChild(NB_Scroller(nw));
+
+    } else {
+	XmNavigatorTrait		t;
+	XmNavigatorDataRec		nd;
+	XmTwoDIntRec			ndmin, ndmax, ndval;
+	/*
+	 * Update the scroller for first/last page
+	 * This also happens in SetValues().
+	 */
+	t = XmeTraitGet(XtClass(NB_Scroller(nw)), XmQTnavigator);
+	if (t) {
+		nd.valueMask = NavValue | NavMinimum | NavMaximum;
+		nd.value = &ndval;
+		nd.minimum = &ndmin;
+		nd.maximum = &ndmax;
+
+		ndval.x = 0;	/* FIX ME */
+		ndmin.x = NB_FirstPageNumber(nw);
+		ndmax.x = NB_LastPageNumber(nw);
+
+		t->setValue(NB_Scroller(nw), &nd, True);
+	}
+    }
 }
 
 static void
@@ -677,11 +820,42 @@ SetValues(Widget old, Widget request, Widget nw,
 		need_refresh = True;
 	}
 
+    if (NE(NB_CurrentPageNumber)) {
+	_XmNotebookShowPage(nw, NB_CurrentPageNumber(nw), 0, NULL);
+	need_refresh = True;
+    }
+
     if (need_layout) {
 	Layout(nw);
 	need_refresh = True;
     }
 
+    if (NE(NB_LastPageNumber)) {
+	NB_DynamicLastPageNum(nw) = False;
+    }
+
+    if (NE(NB_FirstPageNumber) || NE(NB_LastPageNumber)) {
+	XmNavigatorTrait		t;
+	XmNavigatorDataRec		nd;
+	XmTwoDIntRec			ndmin, ndmax, ndval;
+	/*
+	 * Use trait method to update XmPAGE_SCROLLER
+	 */
+	if (NB_Scroller(nw)) {
+	    t = XmeTraitGet(XtClass(NB_Scroller(nw)), XmQTnavigator);
+	    if (t) {
+		nd.valueMask = NavMinimum | NavMaximum;
+		nd.minimum = &ndmin;
+		nd.maximum = &ndmax;
+
+		ndmin.x = NB_FirstPageNumber(nw);
+		ndmax.x = NB_LastPageNumber(nw);
+
+		t->setValue(NB_Scroller(nw), &nd, True);
+	    }
+	}
+    }
+
     need_refresh = True;	/* FIX ME */
 
     return need_refresh;
@@ -689,30 +863,107 @@ SetValues(Widget old, Widget request, Widget nw,
 #undef	NE
 }
 
+static void
+draw_back_pages(Widget w)
+{
+#ifdef LARGE_PAGE
+	XDrawLine(XtDisplay(w), XtWindow(w), NB_BindingGc(w),
+		XtWidth(w) - 2 * NB_MarginWidth(w) - NB_MajorWidth(w) - NB_BackPageSize(w) / 2,
+		0,
+		XtWidth(w) - 2 * NB_MarginWidth(w) - NB_MajorWidth(w) - NB_BackPageSize(w) / 2,
+		NB_PageHeight(w) + NB_MarginHeight(w) + NB_StatusHeight(w) + NB_BackPageSize(w) / 2);
+	XDrawLine(XtDisplay(w), XtWindow(w), NB_BindingGc(w),
+		NB_BindingWidth(w) + NB_MarginWidth(w),
+		NB_PageHeight(w) + NB_MarginHeight(w) + NB_StatusHeight(w) + NB_BackPageSize(w) / 2,
+		XtWidth(w) - 2 * NB_MarginWidth(w) - NB_MajorWidth(w) - NB_BackPageSize(w) / 2,
+		NB_PageHeight(w) + NB_MarginHeight(w) + NB_StatusHeight(w) + NB_BackPageSize(w) / 2);
+#else
+	XDrawLine(XtDisplay(w), XtWindow(w), NB_BindingGc(w),
+		NB_BindingWidth(w) + 2 * NB_MarginWidth(w) + NB_PageWidth(w) + NB_BackPageSize(w) / 2,
+		0,
+		NB_BindingWidth(w) + 2 * NB_MarginWidth(w) + NB_PageWidth(w) + NB_BackPageSize(w) / 2,
+		NB_PageHeight(w) + NB_MarginHeight(w) + NB_StatusHeight(w) + NB_BackPageSize(w) / 2);
+	XDrawLine(XtDisplay(w), XtWindow(w), NB_BindingGc(w),
+		NB_BindingWidth(w) + NB_MarginWidth(w),
+		NB_PageHeight(w) + NB_MarginHeight(w) + NB_StatusHeight(w) + NB_BackPageSize(w) / 2,
+		NB_BindingWidth(w) + NB_MarginWidth(w) + NB_PageWidth(w) + NB_BackPageSize(w) / 2,
+		NB_PageHeight(w) + NB_MarginHeight(w) + NB_StatusHeight(w) + NB_BackPageSize(w) / 2);
+#endif
+}
+
+static void
+draw_binding_pixmap(Widget w, Pixmap p)
+{
+    /* FIX ME */
+
+    Dimension y;
+    /* draw SPIRAL binding */
+    for (y=0; y < XtHeight(w); y += spiral_height)
+	XCopyArea(XtDisplay(w), p, XtWindow(w),
+		NB_BindingGc(w),
+		0, 0,					/* source x,y */
+               NB_BindingWidth(w), spiral_height,      /* width,height */
+		0, y);					/* dest x,y */
+
+    return;
+}
+
+static void
+draw_binding(Widget w)
+{
+    Pixmap p;
+
+    switch(NB_BindingType(w))
+    {
+    case XmNONE: 
+        /* just return */
+        return;
+    case XmPIXMAP: 
+        p = NB_BindingPixmap(w);
+        break;
+    case XmSOLID: 
+        /* FIX ME: fill with solid color and return */
+        return;
+    case XmSPIRAL:
+        p = NB_SpiralPixmap(w);
+        break;
+    case XmPIXMAP_OVERLAP_ONLY: 
+        p = NB_BindingPixmap(w);
+        break;
+    default:
+        /* Silently fix things. */
+        NB_BindingType(w) = XmSPIRAL;
+        p = NB_SpiralPixmap(w);
+        break;
+    }
+
+    draw_binding_pixmap(w, p);
+}
+
+static void
+draw_frame(Widget w)
+{
+   /* FIX ME: draw frame */
+}
+
+    /* Now display other stuff */
 static void
 expose(Widget w, XEvent *event, Region region)
 {
-    Position	y;
+    if (! XtWindow(w))
+	return;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "Expose\n"));
 
     /* Display back page first */
-	/* FIX ME */
+    draw_back_pages(w);
 
-    /* Binding */
+    /* Display Binding */
+    draw_binding(w);
 
-	/* FIX ME don't know where to get the value of the spiral's
-	 * height from */
-#define	SPIRAL_HEIGHT_HACK	25
+    /* Display Frame */
+    draw_frame(w);
 
-    for (y=0; y < XtHeight(w); y += SPIRAL_HEIGHT_HACK)
-	XCopyArea(XtDisplay(w), NB_SpiralPixmap(w), XtWindow(w),
-		NB_BindingGc(w),
-		0, 0,					/* source x,y */
-		NB_BindingWidth(w), SPIRAL_HEIGHT_HACK,	/* width,height */
-		0, y);					/* dest x,y */
-
-    /* Now display other stuff */
     _XmRedisplayGadgets(w, event, region);
 }
 
@@ -720,6 +971,10 @@ static void
 resize(Widget w)
 {
 	DEBUGOUT(XdbDebug(__FILE__, w, "Resize\n"));
+	if (XtIsRealized(w))
+		XClearArea(XtDisplay(w), XtWindow(w), 0, 0, 0, 0, False);
+	Layout(w);
+	_XmNotebookShowPage(w, NB_CurrentPageNumber(w), 0, NULL);
 }
 
 static void
@@ -777,6 +1032,53 @@ _XmNBPageScrollerCallback(Widget w, XtPointer client, XtPointer call)
 		/* event */	cbp->event);
 }
 
+/*
+ * Major tabs should only show if they represent a page,
+ * which is not yet represented by a minor tab.
+ *
+ * The return value reflects whether relayout is needed.
+ */
+static Boolean
+MajorTabsActive(Widget n)
+{
+	XmNotebookWidget nb = (XmNotebookWidget)n;
+	int	i, j, nc = MGR_NumChildren(nb);
+	Boolean	change = False, na;
+
+	for (i=0; i<nc; i++) {
+	    Widget w = MGR_Children(nb)[i];
+	    Boolean	fm = False, fp = False;
+	    if (NBC_ChildType(w) == XmMAJOR_TAB) {
+		for (j=0; j<nc && !fm; j++) {
+		    Widget ww = MGR_Children(nb)[j];
+
+		    if (NBC_PageNumber(ww) != NBC_PageNumber(w))
+			continue;
+
+		    if (NBC_ChildType(ww) == XmPAGE) {
+			fp = True;
+		    } else if (NBC_ChildType(ww) == XmMINOR_TAB) {
+			fm = True;
+		    }
+		}
+
+		if (fm)
+		    na = False;
+		else if (fp)
+		    na = True;
+		else
+		    na = False;
+
+		if (NBC_Active(w) != na) {
+		    NBC_Active(w) = na;
+		    change = True;
+		}
+	    }
+	}
+
+	return change;
+}
+
 static void
 ConstraintInitialize(Widget request, Widget nw,
 		      ArgList args, Cardinal *num_args)
@@ -875,9 +1177,11 @@ ConstraintInitialize(Widget request, Widget nw,
 	} else {
 	    /* FIX ME */
 
-	    if (NB_FirstPageNumber(nb) > NBC_PageNumber(nw))
+	}
+	if (NB_FirstPageNumber(nb) > NBC_PageNumber(nw)) {
 		NB_FirstPageNumber(nb) = NBC_PageNumber(nw);
-	    if (NB_LastPageNumber(nb) < NBC_PageNumber(nw))
+	}
+	if (NB_DynamicLastPageNum(nb) && (NB_LastPageNumber(nb) < NBC_PageNumber(nw))) {
 		NB_LastPageNumber(nb) = NBC_PageNumber(nw);
 	}
 
@@ -886,6 +1190,11 @@ ConstraintInitialize(Widget request, Widget nw,
 	DEBUGOUT(XdbDebug2(__FILE__, nb, nw, "Child has page no %d\n",
 		NBC_PageNumber(nw)));
 
+	if (XtWidth(nw) > NB_PageWidth(nb))
+		XtWidth(nw) = NB_PageWidth(nb);
+	if (XtHeight(nw) > NB_PageHeight(nb))
+		XtHeight(nw) = NB_PageHeight(nb);
+	
     }
 
     if (NBC_ChildType(nw) == XmMAJOR_TAB || NBC_ChildType(nw) == XmMINOR_TAB ||
@@ -904,17 +1213,15 @@ ConstraintInitialize(Widget request, Widget nw,
 	    NBC_PageNumber(nw) = NB_LastAllocNum(nb);
     }
 
-    /*
-     * FIX ME there must be logic behind all this :
-     * which child gets displayed first ?
-     */
-#if 0
-    NBC_Active(nw) = False;
-#else
-    if (NBC_ChildType(nw) == XmPAGE)
-	_XmNotebookShowChild(nb, nw);	/* Last one !! */
-#endif
+    /* Move any child out of sight by default */
+    _XmMoveObject(nw, -10000, -10000);	/* FIX ME */
 
+    NBC_Active(nw) = False;
+
+    if (NBC_ChildType(nw) == XmPAGE && NBC_PageNumber(nw) == 1)
+	_XmNotebookShowChild(nb, nw, NULL);
+
+    (void)MajorTabsActive(nb);
     Layout(nb);
 }
 
@@ -936,6 +1243,9 @@ ConstraintSetValues(Widget current, Widget request, Widget nw,
 	if (need_layout)
 		Layout(nb);
 
+	if (MajorTabsActive(nb))
+		need_refresh = True;
+
 	return need_refresh;
 }
 
@@ -955,12 +1265,18 @@ PreferredSize(Widget nb, Widget instig, XtWidgetGeometry *ig,
 	int		i;
 	Dimension	wid = 0, ht = 0;
 	Dimension	vw, vh, hw, hh;
-	Dimension	verttabwid = 0, verttabht = 0,
-			hortabwid = 0, hortabht = 0;
-	Dimension	wawid = 0, waht = 0;
+	Dimension	verttabwid = 0, verttabht = 0,	/* vertical tab */
+			hortabwid = 0, hortabht = 0;	/* horizontal tab */
+	Dimension	sawid = 0, saht = 0,		/* status area */
+			pswid = 0, psht = 0;		/* page scroller */
 	int		nverttab = 0, nhortab = 0;
 	Widget		child;
 
+	/*
+	 * First loop over all children to figure out the maximum
+	 * geometry for all pages, all major/minor tabs, and all
+	 * status areas.
+	 */
 	for (i=0; i<MGR_NumChildren(nb); i++) {
 		child = MGR_Children(nb)[i];
 
@@ -1013,23 +1329,56 @@ PreferredSize(Widget nb, Widget instig, XtWidgetGeometry *ig,
 		    if (NB_Orientation(nb) == XmHORIZONTAL) {
 			nhortab++;
 
-			if (hortabht > XtHeight(child))
+			if (hortabht < XtHeight(child))
 			    hortabht = XtHeight(child);
-			if (hortabwid > XtWidth(child))
+			if (hortabwid < XtWidth(child))
 			    hortabwid = XtWidth(child);
 		    } else {
 			nverttab++;
 
-			if (verttabht > XtHeight(child))
+			if (verttabht < XtHeight(child))
 			    verttabht = XtHeight(child);
-			if (verttabwid > XtWidth(child))
+			if (verttabwid < XtWidth(child))
 			    verttabwid = XtWidth(child);
 		    }
+		} else if (NBC_ChildType(child) == XmSTATUS_AREA) {
+		    DEBUGOUT(XdbDebug2(__FILE__, nb, child,
+			"PreferredSize STATUS AREA size %d %d\n",
+			XtWidth(child), XtHeight(child)));
+
+			if (sawid < XtWidth(child))
+				sawid = XtWidth(child);
+			if (saht < XtHeight(child))
+				saht = XtHeight(child);
+		} else if (NBC_ChildType(child) == XmPAGE_SCROLLER) {
+		    DEBUGOUT(XdbDebug2(__FILE__, nb, child,
+			"PreferredSize PAGE SCROLLER size %d %d\n",
+			XtWidth(child), XtHeight(child)));
+
+			pswid = XtWidth(child);
+			psht = XtHeight(child);
+		} else {
+			/* This shouldn't happen */
+			_XmWarning(nb, "PreferredSize: child of unknown type 0x%X\n",
+				NBC_ChildType(child));
 		}
 	}
 
+	if (saht < psht)
+		saht = psht;
+
+	if (wid < sawid + NB_MarginWidth(nb) + pswid)
+		wid = sawid + NB_MarginWidth(nb) + pswid;
+
 	NB_PageWidth(nb) = wid;
 	NB_PageHeight(nb) = ht;
+	NB_StatusWidth(nb) = sawid;
+	NB_StatusHeight(nb) = saht;
+
+	DEBUGOUT(XdbDebug(__FILE__, nb,
+		"PreferredSize: %d major tabs (%d x %d), %d minor tabs (%d x %d)\n",
+		nverttab, verttabwid, verttabht,
+		nhortab, hortabwid, hortabht));
 
 	/* FIX ME add borders and such */
 	vw = vh = 0;
@@ -1067,7 +1416,10 @@ PreferredSize(Widget nb, Widget instig, XtWidgetGeometry *ig,
 	/* Page Scroller */
 	NB_ScrollerWidth(nb) = NB_ScrollerHeight(nb) = 0;
 	if (NB_Scroller(nb) && XtIsManaged(NB_Scroller(nb))) {
-		ht += XtHeight(NB_Scroller(nb)) + 2 * NB_MarginHeight(nb);
+		if (XtHeight(NB_Scroller(nb)) > psht)
+			psht = XtHeight(NB_Scroller(nb));
+		if (XtWidth(NB_Scroller(nb)) > pswid)
+			pswid = XtWidth(NB_Scroller(nb));
 
 		NB_ScrollerWidth(nb) = XtWidth(NB_Scroller(nb));
 		NB_ScrollerHeight(nb) = XtHeight(NB_Scroller(nb));
@@ -1077,8 +1429,18 @@ PreferredSize(Widget nb, Widget instig, XtWidgetGeometry *ig,
 			XtHeight(NB_Scroller(nb)), NB_MarginHeight(nb)));
 	}
 
-	/* Binding */
-	wid += NB_BindingWidth(nb) + NB_MarginWidth(nb);
+	if (saht > psht)
+		ht += saht + 2 * NB_MarginHeight(nb);
+	else if (psht)
+		ht += psht + 2 * NB_MarginHeight(nb);
+
+	/* Page Scroller + Status Area : width */
+	if (wid < sawid + pswid + NB_MarginWidth(nb))
+		wid = sawid + pswid + NB_MarginWidth(nb);
+
+	/* Binding, backpage size - this should be the last to influence width */
+	wid += NB_BindingWidth(nb) + NB_MarginWidth(nb) + NB_BackPageSize(nb);
+	ht += NB_BackPageSize(nb);
 
 	DEBUGOUT(XdbDebug(__FILE__, nb, "PreferredSize => %d %d\n", wid, ht));
 
@@ -1093,31 +1455,56 @@ Layout(Widget w /* ... */)
 	Position	majx, majy;
 	Position	minx, miny;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "Layout\n"));
+	DEBUGOUT(XdbDebug(__FILE__, w, "Layout (nb wid %d ht %d)\n",
+		XtWidth(w), XtHeight(w)));
 
+#ifdef LARGE_PAGE
+	if (XtIsRealized(w)) {
+	    NB_PageWidth(w) = XtWidth(w)
+		- 3 * NB_MarginWidth(w) - NB_BackPageSize(w)
+		- NB_MajorWidth(w);
+	    NB_PageHeight(w) = XtHeight(w)
+		- 3 * NB_MarginHeight(w) - NB_BackPageSize(w)
+		- NB_StatusHeight(w) - NB_MinorHeight(w);
+
+	    if (NB_Orientation(w) == XmHORIZONTAL)
+		NB_PageWidth(w) -= NB_BindingWidth(w);
+	    else
+		NB_PageHeight(w) -= NB_BindingWidth(w);
+
+	    DEBUGOUT(XdbDebug(__FILE__, w, "Layout: set page size to %d %d\n",
+		NB_PageWidth(w), NB_PageHeight(w)));
+	}
+
+	majx = XtWidth(w) - NB_MarginWidth(w) - NB_MajorWidth(w);
+#else
 	majx = NB_BindingWidth(w) + NB_MarginWidth(w) + NB_PageWidth(w)
-		+ NB_MarginWidth(w);
+		+ NB_MarginWidth(w) + NB_BackPageSize(w);
+#endif
 	majy = NB_MarginHeight(w);
 
 	minx = NB_BindingWidth(w) + NB_MarginWidth(w);
-	miny = NB_PageHeight(w);
+	miny = NB_PageHeight(w) + NB_MarginHeight(w) * 2 + NB_StatusHeight(w) + NB_BackPageSize(w);
 
 	for (i=0; i<MGR_NumChildren(w); i++) {
 	    Widget	child = MGR_Children(w)[i];
 	    if (! XtIsManaged(child))
 		continue;
 
-	    if (NBC_ChildType(child) == XmMAJOR_TAB) {
+	    if (NBC_ChildType(child) == XmMAJOR_TAB && NBC_Active(child)) {
 		_XmMoveObject(child, majx, majy);
 		majy += XtHeight(child) + NB_MajorSpacing(w);
 	    } else if (NBC_ChildType(child) == XmMINOR_TAB) {
+		DEBUGOUT(XdbDebug2(__FILE__, w, child, "Layout: XmMINOR_TAB\n"));
 		_XmMoveObject(child, minx, miny);
 		minx += XtWidth(child) + NB_MinorSpacing(w);
+/* HERE */
 	    } else if (NBC_ChildType(child) == XmPAGE_SCROLLER) {
 		_XmMoveObject(child,
-			NB_BindingWidth(w) + NB_MarginWidth(w),
+			NB_BindingWidth(w) + NB_MarginWidth(w) * 2 + NB_StatusWidth(w),
 			NB_PageHeight(w) + NB_MarginHeight(w));
 	    } else if (NBC_ChildType(child) == XmSTATUS_AREA) {
+		DEBUGOUT(XdbDebug2(__FILE__, w, child, "Layout: XmSTATUS_AREA\n"));
 	    } else {
 		/* XmPAGE treated elsewhere */
 	    }
@@ -1200,14 +1587,10 @@ GeometryManager(Widget w,
 static void
 ChangeManaged(Widget w)
 {
-#if 0
-	Dimension wid, hei;
-	Dimension reply_wid, reply_hei;
-	XtGeometryResult result;
-	XtWidgetGeometry req;
-#endif
-
+	/* This one usually determines initial size of the widget */
 	PreferredSize(w, NULL, NULL, &XtWidth(w), &XtHeight(w));
+
+	(void)MajorTabsActive(w);
 	Layout(w);
 
 	DEBUGOUT(XdbDebug(__FILE__, w, "ChangeManaged\n"));
@@ -1236,34 +1619,30 @@ _XmNBDefaultLastPageNumber(Widget w, int offset, XrmValue *val)
 	NB_LastPageNumber(w) = 1;
 }
 
-#if 0
-/* Bitmap for a simplistic spiral */
-#define spiral_width 25
-#define spiral_height 25
-static unsigned char spiral_bits[] = {
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0xe0, 0x00, 0x00, 0x00, 0x10, 0x01, 0x00, 0x00, 0x08, 0x02, 0x00, 0x00,
-   0xe8, 0xff, 0xff, 0x00, 0xe8, 0x00, 0x00, 0x00, 0xe8, 0x00, 0x00, 0x00,
-   0xe8, 0xff, 0xff, 0x00, 0x08, 0x02, 0x00, 0x00, 0x10, 0x01, 0x00, 0x00,
-   0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00};
-#else
-#define spiral_width 25
-#define spiral_height 25
-static unsigned char spiral_bits[] = {
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x00, 0x22, 0x00,
-   0x00, 0x00, 0x41, 0x00, 0xfc, 0xff, 0x5f, 0x00, 0x00, 0x00, 0x5c, 0x00,
-   0x00, 0x00, 0x5c, 0x00, 0xfc, 0xff, 0x5f, 0x00, 0x00, 0x00, 0x41, 0x00,
-   0x00, 0x00, 0x22, 0x00, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00};
-#endif
+static void
+_XmNBCreateSpiralPixmap(Widget w)
+{
+	_LtXpmAttributes   attrib; 
+        Pixmap          mask;
+
+	attrib.valuemask = 0;
+
+	_LtXpmCreatePixmapFromData(XtDisplay(w),
+                XRootWindowOfScreen(XtScreen(w)),
+                spiral,
+                &NB_SpiralPixmap(w),
+                &mask,
+                &attrib);
+
+	/* Fallback - e.g. if not getting any colors */
+	if (NB_SpiralPixmap(w) == None)
+	{
+	    NB_SpiralPixmap(w) = XCreatePixmapFromBitmapData(XtDisplay(w),
+		XtWindow(w), spiral_bits, spiral_width, spiral_height,
+		NB_BackPageForeground(w), NB_BackPageBackground(w),
+		DefaultDepthOfScreen(XtScreen(w)));
+	}
+}
 
 /*
  * Free the old Graphics Contexts (if any), and allocate new ones.
@@ -1308,12 +1687,10 @@ _XmNBChangeGC(Widget w)
 	NB_BindingGc(w) = XtAllocateGC(w, 0, mask, &values,
 		dynamic, dontcare);
 
+       /* FIX ME */
     /* Spiral - note this code is also in Realize(). */
-	if (XtWindow(w))
-	    NB_SpiralPixmap(w) = XCreatePixmapFromBitmapData(XtDisplay(w),
-		XtWindow(w), spiral_bits, spiral_width, spiral_height,
-		NB_BackPageForeground(w), NB_BackPageBackground(w),
-		DefaultDepthOfScreen(XtScreen(w)));
+       if (XtWindow(w))
+	   _XmNBCreateSpiralPixmap(w);
 }
 
 static void
@@ -1325,37 +1702,18 @@ Realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
     (*superclass->core_class.realize) (w, value_mask, attributes);
 #undef superclass
 
-    /*
-     * This is an odd place to do this, but that's what the spec says.
-     */
-    if (NB_Scroller(w) == NULL) {
-	Arg	al[1];
-
-	NB_Scroller(w) = XtCreateWidget("PageScroller",
-		xmSpinBoxWidgetClass, w, NULL, 0);
-
-	/* FIX ME should we add a child to it ? */
-	XtSetArg(al[0], XmNspinBoxChildType, XmNUMERIC);
-
-	(void) XtCreateManagedWidget("TextField",
-		xmTextFieldWidgetClass, NB_Scroller(w), al, 1);
-
-	XtManageChild(NB_Scroller(w));
-    }
-
     if (NB_SpiralPixmap(w) == XmUNSPECIFIED_PIXMAP)
-	    NB_SpiralPixmap(w) = XCreatePixmapFromBitmapData(XtDisplay(w),
-		XtWindow(w), spiral_bits, spiral_width, spiral_height,
-		NB_BackPageForeground(w), NB_BackPageBackground(w),
-		DefaultDepthOfScreen(XtScreen(w)));
+	   _XmNBCreateSpiralPixmap(w);
+
+    Layout(w);
+    _XmNotebookShowPage(w, 1, 0, NULL);
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "Realize (end)\n"));
 }
 
 static void
 _XmNBFrameBackgroundDefault(Widget w, int offset, XrmValue *val)
 {
-    double h, s, v;
-    XColor color, back;
-
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmNBFrameBackgroundDefault\n"));
 
     /*
@@ -1371,7 +1729,6 @@ _XmNBFrameBackgroundDefault(Widget w, int offset, XrmValue *val)
 static void
 _XmNBBackPageBackgroundDefault(Widget w, int offset, XrmValue *val)
 {
-    double h, s, v;
     XColor color, back;
 
     color.pixel = NB_FrameBackground(w);
