@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Xme.c,v 1.8 1998/10/03 06:30:04 gritton Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Xme.c,v 1.11 1999/11/11 13:43:31 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Xme.c,v 1.8 1998/10/03 06:30:04 gritton Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Xme.c,v 1.11 1999/11/11 13:43:31 rwscott Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,10 +38,10 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Xme.c,v
 #include <Xm/VendorSEP.h>
 #include <XmI/MessagesI.h>
 
-#ifndef __STDC__
-#include <varargs.h>
-#else
+#if STDC_HEADERS
 #include <stdarg.h>
+#else
+#include <varargs.h>
 #endif
 
 /* DragIcon */
@@ -92,7 +92,7 @@ XmeGetPixmapData(Screen *screen, Pixmap pixmap,
 
 /* ResConvert */
 void
-#ifdef __STDC__
+#if STDC_HEADERS
 XmeWarning(Widget w, char *fmt,...)
 {
     va_list arg_list;
@@ -248,12 +248,22 @@ XmeAddFocusChangeCallback(Widget w, XtCallbackProc cp, XtPointer udata)
 void
 XmeRemoveFocusChangeCallback(Widget w, XtCallbackProc cp, XtPointer udata)
 {
+    XmWidgetExtData extData;
+
     if(!XmIsVendorShell(w))
     {
         XmeWarning(w, _XmMsgProtocols_0000);
         return;
     }
-    XtRemoveCallback(w, XmNfocusMovedCallback, cp, udata);
+
+    extData = _XmGetWidgetExtData(w, XmSHELL_EXTENSION);
+    if (extData == NULL)
+    {
+        XmeWarning(w, "Can not find Vendor Shell Extention Data.");
+        return;
+    }
+
+    XtRemoveCallback(extData->widget, XmNfocusMovedCallback, cp, udata);
 }
 
 /* Visual */
@@ -319,3 +329,8 @@ XmeGetHomeDirName(void)
     return _XmOSGetHomeDirName();
 }
 
+XmFontList
+XmeGetDefaultRenderTable(Widget w, unsigned int fontListType)
+{
+	return NULL;
+}
