@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DialogS.c,v 1.16 1998/10/18 22:00:57 rwscott Exp $
+ * $Id: DialogS.c,v 1.22 1999/09/28 21:45:14 danny Exp $
  *
  * derived from Xt Vendor class.c 
  *
@@ -25,7 +25,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DialogS.c,v 1.16 1998/10/18 22:00:57 rwscott Exp $";
+static const char rcsid[] = "$Id: DialogS.c,v 1.22 1999/09/28 21:45:14 danny Exp $";
 
 #include <stdio.h>
 
@@ -52,7 +52,7 @@ static const char rcsid[] = "$Id: DialogS.c,v 1.16 1998/10/18 22:00:57 rwscott E
 
 #include <XmI/DebugUtil.h>
 
-static void class_initialize();
+static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass wclass);
 
@@ -82,12 +82,12 @@ static XmBaseClassExtRec _XmDialogSCoreClassExtRec = {
     /* set_values_posthook       */ XmInheritSetValuesPosthook,
     /* secondary_object_class    */ (WidgetClass)&xmDialogShellExtClassRec,
     /* secondary_object_create   */ XmInheritSecObjectCreate,
-    /* get_secondary_resources   */ NULL,
+    /* get_secondary_resources   */ XmInheritGetSecResData,
     /* fast_subclass             */ { 0 },
-    /* get_values_prehook        */ NULL,
-    /* get_values_posthook       */ NULL,
-    /* class_part_init_prehook   */ NULL,
-    /* class_part_init_posthook  */ NULL,
+    /* get_values_prehook        */ XmInheritGetValuesPrehook,
+    /* get_values_posthook       */ XmInheritGetValuesPosthook,
+    /* class_part_init_prehook   */ XmInheritClassPartInitPrehook,
+    /* class_part_init_posthook  */ XmInheritClassPartInitPosthook,
     /* ext_resources             */ NULL,
     /* compiled_ext_resources    */ NULL,
     /* num_ext_resources         */ 0,
@@ -248,7 +248,7 @@ XmDialogShellExtClassRec xmDialogShellExtClassRec = {
 WidgetClass xmDialogShellExtObjectClass = (WidgetClass)&xmDialogShellExtClassRec;
 
 static void 
-class_initialize()
+class_initialize(void)
 {
     _XmDialogSCoreClassExtRec.record_type = XmQmotif;
 }
@@ -387,9 +387,12 @@ change_managed(Widget w)
 	    XtMoveWidget(w, px, py);
 	    BB_DefaultPosition(child) = False;
 	}
+
+	if (XmIsBulletinBoard(child))
+	    _XmBulletinBoardMap(child);
+
 	if (CoreMappedWhenManaged(w))
 	{
-
 	    DEBUGOUT(XdbDebug(__FILE__, w, "XtPopup\n"));
 
 	    XtPopup(w, XtGrabNone);
@@ -413,6 +416,9 @@ change_managed(Widget w)
 	 * fixes certain cases (like nedit->open->ok, with no file selected).
 	 */
 	DEBUGOUT(XdbDebug(__FILE__, w, "XtUnmapWidget\n"));
+	if (XmIsBulletinBoard(child))
+		_XmBulletinBoardUnmap(child);
+
 	XtUnmapWidget(w);
     }
 }
@@ -463,6 +469,9 @@ delete_child(Widget w)
     Widget s = XtParent(w);
 
     DEBUGOUT(XdbDebug2(__FILE__, s, w, "delete_child\n"));
+    DEBUGOUT(XdbDebug2("RWS", s, w,"%s:delete_child(%d)\n",
+    	__FILE__, __LINE__
+    	));
 
     if (!XtIsRectObj(w))
     {
@@ -617,6 +626,7 @@ StructureNotifyHandler(Widget w, XtPointer closure,
 		XtUnmapWidget(w);
 	}
 	*/
+
 	break;
 
     case UnmapNotify:
