@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DragBS.c,v 1.10 1998/09/22 22:03:42 jon Exp $
+ * $Id: DragBS.c,v 1.18 2000/01/05 23:44:44 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DragBS.c,v 1.10 1998/09/22 22:03:42 jon Exp $";
+static const char rcsid[] = "$Id: DragBS.c,v 1.18 2000/01/05 23:44:44 rwscott Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -57,11 +57,6 @@ static const char rcsid[] = "$Id: DragBS.c,v 1.10 1998/09/22 22:03:42 jon Exp $"
  * have access to the X server.
  */
 
-/*
- * byte ordering
- */
-extern unsigned char _XmByteOrder();
-
 /*
  * these are used for protected sections
  */
@@ -152,6 +147,9 @@ create_drag_window(Display *display)
     XSetWindowAttributes attr;
 
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:create_drag_window(%d)\n",
+		      __FILE__, __LINE__));
+
     attr.override_redirect = True;
     attr.event_mask = PropertyChangeMask;
 
@@ -180,6 +178,9 @@ read_drag_window(Display *display)
     Window win = None;
 
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:read_drag_window(%d)\n",
+		      __FILE__, __LINE__));
+
     old_handler = XSetErrorHandler(DWError);
 
     error_flag = False;
@@ -204,9 +205,15 @@ read_drag_window(Display *display)
 
     if (error_flag)
     {
+	DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:read_drag_window(%d) - None\n",
+		      __FILE__, __LINE__));
+
 	return None;
     }
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:read_drag_window(%d) - %p\n",
+		      __FILE__, __LINE__, win));
+
     return win;
 }
 
@@ -218,6 +225,9 @@ write_drag_window(Display *display, Window *win)
 {
     Atom mw;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:write_drag_window(%d)\n",
+		      __FILE__, __LINE__));
+
     mw = XmInternAtom(display, _XA_MOTIF_DRAG_WINDOW, False);
 
     XChangeProperty(display, DefaultRootWindow(display), mw,
@@ -232,6 +242,9 @@ get_drag_window(Display *display)
 {
     Window win;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:get_drag_window(%d)\n",
+		      __FILE__, __LINE__));
+
     if (displayToDW == None)
     {
 	displayToDW = XUniqueContext();
@@ -240,9 +253,15 @@ get_drag_window(Display *display)
     if (XFindContext(display, DefaultRootWindow(display),
 		     displayToDW, (XPointer *)&win) == XCSUCCESS)
     {
+	DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:get_drag_window(%d) - %p\n",
+		      __FILE__, __LINE__, win));
+
 	return win;
     }
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:get_drag_window(%d) - None\n",
+		      __FILE__, __LINE__));
+
     return None;
 }
 
@@ -252,6 +271,9 @@ get_drag_window(Display *display)
 static void
 set_drag_window(Display *display, Window win)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:set_drag_window(%d)\n",
+		      __FILE__, __LINE__));
+
     if (displayToDW == None)
     {
 	displayToDW = XUniqueContext();
@@ -274,6 +296,9 @@ set_drag_window(Display *display, Window win)
 static int
 protect_handler(Display *display, XErrorEvent *event)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:protect_handler(%d)\n",
+		      __FILE__, __LINE__));
+
     if (event->error_code == BadWindow &&
 	event->resourceid == errorWindow &&
 	event->serial > firstProtectRequest)
@@ -297,6 +322,9 @@ protect_handler(Display *display, XErrorEvent *event)
 static void
 begin_protection(Display *display, Window win)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:begin_protection(%d)\n",
+		      __FILE__, __LINE__));
+
     bad_window = False;
 
     oldErrorHandler = XSetErrorHandler(protect_handler);
@@ -312,6 +340,9 @@ begin_protection(Display *display, Window win)
 static void
 end_protection(Display *display)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:end_protection(%d)\n",
+		      __FILE__, __LINE__));
+
     XSync(display, False);
 
     XSetErrorHandler(oldErrorHandler);
@@ -328,6 +359,11 @@ _XmReadDragBuffer(XmDndBufMgr bmgr, Boolean read_string,
 {
     int rsize, eaten;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmReadDragBuffer(%d) - read_string %s size %i\n",
+		      __FILE__, __LINE__,
+		      read_string ? "True" : "False",
+		      size));
+
     if (read_string)
     {
 	rsize = bmgr->name_size;
@@ -371,6 +407,11 @@ short
 _XmWriteDragBuffer(XmDndBufMgr bmgr, Boolean write_string,
 		   char *data, short size)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmWriteDragBuffer(%d) write_string %s size %i\n",
+		      __FILE__, __LINE__,
+		      write_string ? "True" : "False",
+		      size));
+
     if (write_string)
     {
 	if ((bmgr->name_size + size) > bmgr->name_avail)
@@ -439,6 +480,9 @@ read_atom_pairs(Display *display)
     unsigned long nitems, bafter;
     unsigned char *prop = NULL;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d)\n",
+		      __FILE__, __LINE__));
+
     pa = XmInternAtom(display, _XA_MOTIF_DRAG_ATOM_PAIRS, False);
 
     win = get_drag_window(display);
@@ -451,6 +495,9 @@ read_atom_pairs(Display *display)
 			   (unsigned char **)&pairs) == Success &&
 	nitems >= 8 && pairs != NULL)
     {
+	DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - got property\n",
+		      __FILE__, __LINE__));
+
 	gotit = True;
     }
 
@@ -474,6 +521,9 @@ read_atom_pairs(Display *display)
 	    XFree((char *)prop);
 	}
 
+	DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - did not get property\n",
+		      __FILE__, __LINE__));
+
 	return False;
     }
 
@@ -489,6 +539,10 @@ read_atom_pairs(Display *display)
 	SWAP4BYTES(pairs->data_size);
     }
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - %i pairs\n",
+		      __FILE__, __LINE__,
+		      pairs->num_pairs));
+
     if (pairs->num_pairs > 0)
     {
 	bmgr.atoms = (char *)pairs;
@@ -511,6 +565,10 @@ read_atom_pairs(Display *display)
 
 	    _XmReadDragBuffer(&bmgr, True, buf, pair.namelen);
 
+	    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - intern %s\n",
+		      __FILE__, __LINE__,
+		      buf));
+
 	    _XmInternAtomAndName(display, pair.atom, buf);
 	}
     }
@@ -538,6 +596,9 @@ write_atom_pairs(Display *display)
     int i;
     Window win;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:write_atom_pairs(%d)\n",
+		      __FILE__, __LINE__));
+
     pair_rec.byte_order = _XmByteOrder();
     pair_rec.protocol_version = DND_PROTOCOL_VERSION;
     pair_rec.num_pairs = XtNumber(motifAtoms);
@@ -611,6 +672,9 @@ get_atoms_table(Display *display)
 {
     XmDndAtomsTable tbl = NULL;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:get_atoms_table(%d)\n",
+		      __FILE__, __LINE__));
+
     if (displayToAtoms == (XContext)0)
     {
 	displayToAtoms = XUniqueContext();
@@ -633,6 +697,9 @@ set_atoms_table(Display *display, XmDndAtomsTable table)
 {
     XmDndAtomsTable atoms;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:set_atoms_table(%d)\n",
+		      __FILE__, __LINE__));
+
     if (displayToAtoms == (XContext)0)
 	displayToAtoms = XUniqueContext();
 
@@ -657,6 +724,9 @@ create_default_atoms_table(Display *display)
 {
     XmDndAtomsTable tbl;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:create_default_atoms_table(%d)\n",
+		      __FILE__, __LINE__));
+
     tbl = (XmDndAtomsTable) XtMalloc(sizeof(XmDndAtomsTableRec));
     tbl->num_entries = 1;
 
@@ -683,6 +753,9 @@ read_atoms_table(Display *display, XmDndAtomsTable tbl)
     XmDndAtomsTableEntryRec atom_ent;
     XmDndBufMgrRec bmgr;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:read_atoms_table(%d)\n",
+		      __FILE__, __LINE__));
+
     da = XmInternAtom(display, _XA_MOTIF_DRAG_ATOMS, False);
 
     win = get_drag_window(display);
@@ -784,6 +857,9 @@ write_atoms_table(Display *display, XmDndAtomsTable tbl)
     int i;
     Window win;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:write_atoms_table(%d)\n",
+		      __FILE__, __LINE__));
+
     if (tbl == NULL)
     {
 	_XmWarning((Widget)XmGetXmDisplay(display),
@@ -844,6 +920,9 @@ get_targets_table(Display *display)
 {
     XmDndTargetsTable targets = NULL;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:get_targets_table(%d)\n",
+		      __FILE__, __LINE__));
+
     if (displayToTargets == (XContext)0)
     {
 	displayToTargets = XUniqueContext();
@@ -852,9 +931,15 @@ get_targets_table(Display *display)
     if (XFindContext(display, DefaultRootWindow(display),
 		     displayToTargets, (XPointer *)&targets) != XCSUCCESS)
     {
+	DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:get_targets_table(%d) - NULL\n",
+		      __FILE__, __LINE__));
+
 	return NULL;
     }
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:get_targets_table(%d) - %p\n",
+		      __FILE__, __LINE__, targets));
+
     return targets;
 }
 
@@ -866,6 +951,9 @@ set_targets_table(Display *display, XmDndTargetsTable table)
 {
     XmDndTargetsTable targets;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:set_targets_table(%d) - %p\n",
+		      __FILE__, __LINE__, table));
+
     if (displayToTargets == (XContext)0)
     {
 	displayToTargets = XUniqueContext();
@@ -896,6 +984,9 @@ create_default_targets_table(Display *display)
 {
     XmDndTargetsTable tbl;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:create_default_targets_table(%d)\n",
+		      __FILE__, __LINE__));
+
     tbl = (XmDndTargetsTable)XtMalloc(sizeof(XmDndTargetsTableRec));
 
     tbl->num_entries = 2;
@@ -926,6 +1017,9 @@ read_targets_table(Display *display, XmDndTargetsTable tbl)
     XmDndBufMgrRec bmgr;
     CARD16 nents;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:read_targets_table(%d)\n",
+		      __FILE__, __LINE__));
+
     ta = XmInternAtom(display, _XA_MOTIF_DRAG_TARGETS, False);
 
     win = get_drag_window(display);
@@ -956,6 +1050,9 @@ read_targets_table(Display *display, XmDndTargetsTable tbl)
 	    XFree((XPointer)targets);
 	}
 
+	DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:read_targets_table(%d) - False\n",
+		      __FILE__, __LINE__));
+
 	return False;
     }
 
@@ -983,6 +1080,7 @@ read_targets_table(Display *display, XmDndTargetsTable tbl)
 
     if (tbl->num_entries < targets->num_target_lists)
     {
+	tbl->num_entries = targets->num_target_lists;
 	tbl->entries =
 	    (XmDndTargetsTableEntry)XtRealloc((char *)tbl->entries,
 					      targets->num_target_lists *
@@ -1024,6 +1122,9 @@ read_targets_table(Display *display, XmDndTargetsTable tbl)
 	}
     }
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:read_targets_table(%d) - True %p\n",
+		      __FILE__, __LINE__, tbl));
+
     return True;
 }
 
@@ -1038,6 +1139,9 @@ write_targets_table(Display *display, XmDndTargetsTable tbl)
     Window win;
     CARD16 nents;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:write_targets_table(%d)\n",
+		      __FILE__, __LINE__));
+
     if (tbl == NULL)
     {
 	_XmWarning((Widget)XmGetXmDisplay(display),
@@ -1104,6 +1208,9 @@ _XmInitAtomPairs(Display *display)
     char *dstr;
     Display *d;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmInitAtomPairs(%d)\n",
+		      __FILE__, __LINE__));
+
     if ((win = read_drag_window(display)) == None)
     {
 	dstr = XDisplayString(display);
@@ -1161,6 +1268,9 @@ _XmInitTargetsTable(Display *display)
     XmDndAtomsTable atoms;
     XmDndTargetsTable targets;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmInitTargetsTable(%d)\n",
+		      __FILE__, __LINE__));
+
     if ((win = read_drag_window(display)) == None)
     {
 
@@ -1241,6 +1351,9 @@ _XmIndexToTargets(Widget shell, Cardinal t_index, Atom **targetsRtn)
     Display *dpy = XtDisplay(shell);
     XmDndTargetsTable targets;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmIndexToTargets(%d)\n",
+		      __FILE__, __LINE__));
+
     if ((targets = get_targets_table(dpy)) == NULL)
     {
 	_XmInitTargetsTable(dpy);
@@ -1260,7 +1373,8 @@ _XmIndexToTargets(Widget shell, Cardinal t_index, Atom **targetsRtn)
 
     if (t_index >= targets->num_entries)
     {
-	_XmWarning(shell, "Index out of range to _XmIndexToTargets.");
+	_XmWarning(shell, "Index out of range to _XmIndexToTargets. %i %i",
+		t_index, targets->num_entries);
 
 	*targetsRtn = NULL;
 
@@ -1280,6 +1394,9 @@ acompare(XmConst void *a, XmConst void *b)
 {
     Atom *pa = (Atom *)a, *pb = (Atom *)b;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:acompare(%d)\n",
+		      __FILE__, __LINE__));
+
     return *pa - *pb;
 }
 
@@ -1295,7 +1412,14 @@ _XmTargetsToIndex(Widget shell, Atom *tlist, Cardinal numTargets)
     int i;
     Atom *atoms;
 
-#ifdef NS45PR1
+    DEBUGOUT(XdbDebug(__FILE__, shell,
+		      "%s:_XmTargetsToIndex(%d) - %p 0x%x targets %p\n", 
+		      __FILE__, __LINE__,
+		      tlist,
+		      numTargets,
+		      tlist ? tlist[0] : 0
+		      ));
+#if 0
     /* For Netscape 4.5 PR1 binary compatibility testing.
        NS seems to be calling this routine with an invalid (huge) number
        of numTargets.  I just bail out of the whole routine for testing.
@@ -1309,8 +1433,35 @@ _XmTargetsToIndex(Widget shell, Atom *tlist, Cardinal numTargets)
 
 	targets = get_targets_table(dpy);
     }
+    DEBUGOUT(XdbDebug(__FILE__, shell,
+		      "%s:_XmTargetsToIndex(%d) - targets->num_entries %i\n", 
+		      __FILE__, __LINE__,
+		      targets->num_entries
+		      ));
+
+#if 1
+    /* 28 Jul 1999
+       Navigator is still calling this with a huge value for numTargets.
+       So for now just do a check for this and bail.....
+     */
+    for (i = 0; i < targets->num_entries; i++)
+    {
+	if (numTargets == targets->entries[i].num_targets)
+	{
+	    break;
+	}
+    }
+    if ((int)numTargets > 1000)
+    {
+    	return(0);
+    }
+#endif
 
     atoms = (Atom *)XtMalloc(numTargets * sizeof(Atom));
+    for	(i = 0; i < numTargets; i++)
+    {
+    	atoms[i] = tlist[i];
+    }
 
     qsort(atoms, numTargets, sizeof(Atom), acompare);
 
@@ -1387,6 +1538,9 @@ _XmAllocMotifAtom(Widget shell, Time time)
     int i;
     char buf[32];
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmAllocMotifAtom(%d)\n",
+		      __FILE__, __LINE__));
+
     if ((atoms = get_atoms_table(dpy)) == NULL)
     {
 	_XmInitTargetsTable(dpy);
@@ -1457,6 +1611,9 @@ _XmFreeMotifAtom(Widget shell, Atom atom)
     Display *dpy = XtDisplay(shell);
     int i;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmFreeMotifAtom(%d)\n",
+		      __FILE__, __LINE__));
+
     if (atom == None)
     {
 	return;
@@ -1511,6 +1668,9 @@ _XmDestroyMotifWindow(Display *dpy)
     Atom dw;
     Window win;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmDestroyMotifWindow(%d)\n",
+		      __FILE__, __LINE__));
+
     win = read_drag_window(dpy);
     if (win == None)
     {
@@ -1537,6 +1697,9 @@ _XmGetDragProxyWindow(Display *display)
     unsigned long nitems, bafter;
     unsigned char *prop = NULL;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmGetDragProxyWindow(%d)\n",
+		      __FILE__, __LINE__));
+
     if ((win = read_drag_window(display)) == None)
     {
 	return None;
@@ -1557,6 +1720,9 @@ _XmGetDragProxyWindow(Display *display)
 	XFree((XPointer)prop);
     }
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmGetDragProxyWindow(%d) - %p\n",
+		      __FILE__, __LINE__, win));
+
     return win;
 }
 
@@ -1578,6 +1744,9 @@ _XmGetDragReceiverInfo(Display *display, Window win, XmDragReceiverInfo ri)
     unsigned long bafter, length;
     Window root;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmGetDragReceiverInfo(%d)\n",
+		      __FILE__, __LINE__));
+
     dri = XmInternAtom(display, _XA_MOTIF_DRAG_RECEIVER_INFO, False);
 
     if (XGetWindowProperty(display, win, dri, 0L, 100000L, False,
@@ -1633,11 +1802,17 @@ _XmGetDragReceiverInfo(Display *display, Window win, XmDragReceiverInfo ri)
 void
 _XmSetDragReceiverInfo(Widget w, Widget widget)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmSetDragReceiverInfo(%d)\n",
+		      __FILE__, __LINE__));
+
 }
 
 void
 _XmClearDragReceiverInfo(Widget shell)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmClearDragReceiverInfo(%d)\n",
+		      __FILE__, __LINE__));
+
 }
 
 /*
@@ -1646,6 +1821,9 @@ _XmClearDragReceiverInfo(Widget shell)
 void
 _XmFreeDragReceiverInfo(XmShellDropSiteInfo di)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmFreeDragReceiverInfo(%d)\n",
+		      __FILE__, __LINE__));
+
     if (di)
     {
 	if (di->info.atoms != NULL)
@@ -1662,6 +1840,11 @@ _XmByteOrder(void)
 {
     static unsigned char byte_order = 0;
 
+    /*
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmByteOrder(%d)\n",
+		      __FILE__, __LINE__));
+    */
+
     if (!byte_order)
     {
 	unsigned int endian = 1;
@@ -1672,14 +1855,3 @@ _XmByteOrder(void)
     return byte_order;
 }
 
-#ifdef NS45PR1
-/* netscape 4.5PR1 wants these; dunno what they're supposed to do -jac */
-int _XmDSIAddChild(){return NULL;}
-int _XmDSIRemoveChild(){return NULL;}
-int _XmDSIDestroy(){return NULL;}
-int _XmDSIGetBorderWidth(){return NULL;}
-int _XmDSIReplaceChild(){return NULL;}
-int _XmDSIGetChildPosition(){return NULL;}
-int _XmDSISwapChildren(){return NULL;}
-#endif
-
