@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DragC.c,v 1.19 1998/09/12 15:05:22 rwscott Exp $
+ * $Id: DragC.c,v 1.25 2000/01/05 23:44:44 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -40,7 +40,7 @@
  * provided "as is" without express or implied warranty.
  */
 
-static const char rcsid[] = "$Id: DragC.c,v 1.19 1998/09/12 15:05:22 rwscott Exp $";
+static const char rcsid[] = "$Id: DragC.c,v 1.25 2000/01/05 23:44:44 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -59,7 +59,7 @@ static const char rcsid[] = "$Id: DragC.c,v 1.19 1998/09/12 15:05:22 rwscott Exp
 
 /* Forward Declarations */
 
-static void class_initialize();
+static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass w_class);
 
@@ -77,17 +77,10 @@ static void _XmDCStartDrag(XmDragContext dc,
 
 static void _XmDCCancelDrag(XmDragContext dc);
 
-unsigned char _XmByteOrder(void);
-
 static void internal_notify(Widget w, XtPointer cd, XtPointer cbs);
 
 static void external_notify(Widget w, XtPointer cd, XtPointer cbs);
 
-Boolean _XmGetDragReceiverInfo(Display *display, Window win,
-			       XmDragReceiverInfo ri);
-
-void _XmFreeDragReceiverInfo(XmShellDropSiteInfo di);
-
 /*
  * Resources for the dragContext class
  */
@@ -360,8 +353,10 @@ XmDragContextClassRec xmDragContextClassRec = {
 WidgetClass xmDragContextClass = (WidgetClass)&xmDragContextClassRec;
 
 static void
-class_initialize()
+class_initialize(void)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:class_initialize(%d)\n",
+    	__FILE__, __LINE__));
 #if 0
     _XmDragCCoreClassExtRec.record_type = XmQmotif;
 #endif
@@ -372,6 +367,8 @@ class_part_initialize(WidgetClass widget_class)
 {
     WidgetClass sc;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:class_part_initialize(%d)\n",
+    	__FILE__, __LINE__));
     _XmFastSubclassInit(widget_class, XmDRAG_CONTEXT_BIT);
 
     sc = widget_class->core_class.superclass;
@@ -395,7 +392,17 @@ initialize(Widget request, Widget new_w,
     XmDropSiteManagerObject dsm;
     Arg argl[2];
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "Initialize\n"));
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:initialize(%d) - %i args\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
+		      *num_args,
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     DC_InDropSite(new_w) = False;
     DC_DragTimerId(new_w) = None;
@@ -457,7 +464,8 @@ initialize(Widget request, Widget new_w,
 static void
 destroy(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "Destroy\n"));
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:destroy(%d)\n",
+    	__FILE__, __LINE__));
 
     if (DC_ExportTargets(w))
     {
@@ -483,7 +491,20 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Atom *exports;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "SetValues\n"));
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:set_values(%d) - %i args\n"
+		      "\t    old X %5i Y %5i W %5i H %5i\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
+		      *num_args,
+		      XtX(old), XtY(old),
+		      XtWidth(old), XtHeight(old),
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     if (DC_ExportTargets(old) != DC_ExportTargets(new_w))
     {
@@ -512,6 +533,9 @@ send_drag_message(Widget dc, Window win, XmDragDropCallbackStruct * cbs,
     XmDndMessage *dnd_message = (XmDndMessage *) & cmsg.data.b[0];
     Atom msg;
 
+    DEBUGOUT(XdbDebug(__FILE__, dc, "%s:send_drag_message(%d)\n",
+    	__FILE__, __LINE__));
+
     msg = XmInternAtom(XtDisplay(dc), _XA_MOTIF_DRAG_AND_DROP_MESSAGE, False);
 
     cmsg.type = ClientMessage;
@@ -576,6 +600,9 @@ receive_drag_message(Widget dc, XEvent *event, XmDragDropCallbackStruct * cbs)
     XmDndMessage *dnd_message = (XmDndMessage *) & cm->data.b[0];
     Atom msg;
 
+    DEBUGOUT(XdbDebug(__FILE__, dc, "%s:receive_drag_message(%d)\n",
+    	__FILE__, __LINE__));
+
     msg = XmInternAtom(XtDisplay(dc), _XA_MOTIF_DRAG_AND_DROP_MESSAGE, False);
 
     if (cm->message_type != msg)
@@ -689,6 +716,9 @@ maybe_send_drag_message(Widget dc, Window win, int reason)
     XmDragTopLevelClientDataStruct tld;
     XmDragMotionClientDataStruct md;
 
+    DEBUGOUT(XdbDebug(__FILE__, dc, "%s:maybe_send_drag_message(%d)\n",
+    	__FILE__, __LINE__));
+
     if (DC_ActiveProtocolStyle(dc) == XmDRAG_DROP_ONLY)
     {
 	if (reason == XmTOP_LEVEL_ENTER)
@@ -821,6 +851,9 @@ invoke_initiator_callback(Widget dc, int cr_reason)
     XmDragDropCallbackStruct cbs;
     XtCallbackList cb = NULL;
 
+    DEBUGOUT(XdbDebug(__FILE__, dc, "%s:invoke_initiator_callback(%d)\n",
+    	__FILE__, __LINE__));
+
     switch (cr_reason)
     {
     case XmCR_TOP_LEVEL_ENTER:
@@ -979,6 +1012,9 @@ invoke_receiver_callback(Widget dc, XmDragDropCallbackStruct * cbs)
     XtCallbackList cb = NULL;
     int state;
 
+    DEBUGOUT(XdbDebug(__FILE__, dc, "%s:invoke_receiver_callback(%d)\n",
+    	__FILE__, __LINE__));
+
     switch (cbs->any.reason)
     {
     case XmCR_DROP_SITE_ENTER:
@@ -1099,6 +1135,9 @@ internal_notify(Widget w, XtPointer cd, XtPointer cbs)
     XmDragDropCallbackStruct *cb = (XmDragDropCallbackStruct *) cbs;
     Widget dc = (Widget)cd;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:internal_notify(%d)\n",
+    	__FILE__, __LINE__));
+
     invoke_receiver_callback(dc, cb);
 }
 
@@ -1108,6 +1147,9 @@ external_notify(Widget w, XtPointer cd, XtPointer cbs)
     XmDragDropCallbackStruct *cb = (XmDragDropCallbackStruct *) cbs;
     Widget dc = (Widget)cd;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:external_notify(%d)\n",
+    	__FILE__, __LINE__));
+
     switch (cb->any.reason)
     {
     case XmCR_DROP_SITE_ENTER:
@@ -1132,7 +1174,8 @@ external_msg_handler(Widget widget, XtPointer client_data, XEvent *event,
     Widget dc = (Widget)client_data;
     XmDragDropCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, widget, "Drag shell focus change\n"));
+    DEBUGOUT(XdbDebug(__FILE__, widget, "%s:external_msg_handler(%d)\n",
+    	__FILE__, __LINE__));
 
     if (dc != NULL && event->type != ClientMessage)
     {
@@ -1168,6 +1211,9 @@ drop_convert_incr_callback(Widget w, Atom *selection, Atom *target,
     Atom ok, fail, drop;
     Widget dc;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:drop_convert_incr_callback(%d)\n",
+    	__FILE__, __LINE__));
+
     sev = XtGetSelectionRequest(w, *selection, NULL);
 
     if ((dc = _XmGetDragContextFromHandle(w, *selection)) == NULL)
@@ -1314,6 +1360,9 @@ drop_lose_incr_callback(Widget w, Atom *selection, XtPointer cd)
 {
     Widget dc;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:drop_lose_incr_callback(%d)\n",
+    	__FILE__, __LINE__));
+
     if ((dc = _XmGetDragContextFromHandle(w, *selection)) == NULL)
     {
 	_XmWarning(w, "bad selection\n");
@@ -1330,6 +1379,9 @@ drop_lose_callback(Widget w, Atom *selection)
 {
     Widget dc;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:drop_lose_callback(%d)\n",
+    	__FILE__, __LINE__));
+
     if ((dc = _XmGetDragContextFromHandle(w, *selection)) == NULL)
     {
 	_XmWarning(w, "bad selection\n");
@@ -1346,6 +1398,9 @@ drop_timeout(XtPointer data, XtIntervalId *id)
 {
     Widget dc = (Widget)data;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:drop_timeout(%d)\n",
+    	__FILE__, __LINE__));
+
     DC_DragCompletionStatus(dc) = XmDROP_CANCEL;
 
     invoke_initiator_callback(dc, XmCR_DROP_START);
@@ -1384,6 +1439,9 @@ create_root_info(Widget w)
     XmDragReceiverInfo ri;
     Display *disp = XtDisplay(w);
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:create_root_info(%d)\n",
+    	__FILE__, __LINE__));
+
     if (DC_NumReceiverInfos(w) == 0)
     {
 	ri = _XmAllocReceiverInfo((XmDragContext)w);
@@ -1429,6 +1487,9 @@ new_root(Widget dc, Window root)
     Cursor cursor;
     Arg args[10];
 
+    DEBUGOUT(XdbDebug(__FILE__, dc, "%s:new_root(%d)\n",
+    	__FILE__, __LINE__));
+
     dos = DC_CurDragOver(dc);
 
     for (i = 0; i < ScreenCount(XtDisplay(dc)); i++)
@@ -1507,12 +1568,16 @@ new_root(Widget dc, Window root)
 /*
  * write the initiator info on the source window
  */
-void
+static void
 write_initiator(Widget dc)
 {
     XmDndSourceProp src_prop;
     Atom init;
 
+    DEBUGOUT(XdbDebug(__FILE__, dc, "%s:write_initiator(%d)\n",
+    	__FILE__, __LINE__
+    	));
+
     src_prop.byte_order = _XmByteOrder();
     src_prop.protocol_version = DND_PROTOCOL_VERSION;
     src_prop.target_index = _XmTargetsToIndex(dc,
@@ -1533,7 +1598,7 @@ write_initiator(Widget dc)
  * child that has the WM_STATE property set -- we need to skip over the
  * frame and the matte.
  */
-Window
+static Window
 find_shell_child(Display *display, Window win)
 {
     Atom type = 0;
@@ -1544,6 +1609,9 @@ find_shell_child(Display *display, Window win)
     Window root, parent, *children;
     Atom state;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:find_shell_child(%d)\n",
+    	__FILE__, __LINE__));
+
     state = XmInternAtom(display, _XA_WM_STATE, True);
 
     XGetWindowProperty(display, win, state, 0L, 0L, False,
@@ -1566,9 +1634,9 @@ find_shell_child(Display *display, Window win)
 	return None;
     }
 
-    while (--nchildren >= 0)
+    while (nchildren > 0)
     {
-	win = find_shell_child(display, children[nchildren]);
+	win = find_shell_child(display, children[--nchildren]);
 
 	if (win != None)
 	{
@@ -1598,6 +1666,9 @@ find_receiver_info(Widget dc, Window win)
     int i;
     XmDragReceiverInfo ri;
 
+    DEBUGOUT(XdbDebug(__FILE__, dc, "%s:find_receiver_info(%d)\n",
+    	__FILE__, __LINE__));
+
     /* maybe there's a match */
     for (i = 0; i < DC_NumReceiverInfos(dc); i++)
     {
@@ -1633,6 +1704,9 @@ get_current_receiver(Widget dc, Window root, Window win)
     Position deltax, deltay;
     int rootx, rooty;
 
+    DEBUGOUT(XdbDebug(__FILE__, dc, "%s:get_current_receiver(%d)\n",
+    	__FILE__, __LINE__));
+
     DC_CrossingTime(dc) = DC_LastChangeTime(dc);
 
     last_style = DC_ActiveProtocolStyle(dc);
@@ -1736,6 +1810,9 @@ drag_main_loop(XtPointer data, XtIntervalId *id)
     XtAppContext app;
     XEvent event;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:drag_main_loop(%d)\n",
+    	__FILE__, __LINE__));
+
     if (dc == NULL)
 	return;
 
@@ -1743,6 +1820,9 @@ drag_main_loop(XtPointer data, XtIntervalId *id)
 
     fw = XmGetFocusWidget(DC_SrcShell(dc));
 
+	/* T. Straumann: avoid dereferencing NULL fw */
+	if (fw)
+	{
     if (_XmGetFocusPolicy(DC_SrcShell(dc)) != XmEXPLICIT)
     {
 	WidgetClass wc = XtClass(fw);
@@ -1760,6 +1840,7 @@ drag_main_loop(XtPointer data, XtIntervalId *id)
     {
 	XtSetKeyboardFocus(DC_SrcShell(dc), NULL);
     }
+	}
 
     write_initiator(dc);
 
@@ -1828,6 +1909,9 @@ add_motion(Widget w, XmMotionBuffer * mb, XEvent *event)
 {
     XmDragReceiverInfo ri;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:add_motion(%d)\n",
+    	__FILE__, __LINE__));
+
     ri = DC_CurrReceiverInfo(w);
 
     if (!ri)
@@ -1880,6 +1964,9 @@ motion_message(Widget w, Window root, Window win)
 {
     Boolean left_drop = False;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:motion_message(%d)\n",
+    	__FILE__, __LINE__));
+
     if (root != DC_CurrWmRoot(w) ||
 	(DC_CurrReceiverInfo(w)->frame != win))
     {
@@ -1952,6 +2039,9 @@ maybe_operation_changed(Widget w, unsigned int state)
 {
     unsigned char ops, op;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:maybe_operation_changed(%d)\n",
+    	__FILE__, __LINE__));
+
     DC_LastEventState(w) = state;
 
     ops = DC_Operations(w);
@@ -2023,6 +2113,9 @@ motion_hysteresis(Widget w, XmMotionBuffer * mb)
     int i, grain_size, box_count, box_offset;
     Window sw = None;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:motion_hysteresis(%d)\n",
+    	__FILE__, __LINE__));
+
     grain_size = mb->num_motions / OPTIMAL_GRANULARITY;
 
     if (grain_size == 0)
@@ -2083,6 +2176,9 @@ drag_finish(Widget w, XEvent *event)
     Arg al[5];
     XtAppContext app;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:drag_finish(%d)\n",
+    	__FILE__, __LINE__));
+
     /* NOTE: This is how we stop the main loop */
     Display_ActiveDC(disp) = NULL;
 
@@ -2408,6 +2504,9 @@ _XmDCMotion(Widget w,
     Boolean done = False;
     XmMotionBuffer mb;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmDCMotion(%d)\n",
+    	__FILE__, __LINE__));
+
     if (event->type == MotionNotify)
     {
 	DEBUGOUT(XdbDebug(__FILE__, w, "_XmDCMotion: Motion\n"));
@@ -2663,3 +2762,77 @@ _XmAllocReceiverInfo(XmDragContext dc)
 
     return ri;
 }
+
+Widget
+XmDragStart(Widget widget,
+	    XEvent *event,
+	    ArgList arglist,
+	    Cardinal argcount)
+{
+    Widget dc = NULL;
+    Widget disp = XmGetXmDisplay(XtDisplay(widget));
+    Arg arg[1];
+    ArgList merged;
+
+    DEBUGOUT(XdbDebug(__FILE__, widget, "XmDragStart()\n"));
+
+    if (Display_DragInitiatorProtocolStyle(disp) != XmDRAG_NONE)
+    {
+	/* appears in a dump of DragC's resources, but isn't documented. */
+	XtSetArg(arg[0], XmNsourceWidget, widget);
+
+	if (arglist)
+	{
+	    merged = XtMergeArgLists(arglist, argcount, arg, 1);
+	}
+	else
+	{
+	    merged = arg;
+	}
+
+	dc = XtCreateWidget("drag_context",
+			    Display_DragContextClass(disp), disp,
+			    merged, argcount + 1);
+
+	if (arglist)
+	{
+	    XtFree((char *)merged);
+	}
+
+	_XmDragStart((XmDragContext)dc, widget, event);
+    }
+
+    return dc;
+}
+
+void
+XmDragCancel(Widget dragcontext)
+{
+    DEBUGOUT(XdbDebug(__FILE__, dragcontext, "XmDragCancel()\n"));
+
+    _XmDragCancel((XmDragContext)dragcontext);
+}
+
+Boolean
+XmTargetsAreCompatible(Display *dpy,
+		       Atom *exportTargets, Cardinal numExportTargets,
+		       Atom *importTargets, Cardinal numImportTargets)
+{
+    int i, j;
+
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "XmTargetsAreCompatible()\n"));
+
+    /* if ANY import targets matches any export target, they're compat */
+    for (i = 0; i < numExportTargets; i++)
+    {
+	for (j = 0; j < numImportTargets; j++)
+	{
+	    if (exportTargets[i] == importTargets[j])
+	    {
+		return True;
+	    }
+	}
+    }
+
+    return False;
+}
