@@ -1,6 +1,6 @@
 /** -*- Mode: C; tab-width: 8; indent-tabs-mode: nil -*-
  *
- * $Id: DropSMgr.c,v 1.17 1998/09/12 15:05:25 rwscott Exp $
+ * $Id: DropSMgr.c,v 1.25 2000/01/05 23:44:46 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DropSMgr.c,v 1.17 1998/09/12 15:05:25 rwscott Exp $";
+static const char rcsid[] = "$Id: DropSMgr.c,v 1.25 2000/01/05 23:44:46 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -45,10 +45,6 @@ static const char rcsid[] = "$Id: DropSMgr.c,v 1.17 1998/09/12 15:05:25 rwscott
 
 /* Forward Declarations */
 
-extern void _XmSetDragReceiverInfo(Widget w, Widget widget);
-
-extern void _XmClearDragReceiverInfo(Widget shell);
-
 static void __XmDSMCreateInfo(XmDropSiteManagerObject dsm,
 			      Widget widget,
 			      ArgList args,
@@ -156,79 +152,6 @@ static XtResource resources[] =
 };
 #undef Offset
 
-#define Offset(f)	XtOffsetOf(XmDropSiteInfoRec, f)
-XtResource _XmDSResources[] =
-{
-    {
-	XmNanimationMask, XmCAnimationMask, XmRAnimationMask,
-	sizeof(Pixmap), Offset(animationMask),
-	XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXMAP
-    },
-    {
-	XmNanimationPixmap, XmCAnimationPixmap, XmRAnimationPixmap,
-	sizeof(Pixmap), Offset(animationPixmap),
-	XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXMAP
-    },
-    {
-	XmNanimationPixmapDepth, XmCAnimationPixmapDepth, XmRCardinal,
-	sizeof(Cardinal), Offset(animationPixmapDepth),
-	XmRImmediate, (XtPointer)0
-    },
-    {
-	XmNanimationStyle, XmCAnimationStyle, XmRAnimationStyle,
-	sizeof(unsigned char), Offset(animationStyle),
-	XmRImmediate, (XtPointer)XmDRAG_UNDER_HIGHLIGHT
-    },
-    {
-	XmNdragProc, XmCDragProc, XmRProc,
-	sizeof(XtCallbackProc), Offset(dragProc),
-	XmRImmediate, (XtPointer)NULL
-    },
-    {
-	XmNdropProc, XmCDropProc, XmRProc,
-	sizeof(XtCallbackProc), Offset(dropProc),
-	XmRImmediate, (XtPointer)NULL
-    },
-    {
-	XmNdropRectangles, XmCDropRectangles, XmRRectangleList,
-	sizeof(XRectangle *), Offset(dropRectangles),
-	XmRImmediate, (XtPointer)NULL
-    },
-    {
-	XmNdropSiteActivity, XmCDropSiteActivity, XmRDropSiteActivity,
-	sizeof(unsigned char), Offset(dropSiteActivity),
-	XmRImmediate, (XtPointer)XmDROP_SITE_ACTIVE
-    },
-    {
-	XmNdropSiteType, XmCDropSiteType, XmRDropSiteType,
-	sizeof(unsigned char), Offset(dropSiteType),
-	XmRImmediate, (XtPointer)XmDROP_SITE_SIMPLE
-    },
-    {
-	XmNimportTargets, XmCImportTargets, XmRAtomList,
-	sizeof(Atom *), Offset(importTargets),
-	XmRImmediate, (XtPointer)NULL
-    },
-    {
-	XmNnumImportTargets, XmCNumImportTargets, XmRCardinal,
-	sizeof(Cardinal), Offset(numImportTargets),
-	XmRImmediate, (XtPointer)0
-    },
-    {
-	XmNnumDropRectangles, XmCNumDropRectangles, XmRCardinal,
-	sizeof(Cardinal), Offset(numDropRectangles),
-	XmRImmediate, (XtPointer)1
-    },
-    {
-	XmNdropSiteOperations, XmCDropSiteOperations, XmRDropSiteOperations,
-	sizeof(unsigned char), Offset(dropSiteOperations),
-	XmRImmediate, (XtPointer)(XmDROP_MOVE | XmDROP_COPY)
-    }
-};
-#undef Offset
-
-int _XmNumDSResources = XtNumber(_XmDSResources);
-
 /* *INDENT-OFF* */
 #if 0
 static XmBaseClassExtRec _XmDropSMgrObjectClassExtRec = {
@@ -269,7 +192,7 @@ XmDropSiteManagerClassRec xmDropSiteManagerClassRec = {
 	/* class_inited          */ False,
 	/* initialize            */ initialize,
 	/* initialize_hook       */ NULL,
-	/* obj1                  */ NULL,
+	/* obj1                  */ NULL, /* Motif has one of these */
 	/* obj2                  */ NULL,
 	/* obj3                  */ 0,
 	/* resources             */ resources,
@@ -331,6 +254,8 @@ WidgetClass xmDropSiteManagerObjectClass = (WidgetClass)(&xmDropSiteManagerClass
 static void
 class_initialize(void)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:class_initialize(%d)\n",
+    	__FILE__, __LINE__));
 #if 0
     _XmDropSMgrObjectClassExtRec.record_type = XmQmotif;
 #endif
@@ -341,6 +266,8 @@ class_part_initialize(WidgetClass widget_class)
 {
     WidgetClass sc;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:class_part_initialize(%d)\n",
+    	__FILE__, __LINE__));
     _XmFastSubclassInit(widget_class, XmDROP_SITE_MANAGER_BIT);
 
     sc = widget_class->core_class.superclass;
@@ -452,6 +379,18 @@ initialize(Widget request, Widget new_w,
 {
     XmDropSiteInfoRec hr;
 
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:initialize(%d) - %i args\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
+		      *num_args,
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+
     DS_CurAnimate(new_w) = True;
 
     DS_CurDropSiteStatus(new_w) = XmINVALID_DROP_SITE;
@@ -495,6 +434,8 @@ initialize(Widget request, Widget new_w,
 static void
 destroy(Widget w)
 {
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:destroy(%d)\n",
+    	__FILE__, __LINE__));
     DSMDestroyTable((XmDropSiteManagerObject)w);
 
     _XmRegionDestroy(DS_CurAncestorClipRegion(w));
@@ -502,20 +443,39 @@ destroy(Widget w)
 }
 
 static Boolean
-set_values(Widget current, Widget request, Widget new_w,
+set_values(Widget old, Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:set_values(%d) - %i args\n"
+		      "\t    old X %5i Y %5i W %5i H %5i\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
+		      *num_args,
+		      XtX(old), XtY(old),
+		      XtWidth(old), XtHeight(old),
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+
     return True;
 }
 
 static void
 updateReceiverInfo(Widget w, XtPointer cd, XEvent *ev, Boolean *ctd)
 {
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:updateReceiverInfo(%d)\n",
+    	__FILE__, __LINE__));
 }
 
 static void
 externalSourceHandler(Widget w, XtPointer cd, XEvent *ev, Boolean *ctd)
 {
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:externalSourceHandler(%d)\n",
+    	__FILE__, __LINE__));
 #if 0
     Widget dc, disp;
     XmDropSiteManagerObject dsm;
@@ -545,6 +505,8 @@ updateTree(Widget w, XtPointer cd, XtPointer cbs)
     Widget disp = XmGetXmDisplay(XtDisplay(w));
     XmTreeUpdateCallbackStruct *tucbs = (XmTreeUpdateCallbackStruct *) cbs;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:updateTree(%d)\n",
+    	__FILE__, __LINE__));
     if (Display_DragReceiverProtocolStyle(disp) == XmDRAG_NONE)
     {
 	return;
@@ -592,6 +554,8 @@ destroyInfo(Widget w, XtPointer cd, XtPointer cbs)
 {
     Widget dsm = (Widget)cd;
 
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:destroyInfo(%d)\n",
+    	__FILE__, __LINE__));
     DSMDestroyInfo((XmDropSiteManagerObject)dsm, w);
 }
 
@@ -602,6 +566,8 @@ createShellInfoRec(XmDropSiteManagerObject dsm, Widget shell)
     XmDropSiteInfo dsi;
     XRectangle rect;
 
+    DEBUGOUT(XdbDebug(__FILE__, shell, "%s:createShellInfoRec(%d)\n",
+    	__FILE__, __LINE__));
     dsi = (XmDropSiteInfo) XtCalloc(1, sizeof(XmDropSiteInfoRec));
 
     dsi->leaf = True;
@@ -631,6 +597,9 @@ createShellInfoRec(XmDropSiteManagerObject dsm, Widget shell)
 static void
 addChildToComposite(XmDropSiteInfo pi, XmDropSiteInfo ci, short pos)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:addChildToComposite(%d)\n",
+    	__FILE__, __LINE__));
+
     if (pi == NULL || ci == NULL)
 	return;
 
@@ -675,8 +644,11 @@ addChildToComposite(XmDropSiteInfo pi, XmDropSiteInfo ci, short pos)
     if (pi->numChildren)
     {
 
-	bcopy(&pi->children[pos], &pi->children[pos + 1],
-	      (pi->numChildren - pos) * sizeof(XmDropSiteInfo));
+	/* T. Straumann: use mmove() !! (overlap) 
+	 * NOTE: src/dest swapped
+	 */
+	memmove(&pi->children[pos + 1],&pi->children[pos],
+	        (pi->numChildren - pos) * sizeof(XmDropSiteInfo));
     }
 
     pi->children[pos] = ci;
@@ -692,6 +664,9 @@ removeChildFromComposite(XmDropSiteInfo pi, XmDropSiteInfo dsi)
 {
     int i;
 
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:removeChildFromComposite(%d)\n",
+    	__FILE__, __LINE__));
+
     if (pi->dropSiteType != XmDROP_SITE_COMPOSITE)
     {
 	return;
@@ -710,8 +685,11 @@ removeChildFromComposite(XmDropSiteInfo pi, XmDropSiteInfo dsi)
 
     if (i != pi->numChildren - 1)
     {
-	bcopy(&pi->children[i + 1], &pi->children[i],
-	      (pi->numChildren - i - 1));
+	/* T. Straumann: use mmove() !! (overlap) 
+	 * NOTE: src/dest swapped
+	 */
+	memmove(&pi->children[i], &pi->children[i + 1],
+	      (pi->numChildren - i - 1)*sizeof(XmDropSiteInfo));
     }
 
     pi->numChildren--;
@@ -720,6 +698,9 @@ removeChildFromComposite(XmDropSiteInfo pi, XmDropSiteInfo dsi)
 static void
 destroyInfoRec(XmDropSiteInfo dsi, Boolean thorough)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:destroyInfoRec(%d)\n",
+    	__FILE__, __LINE__));
+
     if (dsi->dropSiteType == XmDROP_SITE_COMPOSITE && thorough &&
 	dsi->numChildren != 0)
     {
@@ -755,7 +736,12 @@ __XmDSMCreateInfo(XmDropSiteManagerObject object,
     Widget shell;
     XmDropSiteInfo info;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)object, "DSM CreateInfo\n"));
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)object,
+		      "%s:__XmDSMCreateInfo(%d) - %i args\n", 
+		      __FILE__, __LINE__,
+		      num_args
+		      ));
+    DEBUGOUT(XdbPrintArgList(__FILE__, (Widget)object, args, num_args, False));
 
     DSMStartUpdate(object, widget);
 
@@ -1112,6 +1098,9 @@ loc_in_info(XmDropSiteManagerObject dsm, XmDropSiteInfo dsi,
 {
     Position tx, ty;
 
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "%s:loc_in_info(%d)\n",
+    	__FILE__, __LINE__));
+
     XtTranslateCoords(dsi->dropSite, 0, 0, &tx, &ty);
 
     tx -= DS_RootX(dsm);
@@ -1137,6 +1126,9 @@ loc_to_info(XmDropSiteManagerObject dsm, XmDropSiteInfo dsi,
     int i;
     XmDropSiteInfo child;
 
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "%s:loc_to_info(%d)\n",
+    	__FILE__, __LINE__));
+
     if (!dsi->leaf)
     {
 
@@ -1186,6 +1178,9 @@ animate(XmDropSiteManagerObject dsm,
     XmDropSiteInfo dsi = (XmDropSiteInfo) DS_CurInfo(dsm);
     Widget dc;
 
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "%s:animate(%d)\n",
+    	__FILE__, __LINE__));
+
     if (dsi && dsi->animationStyle == XmDRAG_UNDER_NONE)
     {
 	return;
@@ -1207,6 +1202,9 @@ drop_site_enter(XmDropSiteManagerObject dsm, XmDragMotionClientDataStruct * cd,
     int nimps;
     Widget par;
 
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "%s:drop_site_enter(%d)\n",
+    	__FILE__, __LINE__));
+
     dpc.reason = XmCR_DROP_SITE_ENTER_MESSAGE;
     dpc.event = NULL;
     dpc.timeStamp = cbs->timeStamp;
@@ -1325,6 +1323,9 @@ drop_site_leave(XmDropSiteManagerObject dsm, XmDragMotionClientDataStruct * cd,
     XmDropSiteLeaveCallbackStruct dsl;
     Position x, y;
 
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "%s:drop_site_leave(%d)\n",
+    	__FILE__, __LINE__));
+
     dpc.reason = XmCR_DROP_SITE_LEAVE_MESSAGE;
     dpc.event = NULL;
     dpc.timeStamp = cbs->timeStamp;
@@ -1372,6 +1373,9 @@ drag_motion(XmDropSiteManagerObject dsm, XmDragMotionClientDataStruct * cd,
     XmDragMotionCallbackStruct dm;
     Position x, y;
 
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "%s:drag_motion(%d)\n",
+    	__FILE__, __LINE__));
+
     dpc.reason = XmCR_DROP_SITE_MOTION_MESSAGE;
     dpc.event = NULL;
     dpc.timeStamp = cbs->timeStamp;
@@ -1626,7 +1630,15 @@ __XmDSMProcessDrop(XmDropSiteManagerObject dsm,
 
 	dpc.dropAction = cbs->dropAction;
 
-	(*dsi->dropProc) (dsi->dropSite, NULL, (XtPointer)&dpc);
+	/* T. Straumann: safe guard against NULL added */
+	if (dsi->dropProc)
+	{
+		(*dsi->dropProc) (dsi->dropSite, NULL, (XtPointer)&dpc);
+	}
+	else
+	{
+		_XmWarning((Widget)dsm, "__XmDSMProcessDrop():  no dsi->dropProc!\n");
+	}
 
 	cbs->operation = dpc.operation;
 	cbs->operations = dpc.operations;
@@ -2037,7 +2049,7 @@ __XmDSMRegisterInfo(XmDropSiteManagerObject dsm,
 {
     XmDropSiteInfo dsi = (XmDropSiteInfo) info;
     DSInfoTable *dst;
-    int hash;
+    long hash;
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "DSM RegisterInfo\n"));
 
@@ -2048,7 +2060,7 @@ __XmDSMRegisterInfo(XmDropSiteManagerObject dsm,
 
     dst = (DSInfoTable *) DS_DSTable(dsm);
 
-    hash = (int)widget & dst->num_buckets;
+    hash = (long)widget & dst->num_buckets;
 
     dsi->next = dst->buckets[hash];
     dst->buckets[hash] = dsi;
@@ -2064,14 +2076,14 @@ __XmDSMWidgetToInfo(XmDropSiteManagerObject dsm,
 		    Widget widget)
 {
     DSInfoTable *info;
-    int hash;
+    long hash;
     XmDropSiteInfo dsi;
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "DSM WidgetToInfo\n"));
 
     info = (DSInfoTable *) DS_DSTable(dsm);
 
-    hash = (int)widget & info->num_buckets;
+    hash = (long)widget & info->num_buckets;
 
     if (info->buckets[hash] == NULL)
     {
@@ -2104,7 +2116,7 @@ __XmDSMUnregisterInfo(XmDropSiteManagerObject dsm,
 {
     XmDropSiteInfo dsi = (XmDropSiteInfo) info, *ptr;
     DSInfoTable *dst;
-    int hash;
+    long hash;
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "DSM UnregisterInfo\n"));
 
@@ -2115,7 +2127,7 @@ __XmDSMUnregisterInfo(XmDropSiteManagerObject dsm,
 
     dst = (DSInfoTable *) DS_DSTable(dsm);
 
-    hash = (int)dsi->dropSite & dst->num_buckets;
+    hash = (long)dsi->dropSite & dst->num_buckets;
 
     for (ptr = &dst->buckets[hash]; *ptr; ptr = &((*ptr)->next))
     {
@@ -2177,6 +2189,9 @@ has_ds_offspring(XmDropSiteManagerObject dsm, Widget w)
 {
     int i;
 
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "%s:has_ds_offspring(%d)\n",
+    	__FILE__, __LINE__));
+
     if (!XtIsComposite(w))
     {
 	return False;
@@ -2277,4 +2292,113 @@ _XmSyncDropSiteTree(Widget shell)
 void
 _XmIEndUpdate(XtPointer client_data, XtIntervalId *interval_id)
 {
+    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmIEndUpdate(%d)\n",
+    	__FILE__, __LINE__));
+
+}
+
+void
+XmDropSiteConfigureStackingOrder(Widget widget,
+				 Widget Sibling,
+				 Cardinal stack_mode)
+{
+    DEBUGOUT(XdbDebug(__FILE__, widget, "XmDropSiteConfigureStackingOrder()\n"));
+
+    /* FIX ME */
+}
+
+void
+XmDropSiteEndUpdate(Widget widget)
+{
+    Widget disp = XmGetXmDisplay(XtDisplay(widget));
+
+    DEBUGOUT(XdbDebug(__FILE__, widget, "XmDropSiteEndUpdate()\n"));
+
+    DSMEndUpdate(_XmGetDropSiteManagerObject((XmDisplay)disp), widget);
+}
+
+Status
+XmDropSiteQueryStackingOrder(Widget widget,
+			     Widget *parent_return,
+			     Widget **child_returns,
+			     Cardinal *num_child_returns)
+{
+    DEBUGOUT(XdbDebug(__FILE__, widget, "XmDropSiteQueryStackingOrder()\n"));
+
+    /* FIX ME */
+
+    return 0;
+}
+
+void
+XmDropSiteRegister(Widget widget,
+		   ArgList arglist,
+		   Cardinal argcount)
+{
+    Widget disp = XmGetXmDisplay(XtDisplay(widget));
+
+    DEBUGOUT(XdbDebug(__FILE__, widget,
+		      "%s:XmDropSiteRegister(%d) - %i args\n",
+		      __FILE__, __LINE__,
+		      argcount));
+    DEBUGOUT(XdbPrintArgList(__FILE__, widget, arglist, argcount, False));
+
+    DSMCreateInfo(_XmGetDropSiteManagerObject((XmDisplay)disp),
+		  widget, arglist, argcount);
+}
+
+void
+XmDropSiteRetrieve(Widget widget,
+		   ArgList arglist,
+		   Cardinal argcount)
+{
+    Widget disp = XmGetXmDisplay(XtDisplay(widget));
+
+    DEBUGOUT(XdbDebug(__FILE__, widget, "XmDropSiteRetrieve()\n"));
+
+    DSMRetrieveInfo(_XmGetDropSiteManagerObject((XmDisplay)disp),
+		    widget, arglist, argcount);
+}
+
+void
+XmDropSiteStartUpdate(Widget widget)
+{
+    Widget disp = XmGetXmDisplay(XtDisplay(widget));
+
+    DEBUGOUT(XdbDebug(__FILE__, widget, "XmDropSiteStartUpdate()\n"));
+
+    DSMStartUpdate(_XmGetDropSiteManagerObject((XmDisplay)disp), widget);
+}
+
+void
+XmDropSiteUnregister(Widget widget)
+{
+    Widget disp = XmGetXmDisplay(XtDisplay(widget));
+
+    DEBUGOUT(XdbDebug(__FILE__, widget, "XmDropSiteUnregister()\n"));
+
+    DSMDestroyInfo(_XmGetDropSiteManagerObject((XmDisplay)disp), widget);
+}
+
+void
+XmDropSiteUpdate(Widget widget,
+		 ArgList arglist,
+		 Cardinal argcount)
+{
+    Widget disp = XmGetXmDisplay(XtDisplay(widget));
+
+    DEBUGOUT(XdbDebug(__FILE__, widget, "XmDropSiteUpdate()\n"));
+
+    DSMUpdateInfo(_XmGetDropSiteManagerObject((XmDisplay)disp),
+		  widget, arglist, argcount);
+}
+
+XmDropSiteVisuals
+XmDropSiteGetActiveVisuals(Widget widget)
+{
+    /* FIX ME */
+
+    DEBUGOUT(XdbDebug(__FILE__, widget, "XmDropSiteGetActiveVisuals()\n"));
+
+    return NULL;
 }
