@@ -1,7 +1,7 @@
 #define MOZILLA
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.59 1998/10/02 04:17:24 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.83 1999/12/08 20:57:51 danny Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.59 1998/10/02 04:17:24 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.83 1999/12/08 20:57:51 danny Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -49,7 +49,7 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v
 
 /* Forward Declarations */
 
-static void class_initialize();
+static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass w_class);
 
@@ -85,9 +85,6 @@ static void _XmFsbButton(Widget w, XtPointer client, XtPointer call);
 
 static void _XmFsbFileSelect(Widget w, XtPointer client, XtPointer call);
 
-/* Extern */
-Boolean _XmSelectionBoxMatch(XmSelectionBoxWidget w);
-
 static XtAccelerators text_accelerators_parsed = NULL;
 
 
@@ -202,7 +199,14 @@ static XtResource resources[] =
 	XmNfileListItemCount, XmCItemCount, XmRInt,
 	sizeof(int), SBOffset(list_item_count),
 	XmRImmediate, (XtPointer)XmUNSPECIFIED
+    },
+#if XmVERSION > 2
+    {
+	XmNpathMode, XmCPathMode, XtEnum,
+	sizeof(int), SBOffset(path_mode),
+	XmRImmediate, XmPATH_MODE_FULL
     }
+#endif
 };
 
 /* Synthetic resources */
@@ -419,7 +423,7 @@ WidgetClass xmFileSelectionBoxWidgetClass = (WidgetClass)&xmFileSelectionBoxClas
 #define DEFAULT_LIST_WIDTH	100
 
 static void
-class_initialize()
+class_initialize(void)
 {
     if (text_accelerators_parsed == NULL)
     {
@@ -490,34 +494,9 @@ initialize(Widget request, Widget new_w,
 	FS_DirListLabelString(new_w) =
 		XmStringCopy(FS_DirListLabelString(new_w));
     }
-
-    FS_DirListLabel(new_w) = _XmBB_CreateLabelG(new_w,
-						FS_DirListLabelString(new_w),
-						"Dir");
-    argc = 0;
-    XtSetArg(argl[argc], XmNalignment, XmALIGNMENT_BEGINNING);
-    argc++;
-    XtSetValues(FS_DirListLabel(new_w), argl, argc);
-    XtManageChild(FS_DirListLabel(new_w));
-
-    /*
-     * dir list
-     */
-    argc = 0;
-    XtSetArg(argl[argc], XmNvisibleItemCount, SB_ListVisibleItemCount(new_w));
-    argc++;
-    XtSetArg(argl[argc], XmNselectionPolicy, XmBROWSE_SELECT);
-    argc++;
-    XtSetArg(argl[argc], XmNscrollBarDisplayPolicy, XmSTATIC);
-    argc++;
-    XtSetArg(argl[argc], XmNlistSizePolicy, XmCONSTANT);
-    argc++;
-
-    FS_DirList(new_w) = XmCreateScrolledList(new_w, "DirList", argl, argc);
-
-    XtManageChild(FS_DirList(new_w));
-    XtAddCallback(FS_DirList(new_w), XmNbrowseSelectionCallback, _XmFsbButton, NULL);
-    XtAddCallback(FS_DirList(new_w), XmNdefaultActionCallback, _XmFsbButton, NULL);
+    if (FS_Pattern(new_w) != NULL) {
+	FS_Pattern(new_w) = XmStringCopy(FS_Pattern(new_w));
+    }
 
     /* rws file list always has a scroll bar
      * file list
@@ -570,7 +549,7 @@ initialize(Widget request, Widget new_w,
     if (SB_ApplyLabelString(new_w) == (XmString)XmUNSPECIFIED)
     {
 	SB_ApplyLabelString(new_w) =
-	    _XmOSGetLocalizedString(NULL,	/* FIX ME */
+	    _XmOSGetLocalizedString(NULL,	/* FIX ME */  /* leaks */
 				    new_w,
 				    XmNfilterLabelString,
 				    "Filter");
@@ -592,6 +571,15 @@ initialize(Widget request, Widget new_w,
     XtSetValues(FS_FilterLabel(new_w), argl, argc);
     XtManageChild(FS_FilterLabel(new_w));
 
+    FS_DirListLabel(new_w) = _XmBB_CreateLabelG(new_w,
+						FS_DirListLabelString(new_w),
+						"Dir");
+    argc = 0;
+    XtSetArg(argl[argc], XmNalignment, XmALIGNMENT_BEGINNING);
+    argc++;
+    XtSetValues(FS_DirListLabel(new_w), argl, argc);
+    XtManageChild(FS_DirListLabel(new_w));
+
     /*
      * filter text
      */
@@ -611,6 +599,25 @@ initialize(Widget request, Widget new_w,
 
     XtManageChild(FS_FilterText(new_w));
 
+    /*
+     * dir list
+     */
+    argc = 0;
+    XtSetArg(argl[argc], XmNvisibleItemCount, SB_ListVisibleItemCount(new_w));
+    argc++;
+    XtSetArg(argl[argc], XmNselectionPolicy, XmBROWSE_SELECT);
+    argc++;
+    XtSetArg(argl[argc], XmNscrollBarDisplayPolicy, XmSTATIC);
+    argc++;
+    XtSetArg(argl[argc], XmNlistSizePolicy, XmCONSTANT);
+    argc++;
+
+    FS_DirList(new_w) = XmCreateScrolledList(new_w, "DirList", argl, argc);
+
+    XtManageChild(FS_DirList(new_w));
+    XtAddCallback(FS_DirList(new_w), XmNbrowseSelectionCallback, _XmFsbButton, NULL);
+    XtAddCallback(FS_DirList(new_w), XmNdefaultActionCallback, _XmFsbButton, NULL);
+
     /* rws
      *  Override callbacks same as the list
      *  get rid of BBautoUnmanage
@@ -692,8 +699,9 @@ initialize(Widget request, Widget new_w,
         if (XmStringGetLtoR(FS_DirMask(new_w), XmFONTLIST_DEFAULT_TAG, &mask))
         {
             String pat = _XmOSFindPatternPart(mask);
-	    if (!_XmStringIsSpecified(FS_Pattern(new_w)))
+	    if (!_XmStringIsSpecified(FS_Pattern(new_w))) {
 		FS_Pattern(new_w) = XmStringCreateLocalized(pat);
+	    }
 
             if (pat > mask && !_XmStringIsSpecified(FS_Directory(new_w)))
 	    {
@@ -719,10 +727,20 @@ initialize(Widget request, Widget new_w,
 static void
 destroy(Widget w)
 {
+    DEBUGOUT(XdbDebug("RWS", w,"%s:destroy(%d)\n",
+    	__FILE__, __LINE__
+    	));
+    DEBUGOUT(XdbDebug("NEDIT", w,"%s:destroy(%d)\n",
+    	__FILE__, __LINE__
+    	));
     XmStringFree(FS_DirListLabelString(w));
     XmStringFree(FS_FileListLabelString(w));
     XmStringFree(FS_FilterLabelString(w));
     XmStringFree(FS_NoMatchString(w));
+	/* T. Straumann: don't forget to free these */
+	XmStringFree(FS_DirMask(w));
+	XmStringFree(FS_Pattern(w));
+	XmStringFree(FS_Directory(w));
 }
 
 static Boolean
@@ -732,6 +750,7 @@ set_values(Widget old, Widget request, Widget new_w,
     Arg al[10];
     int ac;
     Boolean refresh = False;
+	Boolean reSearch;
 
     DEBUGOUT(XdbDebug(__FILE__, new_w,
 		      "set_values: %i args\n"
@@ -803,12 +822,23 @@ set_values(Widget old, Widget request, Widget new_w,
 	XmStringFree(FS_DirListLabelString(old));
 
 	ac = 0;
-	XtSetArg(al[ac], XmNlabelString, FS_FilterLabelString(new_w)); ac++;
-
+	XtSetArg(al[ac], XmNlabelString, FS_DirListLabelString(new_w)); ac++;
+	XtSetValues(FS_DirListLabel(new_w),al,ac);
 
 	refresh = True;
     }
 
+	/* T. Straumann: this is a little weird:
+	 *
+	 * 				 FS_FileLabelString() is actually SB_ListLabelString().
+	 *
+	 *				 NOTE: I don't like this; set_values on FS_FileLabelString
+	 *					   is done by the SB widget.
+	 *					   However we, i.e. FSB does the initialization and
+	 *					   the destruction.
+	 *					   IMHO this is quite confusing and inelegant.
+	 */
+
     /* label string */
     if (FS_FilterLabelString(new_w) != FS_FilterLabelString(old))
     {
@@ -826,18 +856,68 @@ set_values(Widget old, Widget request, Widget new_w,
     {
 	String value;
 
-	XmStringGetLtoR(FS_DirSpec(new_w), XmFONTLIST_DEFAULT_TAG, &value);
+		if (XmStringGetLtoR(FS_DirSpec(new_w), XmFONTLIST_DEFAULT_TAG, &value))
+		{
+		XtVaSetValues(SB_Text(new_w), XmNvalue, value, NULL);
+		XtFree(value);
+		refresh = True;
+		}
 
-	XtVaSetValues(SB_Text(new_w), XmNvalue, value, NULL);
-	XtFree(value);
-
-	refresh = True;
     }
 
-    if (!XmStringCompare(FS_Directory(new_w), FS_Directory(old)) ||
-	!XmStringCompare(FS_DirMask(new_w), FS_DirMask(old)) ||
-	!XmStringCompare(FS_Pattern(new_w), FS_Pattern(old)) ||
-	FS_FileTypeMask(new_w) != FS_FileTypeMask(old))
+	/* T. Straumann: if any new string is supplied, we must 
+	 *				 free the old one and make a copy of the user
+	 *				 supplied value.
+	 *				 If the strings are equal, we MUST move the
+	 *				 old copy to the new_w and let the user's
+	 *				 copy alone!!!
+	 */
+	reSearch = False;
+    if ( !XmStringCompare(FS_Directory(new_w), FS_Directory(old)))
+	{
+		XmStringFree(FS_Directory(old));
+		FS_Directory(new_w) = XmStringCopy(FS_Directory(new_w));
+		reSearch = True;
+	}
+	else
+	{
+		FS_Directory(new_w) = FS_Directory(old);
+	}
+
+	if (!XmStringCompare(FS_DirMask(new_w), FS_DirMask(old)))
+	{
+		XmStringFree(FS_DirMask(old));
+		FS_DirMask(new_w) = XmStringCopy(FS_DirMask(new_w));
+		reSearch = True;
+	}
+	else
+	{
+		FS_DirMask(new_w) = FS_DirMask(old);
+	}
+
+	if (!XmStringCompare(FS_Pattern(new_w), FS_Pattern(old)))
+	{
+		XmStringFree(FS_Pattern(old));
+		FS_Pattern(new_w) = XmStringCopy(FS_Pattern(new_w));
+		reSearch = True;
+	}
+	else
+	{
+		FS_Pattern(new_w) = FS_Pattern(old);
+	}
+
+	if (!XmStringCompare(FS_NoMatchString(new_w), FS_NoMatchString(old)))
+	{
+		XmStringFree(FS_NoMatchString(old));
+		FS_NoMatchString(new_w) = XmStringCopy(FS_NoMatchString(new_w));
+		reSearch = True;
+	}
+	else
+	{
+		FS_NoMatchString(new_w) = FS_NoMatchString(old);
+	}
+
+	if ( reSearch || (FS_FileTypeMask(new_w) != FS_FileTypeMask(old)))
     {
 	_XmFileSelectionSearch(new_w);
     }
@@ -897,7 +977,10 @@ GetPreferredListWidth(Widget w)
 
     for (i = 0; i < NumItems; i++)
     {
-	XmStringGetLtoR(Items[i], XmFONTLIST_DEFAULT_TAG, &Item);
+	if ( ! XmStringGetLtoR(Items[i], XmFONTLIST_DEFAULT_TAG, &Item) )
+	{
+	continue;
+	}
 
 	if ((tmp = strrchr(Item, '/')) != NULL)
 	{
@@ -919,7 +1002,7 @@ GetPreferredListWidth(Widget w)
 	XtFree(Item);
     }
 
-    Width += 7;			/* Can someone explain this? What am I missing */
+    Width += 6;			/* Can someone explain this? What am I missing */
     Width += (listmargin * 2);
     Width += (highlight * 2);
 
@@ -1060,6 +1143,7 @@ _XmDirListFix(XmGeoMatrix geoSpec,
 	    XmKidGeometry fileList = &rowPtr[1];
 	    XmKidGeometry fileListLabel = &rowPtr[-2];
 	    XmKidGeometry dirList = &rowPtr[0];
+	    XmKidGeometry dirListLabel = &rowPtr[-3];
 	    Dimension DirPrefWidth;
 	    Dimension FilePrefWidth;
 
@@ -1082,13 +1166,13 @@ _XmDirListFix(XmGeoMatrix geoSpec,
 		fileList->box.x = geoSpec->width
 		    - fileList->box.width
 		    - geoSpec->margin_w
-		    - 2 * fileList->box.border_width;
+		    - 2 * fileList->box.border_width - 1;
 	    }
 	    else
 	    {
 	    	FilePrefWidth = 0;
 		fileList->box.width = FilePrefWidth;
-		fileList->box.x = geoSpec->width;
+		fileList->box.x = geoSpec->width - 1;
 	    }
 
 	    /*
@@ -1096,7 +1180,19 @@ _XmDirListFix(XmGeoMatrix geoSpec,
 	     */
 	     if (fileListLabel->kid == SB_ListLabel(geoSpec->composite))
 	     {
-		fileListLabel->box.x = fileList->box.x;
+		if (fileList->kid)
+		{
+		    fileListLabel->box.x = fileList->box.x;
+		    fileListLabel->box.width = fileList->box.width;
+		}
+		else
+		{
+		    fileListLabel->box.x = geoSpec->width
+			- XtWidth(fileListLabel->kid)
+			- geoSpec->margin_w
+			- 2 * fileList->box.border_width;
+		    fileListLabel->box.width = XtWidth(fileListLabel->kid);
+		}
 	     }
 	    /*
 	     * Position the dir list
@@ -1118,6 +1214,20 @@ _XmDirListFix(XmGeoMatrix geoSpec,
     		fileList->box.height,
     		fileList->box.x,
     		fileList->box.y ));
+	    /*
+	     * Position the dir list label
+	     */
+	     if (dirListLabel->kid == FS_DirListLabel(geoSpec->composite))
+	     {
+		if (fileList->kid)
+		{
+		    dirListLabel->box.width = dirList->box.width;
+		}
+		else
+		{
+		    dirListLabel->box.width = XtWidth(dirListLabel->kid);
+		}
+	     }
 	}
 	break;
 
@@ -1383,8 +1493,7 @@ geoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 	layoutPtr->sticky_end = True;
 	layoutPtr->space_above =
 	    (FS_DirListLabel(_w) && XtIsManaged(FS_DirListLabel(_w)))
-	    ? BB_MarginHeight(_w)
-	    : 0;
+	    ? 0 : BB_MarginHeight(_w) ;
 	layoutPtr->min_height = 40;
 	layoutPtr->space_between = BB_MarginWidth(_w);
 	newRow = True;
@@ -1403,8 +1512,7 @@ geoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 	layoutPtr->sticky_end = True;
 	layoutPtr->space_above =
 	    (SB_ListLabel(_w) && XtIsManaged(SB_ListLabel(_w)))
-	    ? BB_MarginHeight(_w)
-	    : 0;
+	    ? 0 : BB_MarginHeight(_w) ;
 	layoutPtr->min_height = 40;
 	layoutPtr->space_between = BB_MarginWidth(_w);
 	newRow = True;
@@ -1483,8 +1591,16 @@ geoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     {
 	layoutPtr->fill_mode = XmGEO_CENTER;
 	layoutPtr->fit_mode = XmGEO_WRAP;
-	layoutPtr->even_width = 1;
-	layoutPtr->even_height = 1;
+	if (LabG_RecomputeSize(SB_OkButton(_w)))
+	{
+	    layoutPtr->even_width = 6;
+	    layoutPtr->even_height = 6;
+	}
+	else
+	{
+	    layoutPtr->even_width = 1;
+	    layoutPtr->even_height = 1;
+	}
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	newRow = True;
     }
@@ -1498,8 +1614,17 @@ geoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 	    _XmBulletinBoardSetDefaultShadow(extras[i]);
 	    layoutPtr->fill_mode = XmGEO_CENTER;
 	    layoutPtr->fit_mode = XmGEO_WRAP;
-	    layoutPtr->even_width = 1;
-	    layoutPtr->even_height = 1;
+	    if ((XmIsPushButton(extras[i]) && Lab_RecomputeSize(extras[i])) || 
+	        (XmIsPushButtonGadget(extras[i]) && LabG_RecomputeSize(extras[i])))
+	    {
+		layoutPtr->even_width = 6;
+		layoutPtr->even_height = 6;
+	    }
+	    else
+	    {
+		layoutPtr->even_width = 1;
+		layoutPtr->even_height = 1;
+	    }
 	    layoutPtr->space_above = BB_MarginHeight(_w);
 	    newRow = True;
 	}
@@ -1510,8 +1635,16 @@ geoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     {
 	layoutPtr->fill_mode = XmGEO_CENTER;
 	layoutPtr->fit_mode = XmGEO_WRAP;
-	layoutPtr->even_width = 1;
-	layoutPtr->even_height = 1;
+	if (LabG_RecomputeSize(SB_ApplyButton(_w)))
+	{
+	    layoutPtr->even_width = 6;
+	    layoutPtr->even_height = 6;
+	}
+	else
+	{
+	    layoutPtr->even_width = 1;
+	    layoutPtr->even_height = 1;
+	}
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	newRow = True;
     }
@@ -1521,8 +1654,16 @@ geoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     {
 	layoutPtr->fill_mode = XmGEO_CENTER;
 	layoutPtr->fit_mode = XmGEO_WRAP;
-	layoutPtr->even_width = 1;
-	layoutPtr->even_height = 1;
+	if (LabG_RecomputeSize(SB_CancelButton(_w)))
+	{
+	    layoutPtr->even_width = 6;
+	    layoutPtr->even_height = 6;
+	}
+	else
+	{
+	    layoutPtr->even_width = 1;
+	    layoutPtr->even_height = 1;
+	}
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	newRow = True;
     }
@@ -1532,8 +1673,24 @@ geoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     {
 	layoutPtr->fill_mode = XmGEO_CENTER;
 	layoutPtr->fit_mode = XmGEO_WRAP;
-	layoutPtr->even_width = 1;
-	layoutPtr->even_height = 1;
+	if (LabG_RecomputeSize(SB_HelpButton(_w)))
+	{
+	    if (SB_List(_w) && XtIsManaged(XtParent(SB_List(_w))))
+	    {
+	    layoutPtr->even_width = 6;
+	    layoutPtr->even_height = 6;
+	    }
+	    else
+	    {
+	    layoutPtr->even_width = 4;
+	    layoutPtr->even_height = 4;
+	    }
+	}
+	else
+	{
+	    layoutPtr->even_width = 1;
+	    layoutPtr->even_height = 1;
+	}
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	newRow = True;
     }
@@ -1586,12 +1743,28 @@ focus_moved_proc(Widget wid, XtPointer client_data, XtPointer data)
 static Boolean
 noGeoRequest(XmGeoMatrix _geoSpec)
 {
+	/*
 	return True;
+	*/
+    DEBUGOUT(XdbDebug("FSB", _geoSpec->composite, "%s:noGeoRequest(%d) - instigator %s\n",
+    	__FILE__, __LINE__,
+    	_geoSpec->instigator ? XtName(_geoSpec->instigator) : "NULL"));
+
     if (BB_InSetValues(_geoSpec->composite) &&
 	(XtClass(_geoSpec->composite) == xmFileSelectionBoxWidgetClass))
     {
 	return True;
     }
+    if (_geoSpec->instigator && FS_DirList(_geoSpec->composite) && 
+        (XtParent(FS_DirList(_geoSpec->composite)) == _geoSpec->instigator))
+    {
+	return True;
+    }
+    if (_geoSpec->instigator && SB_List(_geoSpec->composite) && 
+        (XtParent(SB_List(_geoSpec->composite)) == _geoSpec->instigator))
+    {
+	return True;
+    }
 
     return False;
 }
@@ -1645,13 +1818,15 @@ defaultDirSearchProc(Widget widget,
 
 	XtFree(entries[i]);
     }
-    XtFree((char *)entries);
+    if (entries)
+    {
+	XtFree((char *)entries);
+    }
 
     argc = 0;
     XtSetArg(argl[argc], XmNitems, table); argc++;
     XtSetArg(argl[argc], XmNitemCount, numents); argc++;
-    XtSetValues(FS_DirList(widget), argl, argc);
-
+    XtSetValues(FS_DirList(widget), argl, argc); 
     for (i = 0; i < numents; i++)
     {
     	XmStringFree(table[i]);
@@ -1701,7 +1876,7 @@ defaultFileSearchProc(Widget widget, XtPointer data)
     entries = NULL;
     numents = numalloc = 0;
 
-    _XmOSBuildFileList(dir, pat, FS_FileTypeMask(widget),
+    _XmOSBuildFileList(dir, pat, FS_FileTypeMask(widget),  /* FIX ME: leaks */
 		       &entries, &numents, &numalloc);
     XtFree(dir);
     XtFree(pat);
@@ -1723,20 +1898,21 @@ defaultFileSearchProc(Widget widget, XtPointer data)
 	table[i] = XmStringCreateLtoR(entries[i], XmFONTLIST_DEFAULT_TAG);
 	XtFree(entries[i]);
     }
-    XtFree((char *)entries);
+    if (entries)
+    {
+	XtFree((char *)entries);
+    }
 
     argc = 0;
     XtSetArg(argl[argc], XmNfileListItems, table); argc++;
     XtSetArg(argl[argc], XmNfileListItemCount, numents); argc++;
-    XtSetValues((widget), argl, argc);
+    XtSetValues((widget), argl, argc); 
 
-    /*
     for (i = 0; i < numents; i++)
     {
 	XmStringFree(table[i]);
     }
     XtFree((String)table);
-    */
 
     FS_ListUpdated(widget) = True;
 }
@@ -1953,11 +2129,12 @@ _XmFileSelectionSearch(Widget w)
        to get the XmNdirectory and compare this to the last time it was 
        called.  If it is the same, the search is not done.
      */
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFileSelectionSearch()\n\tdirectory %s\n\tin.dir %s\n\tout.dir %s\n",
-    		XdbXmString2String(FS_Directory(w)),
-    		XdbXmString2String(in.dir),
-    		XdbXmString2String(out.dir)
-    		));
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFileSelectionSearch()\n\tdirectory %s\n",
+		      XdbXmString2String(FS_Directory(w))));
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFileSelectionSearch()\n\tin.dir %s\n",
+		      XdbXmString2String(in.dir)));
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFileSelectionSearch()\n\tout.dir %s\n",
+		      XdbXmString2String(out.dir)));
     if (FS_Directory(w))
     {
 	XmStringFree(FS_Directory(w));
@@ -2018,31 +2195,27 @@ _XmFileSelectionSearch(Widget w)
 		XtSetValues((w), argl, argc);
 		XmStringFree(table[0]);
 		XtFree((String)table);
-
-		argc = 0;
-
-		value = XtMalloc(1);
-		*value = 0;
-		XtSetArg(argl[argc], XmNvalue, value); argc++;
-		XtSetValues(SB_Text(w), argl, argc);
-
-		XtFree(value);
 	    }
-	    else if (FS_ListUpdated(w))
+	    else
+	    {
+		FS_StateFlags(w) &= ~XmFS_NO_MATCH;
+	    }
+	    if (FS_ListUpdated(w))
 	    {
 		if (FS_DirSpec(w))
 		{
 		    XmStringFree(FS_DirSpec(w));
 		}
 		FS_DirSpec(w) = XmStringCopy(out.dir);
-		XmStringGetLtoR(out.dir, XmFONTLIST_DEFAULT_TAG, &value);
+			if (XmStringGetLtoR(out.dir, XmFONTLIST_DEFAULT_TAG, &value))
+			{
 
-		FS_StateFlags(w) &= ~XmFS_NO_MATCH;
-		argc = 0;
-		XtSetArg(argl[argc], XmNvalue, value); argc++;
-		XtSetValues(SB_Text(w), argl, argc);
+			argc = 0;
+			XtSetArg(argl[argc], XmNvalue, value); argc++;
+			XtSetValues(SB_Text(w), argl, argc);
 
-		XtFree(value);
+			XtFree(value);
+			}
 	    }
 	    FS_StateFlags(w) &= ~XmFS_IN_FILE_SEARCH;
 	}
@@ -2085,6 +2258,11 @@ _XmFileSelectionSearch(Widget w)
     {
     	XmStringFree(out.mask);
     }
+	/* T. Straumann: fixed memory leak */
+	if (out.pattern)
+	{
+		XmStringFree(out.pattern);
+	}
     if (XtClass(w) == xmFileSelectionBoxWidgetClass)
     {
 	_XmBulletinBoardSizeUpdate(w);
@@ -2129,11 +2307,17 @@ XmCreateFileSelectionDialog(Widget parent,
 	XtSetArg(al[ac], argList[i].name, argList[i].value); ac++;
     }
 
+    /*
     d = XtCreateWidget(s, xmDialogShellWidgetClass, parent, al, ac);
+    */
+    d = XmCreateDialogShell(parent, s, al, ac);
     XtFree(s);
 
     XtSetArg(al[ac], XmNdialogType, XmDIALOG_FILE_SELECTION); ac++;
     ret = XtCreateWidget(name, xmFileSelectionBoxWidgetClass, d, al, ac);
+    XtAddCallback(ret, XmNdestroyCallback,
+		  _XmDestroyParentCallback,
+		  (XtPointer)ret);
     XtFree((XtPointer)al);
 
     return ret;
@@ -2570,6 +2754,16 @@ _XmFsbFileSelect(Widget w, XtPointer client, XtPointer call)
     XmTextFieldSetInsertionPosition(SB_Text(fsb),
 				    XmTextFieldGetLastPosition(SB_Text(fsb)));
     XtFree(s);
+    if (SB_TextString(fsb))
+    {
+    	XmStringFree(SB_TextString(fsb));
+    }
+    SB_TextString(fsb) = XmStringCopy(lp->item);
+    if (FS_DirMask(fsb))
+    {
+    	XmStringFree(FS_DirMask(fsb));
+    }
+    FS_DirMask(fsb) = XmStringCopy(lp->item);
 }
 
 /*
@@ -2652,7 +2846,10 @@ void
 FSBGetListItems(Widget wid, int resource_offset, XtArgVal *value)
 {
     /* According to Ken this is copied in SetValues, but not in GetValues */
+    /*
     *value = (XtArgVal)SB_ListItems(wid);
+    */
+    _XmSelectionBoxGetListItems(wid, resource_offset, value);
 }
 
 void 
@@ -2666,7 +2863,7 @@ void
 FSBGetPattern(Widget wid, int resource_offset, XtArgVal *value)
 {
     /* FIX ME : a copy or the real thing ? */
-    *value = (XtArgVal)FS_Pattern(wid);
+    *value = (XtArgVal)XmStringCopy(FS_Pattern(wid));
 }
 
 static void
@@ -2826,6 +3023,3 @@ FileSelectionBoxRestore(Widget w, XEvent *event, String *params, Cardinal *num_p
 					XmTextFieldGetLastPosition(text));
     }
 }
-
-
-
