@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ImageCache.c,v 1.13 1998/10/13 05:48:20 gritton Exp $
+ * $Id: ImageCache.c,v 1.17 1999/07/31 08:44:19 danny Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ImageCache.c,v 1.13 1998/10/13 05:48:20 gritton Exp $";
+static const char rcsid[] = "$Id: ImageCache.c,v 1.17 1999/07/31 08:44:19 danny Exp $";
 
 /*
  * --aldi 97/01/11: changed to use the generic hashtable module.
@@ -32,6 +32,7 @@ static const char rcsid[] = "$Id: ImageCache.c,v 1.13 1998/10/13 05:48:20 gritto
 #include <XmI/HashI.h>
 #include <X11/Xfuncs.h>
 #include <XmI/ImageCacheI.h>
+#include <Xm/XmosP.h>
 
 #include <Xm/XmP.h>
 #include <Xm/ScreenP.h>
@@ -279,8 +280,8 @@ XmGetPixmapByDepth(Screen *screen, char *image_name,
     GC gc;
     XGCValues values;
 #ifdef NONSTANDARD_CONVERTERS
-    XmXpmColorSymbol xpm_colour_symbols[2];
-    XmXpmAttributes xpm_attrib;
+    _LtXpmColorSymbol xpm_colour_symbols[2];
+    _LtXpmAttributes xpm_attrib;
 #endif /* NONSTANDARD_CONVERTERS */
 
     values.foreground = foreground;
@@ -461,14 +462,14 @@ XmGetPixmapByDepth(Screen *screen, char *image_name,
     xpm_attrib.numsymbols = XtNumber(xpm_colour_symbols);
     xpm_attrib.depth = depth;
     xpm_attrib.closeness = 40000;
-    xpm_attrib.valuemask = ( XmXpmSize | XmXpmReturnPixels | XmXpmColorSymbols
-			     | XmXpmCloseness | XmXpmDepth );
+    xpm_attrib.valuemask = ( _LtXpmSize | _LtXpmReturnPixels | _LtXpmColorSymbols
+			     | _LtXpmCloseness | _LtXpmDepth );
 
-    if (_XmXpmReadFileToPixmap(DisplayOfScreen(screen),
+    if (_LtXpmReadFileToPixmap(DisplayOfScreen(screen),
                                RootWindowOfScreen(screen),
                                pathname_to_pixmap,
 			       &new_pix, &mask,
-			       &xpm_attrib) == XmXpmSuccess)
+			       &xpm_attrib) == _LtXpmSuccess)
     {
 	/* add the pixmap to the cache */
 	LTAddPixmapToCache(image_name, new_pix, screen,
@@ -848,24 +849,24 @@ LTSetupImageCache(void)
  * calculate the hash value for an entry and compare two entries. This
  * is done on the tuple (image name, screen, foreground, background, depth).
  */
-static unsigned int
+static unsigned long
 LTPixmapGetHash(LTHashItemID id)
 {
     LTPixmapDesc p = (LTPixmapDesc) id;
-    unsigned int hash;
+    unsigned long hash;
     char *pStr, ch;
 
-    hash = 0;
+    hash = 0L;
     pStr = p->image_name;
     while ((ch = *pStr++) != 0)
     {
 	hash = (hash << 3) + ch;
     }
 
-    hash += (unsigned int)p->screen;
-    hash += ((unsigned int)p->foreground) * 71;
-    hash += (unsigned int)p->background;
-    hash += (unsigned int)p->depth;
+    hash += (unsigned long)p->screen;
+    hash += ((unsigned long)p->foreground) * 71;
+    hash += (unsigned long)p->background;
+    hash += (unsigned long)p->depth;
 
     return hash;
 }
@@ -888,13 +889,13 @@ LTPixmapCompare(LTHashItemID id1, LTHashItemID id2)
  * is merely a hash table that is used to lookup the information about a
  * particular pixmap using the pixmap and screen ids.
  */
-static unsigned int
+static unsigned long
 LTRevPixmapGetHash(LTHashItemID id)
 {
     LTPixmapDesc p = (LTPixmapDesc) id;
 
-    return ((unsigned int)p->pixmap) +
-	((unsigned int)p->screen);
+    return ((unsigned long)p->pixmap) +
+	((unsigned long)p->screen);
 }
 
 static Boolean
@@ -1033,6 +1034,14 @@ LTCreateSearchPath(void)
 {
     char *XBMLANGPATH = XtNewString((String)getenv("XBMLANGPATH"));
     char *XAPPLRESDIR = XtNewString((String)getenv("XAPPLRESDIR"));
+    char *HOME = _XmOSGetHomeDirName();		/* getenv(HOME) */
+    int HOME_length;
+
+    if (HOME == NULL) {
+	HOME = "";
+	HOME_length = 0;
+    } else
+	HOME_length = strlen(HOME);
 
     if (XBMLANGPATH)
     {
@@ -1045,8 +1054,6 @@ LTCreateSearchPath(void)
     }
     else if (XAPPLRESDIR)
     {
-	char *HOME = getenv("HOME");
-	int HOME_length = strlen(HOME);
 	int XAPPLRESDIR_length = strlen(XAPPLRESDIR);
 
 	_search_path = (char *)XtMalloc(strlen(XAPPLRESDIR_set_pattern)
@@ -1062,9 +1069,6 @@ LTCreateSearchPath(void)
     }
     else
     {				/* neither of them was set... */
-	char *HOME = getenv("HOME");
-	int HOME_length = strlen(HOME);
-
 	_search_path = (char *)XtMalloc(strlen(nothing_set_pattern)
 					+ HOME_length * 7 + 1);
 
