@@ -1,6 +1,6 @@
 /**
  *
- * $Id: LTCvt.c,v 1.5 1998/09/12 15:05:32 rwscott Exp $
+ * $Id: LTCvt.c,v 1.9 2000/01/03 23:29:11 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: LTCvt.c,v 1.5 1998/09/12 15:05:32 rwscott Exp $";
+static const char rcsid[] = "$Id: LTCvt.c,v 1.9 2000/01/03 23:29:11 rwscott Exp $";
 
 #ifdef NONSTANDARD_CONVERTERS 
 
@@ -59,12 +59,12 @@ extern Boolean LTAddPixmapToCache(char *pixmap_name, Pixmap pixmap,
  * of the pixmap and also where the foreground, background
  * and depth are equal to zero.
  */
-Pixmap
+static Pixmap
 _XmNSEGetPixmap(Screen *screen, char *fname)
 {
     static Colormap _cmap;
 
-    XmXpmAttributes xpm_attrib;
+    _LtXpmAttributes xpm_attrib;
     Pixmap pmap;
     Pixmap mask;
 
@@ -131,12 +131,12 @@ _XmNSEGetPixmap(Screen *screen, char *fname)
 
     xpm_attrib.colormap = _cmap;
     xpm_attrib.closeness = 40000;
-    xpm_attrib.valuemask = XmXpmSize | XmXpmReturnPixels
-	| XmXpmColormap | XmXpmCloseness;
+    xpm_attrib.valuemask = _LtXpmSize | _LtXpmReturnPixels
+	| _LtXpmColormap | _LtXpmCloseness;
 
-    if (_XmXpmReadFileToPixmap(dpy, w, pathname_to_pixmap,
+    if (_LtXpmReadFileToPixmap(dpy, w, pathname_to_pixmap,
 			       &pmap, &mask,
-			       &xpm_attrib) == XmXpmSuccess)
+			       &xpm_attrib) == _LtXpmSuccess)
     {
 	LTAddPixmapToCache(fname, pmap, screen, 0, 0, 0, 0, 0, 0, 0);
 	/* FIX ME! */
@@ -169,7 +169,6 @@ _XmNSECvtStringToPixmap(Display *dpy,
     static Pixmap _pmap;
     Screen *screen;
     char *name;
-    Pixmap _XmNSEGetPixmap(Screen *, char *);
 
     if (_XmGetDefaultDisplay() == NULL)
     {
@@ -310,34 +309,23 @@ static ConversionMap shape_style_map[] =
 	    return True;					\
 	}
 
-static Boolean _XmNSECvtShapeStyleToString();
-static Boolean _XmNSECvtBooleanToString();
-static Boolean _XmNSECvtBoolToString();
-
 static void
-XtDisplayConversionWarning(type, from, to)
-     String type;
-     int from;
-     String to;
+XtDisplayConversionWarning(String type, XtPointer from, String to)
 {
     String params[2];
     Cardinal num_params = 2;
 
     params[0] = (String)from;
-    params[1] = (String)to;
+    params[1] = to;
     XtWarningMsg("conversionError", type, XtCXtToolkitError,
 		 "Cannot convert value %d to type %s",
 		 params, &num_params);
 }
 
 static Boolean
-_XmNSECvtShapeStyleToString(dpy, args, numArgs, fromVal, toVal, data)
-     Display *dpy;
-     XrmValue *args;
-     Cardinal *numArgs;
-     XrmValue *fromVal;
-     XrmValue *toVal;
-     XtPointer *data;
+_XmNSECvtShapeStyleToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+			    XrmValue *fromVal, XrmValue *toVal,
+			    XtPointer *data)
 {
     int *i = (int *)(fromVal->addr);
     int index;
@@ -351,18 +339,13 @@ _XmNSECvtShapeStyleToString(dpy, args, numArgs, fromVal, toVal, data)
 	}
     }
 
-    XtDisplayConversionWarning(XtRShapeStyle, *i, XtRString);
+    XtDisplayConversionWarning(XtRShapeStyle, (XtPointer)*i, XtRString);
     return False;
 }
 
 static Boolean
-_XmNSECvtBooleanToString(dpy, args, numArgs, fromVal, toVal, data)
-     Display *dpy;
-     XrmValue *args;
-     Cardinal *numArgs;
-     XrmValue *fromVal;
-     XrmValue *toVal;
-     XtPointer *data;
+_XmNSECvtBooleanToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+			 XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
 {
     Boolean *i = (Boolean *)(fromVal->addr);
     int index;
@@ -376,18 +359,13 @@ _XmNSECvtBooleanToString(dpy, args, numArgs, fromVal, toVal, data)
 	}
     }
 
-    XtDisplayConversionWarning(XtRBoolean, *i, XtRString);
+    XtDisplayConversionWarning(XtRBoolean, (XtPointer)(long)*i, XtRString);
     return False;
 }
 
 static Boolean
-_XmNSECvtBoolToString(dpy, args, numArgs, fromVal, toVal, data)
-     Display *dpy;
-     XrmValue *args;
-     Cardinal *numArgs;
-     XrmValue *fromVal;
-     XrmValue *toVal;
-     XtPointer *data;
+_XmNSECvtBoolToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+		      XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
 {
     Boolean *i = (Boolean *)(fromVal->addr);
     int index;
@@ -401,18 +379,33 @@ _XmNSECvtBoolToString(dpy, args, numArgs, fromVal, toVal, data)
 	}
     }
 
-    XtDisplayConversionWarning(XtRBool, *i, XtRString);
+    XtDisplayConversionWarning(XtRBool, (XtPointer)(long)*i, XtRString);
     return False;
 }
 
 static Boolean
-_XmNSECvtPositionToString(dpy, args, numArgs, fromVal, toVal, data)
-     Display *dpy;
-     XrmValue *args;
-     Cardinal *numArgs;
-     XrmValue *fromVal;
-     XrmValue *toVal;
-     XtPointer *data;
+_XmNSECvtAttachmentToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+			  XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
+{
+    int size;
+    char *buffer;
+
+    if (*numArgs != 0)
+    {
+	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
+		  XtNwrongParameters, "cvtattachmentToString", XtCXtToolkitError,
+		      "Attachment to String conversion needs no extra arguments",
+			(String *)NULL, (Cardinal *)NULL);
+    }
+
+    buffer = XdbAttachment2String(*(Position *)fromVal->addr);
+    size = strlen(buffer);
+    string_convert_done(buffer);
+}
+
+static Boolean
+_XmNSECvtPositionToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+			  XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
 {
     int size;
     static char buffer[32];
@@ -431,13 +424,8 @@ _XmNSECvtPositionToString(dpy, args, numArgs, fromVal, toVal, data)
 }
 
 static Boolean
-_XmNSECvtIntToString(dpy, args, numArgs, fromVal, toVal, data)
-     Display *dpy;
-     XrmValue *args;
-     Cardinal *numArgs;
-     XrmValue *fromVal;
-     XrmValue *toVal;
-     XtPointer *data;
+_XmNSECvtIntToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+		     XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
 {
     int size;
     static char buffer[32];
@@ -456,13 +444,8 @@ _XmNSECvtIntToString(dpy, args, numArgs, fromVal, toVal, data)
 }
 
 static Boolean
-_XmNSECvtCardinalToString(dpy, args, numArgs, fromVal, toVal, data)
-     Display *dpy;
-     XrmValue *args;
-     Cardinal *numArgs;
-     XrmValue *fromVal;
-     XrmValue *toVal;
-     XtPointer *data;
+_XmNSECvtCardinalToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+			  XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
 {
     int size;
     static char buffer[32];
@@ -481,13 +464,8 @@ _XmNSECvtCardinalToString(dpy, args, numArgs, fromVal, toVal, data)
 }
 
 static Boolean
-_XmNSECvtPixelToString(dpy, args, numArgs, fromVal, toVal, data)
-     Display *dpy;
-     XrmValue *args;
-     Cardinal *numArgs;
-     XrmValue *fromVal;
-     XrmValue *toVal;
-     XtPointer *data;
+_XmNSECvtPixelToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+		       XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
 {
     int size;
     static char buffer[32];
@@ -506,13 +484,8 @@ _XmNSECvtPixelToString(dpy, args, numArgs, fromVal, toVal, data)
 }
 
 static Boolean
-_XmNSECvtColorToString(dpy, args, numArgs, fromVal, toVal, data)
-     Display *dpy;
-     XrmValue *args;
-     Cardinal *numArgs;
-     XrmValue *fromVal;
-     XrmValue *toVal;
-     XtPointer *data;
+_XmNSECvtColorToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+		       XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
 {
     int size;
     static char buffer[32];
@@ -532,13 +505,9 @@ _XmNSECvtColorToString(dpy, args, numArgs, fromVal, toVal, data)
 }
 
 static Boolean
-_XmNSECvtUnsignedCharToString(dpy, args, numArgs, fromVal, toVal, data)
-     Display *dpy;
-     XrmValue *args;
-     Cardinal *numArgs;
-     XrmValue *fromVal;
-     XrmValue *toVal;
-     XtPointer *data;
+_XmNSECvtUnsignedCharToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+			      XrmValue *fromVal, XrmValue *toVal,
+			      XtPointer *data)
 {
     int size;
     static char buffer[32];
@@ -558,13 +527,8 @@ _XmNSECvtUnsignedCharToString(dpy, args, numArgs, fromVal, toVal, data)
 }
 
 static Boolean
-_XmNSECvtDimensionToString(dpy, args, numArgs, fromVal, toVal, data)
-     Display *dpy;
-     XrmValue *args;
-     Cardinal *numArgs;
-     XrmValue *fromVal;
-     XrmValue *toVal;
-     XtPointer *data;
+_XmNSECvtDimensionToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+			   XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
 {
     int size;
     static char buffer[32];
@@ -583,13 +547,8 @@ _XmNSECvtDimensionToString(dpy, args, numArgs, fromVal, toVal, data)
 }
 
 static Boolean
-_XmNSECvtXmStringToString(dpy, args, numArgs, fromVal, toVal, data)
-     Display *dpy;
-     XrmValue *args;
-     Cardinal *numArgs;
-     XrmValue *fromVal;
-     XrmValue *toVal;
-     XtPointer *data;
+_XmNSECvtXmStringToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+			  XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
 {
     int size;
     char *s;
@@ -627,7 +586,7 @@ static XtConvertArgRec args[] =
  * register nonstandard extensions, if any
  */
 void
-_XmRegisterNSEConverters()
+_XmRegisterNSEConverters(void)
 {
 #if 1
     XtSetTypeConverter(XmRString,	/* source type */
@@ -660,6 +619,8 @@ _XmRegisterNSEConverters()
 		       NULL, 0, XtCacheNone, NULL);
     XtSetTypeConverter(XtRBool, XtRString, _XmNSECvtBoolToString,
 		       NULL, 0, XtCacheNone, NULL);
+    XtSetTypeConverter(XmRAttachment, XtRString, _XmNSECvtAttachmentToString,
+		       NULL, 0, XtCacheNone, NULL);
     XtSetTypeConverter(XtRPosition, XtRString, _XmNSECvtPositionToString,
 		       NULL, 0, XtCacheNone, NULL);
     XtSetTypeConverter(XtRInt, XtRString, _XmNSECvtIntToString,
