@@ -1,6 +1,6 @@
 /**
  *
- * $Id: MenuUtil.c,v 1.32 1998/10/17 06:25:04 gritton Exp $
+ * $Id: MenuUtil.c,v 1.61 1999/11/18 19:47:55 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: MenuUtil.c,v 1.32 1998/10/17 06:25:04 gritton Exp $";
+static const char rcsid[] = "$Id: MenuUtil.c,v 1.61 1999/11/18 19:47:55 amai Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -203,7 +203,7 @@ _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     /* Widget cb = NULL; */
     XmRowColumnWidget rc;
 
-    if (!_XmIsEventUnique(ev))
+    if (ev && !_XmIsEventUnique(ev))
     {
 	return;
     }
@@ -211,6 +211,7 @@ _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     _XmRecordEvent(ev);
 
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmMenuEscape()\n"));
+    DEBUGOUT(XdbDebug("ESC", w, "_XmMenuEscape()\n"));
     if (XmIsRowColumn(w))
     {
 	rc = (XmRowColumnWidget)w;
@@ -220,6 +221,7 @@ _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	rc = (XmRowColumnWidget)XtParent(w);
     }
     DEBUGOUT(XdbDebug(__FILE__, w, "    Menu      %s (%s)\n", XtName(rc), XdbRcType2String(RC_Type(rc))));
+    DEBUGOUT(XdbDebug("ESC", w, "    Menu      %s (%s)\n", XtName(rc), XdbRcType2String(RC_Type(rc))));
 
     if (RC_Type(rc) == XmMENU_BAR)
     {
@@ -239,12 +241,14 @@ _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     else if (_XmIsActiveTearOff((Widget)rc))
     {
 	DEBUGOUT(XdbDebug(__FILE__, w, "    Menu is TornOff\n"));
+	DEBUGOUT(XdbDebug("ESC", w, "    Menu is TornOff\n"));
 	if (RC_PopupPosted(rc))
 	{
 	Boolean poppedUp;
 	Widget menu = RC_PopupPosted(rc);
 
 	    DEBUGOUT(XdbDebug(__FILE__, w, "    Menu has %s posted\n", XtName(menu)));
+	    DEBUGOUT(XdbDebug("ESC", w, "    Menu has %s posted\n", XtName(menu)));
 	    while (RC_PopupPosted(menu))
 	    {
 		menu = RC_PopupPosted(menu);
@@ -255,7 +259,30 @@ _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	else
 	{
 	    DEBUGOUT(XdbDebug(__FILE__, w, "    Menu has nothing posted\n"));
-	    _XmDismissTearOff(XtParent(rc), NULL, NULL);
+	    DEBUGOUT(XdbDebug("ESC", w, "    Menu has nothing posted\n"));
+	    if (RC_TornOff(rc))
+	    {
+		RCClass_MenuProcs(XtClass(rc))(XmMENU_DISARM, (Widget)rc, NULL);
+		_XmDismissTearOff(XtParent(rc), NULL, NULL);
+	    }
+	    else
+	    {
+	    Boolean poppedUp;
+	    Widget cb = RC_CascadeBtn(rc);
+	    unsigned char type = XmIsGadget(cb) ? LabG_MenuType(cb) : Lab_MenuType(cb);
+
+		if (type == XmMENU_BAR)
+		{
+		    RCClass_MenuProcs(XtClass(XtParent(cb)))(XmMENU_BAR_CLEANUP,
+							 XtParent(cb),
+							 NULL);
+		}
+		else
+		{
+		    RC_MenuShellPopdown(cb, ev, &poppedUp);
+		    _XmMenuArmItem(cb);
+		}
+	    }
 	}
     }
     else if (RC_CascadeBtn(rc))
@@ -265,6 +292,7 @@ _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     unsigned char type = XmIsGadget(cb) ? LabG_MenuType(cb) : Lab_MenuType(cb);
 
 	DEBUGOUT(XdbDebug(__FILE__, w, "    Posted by %s (%s)\n", XtName(cb), XdbRcType2String(type)));
+	DEBUGOUT(XdbDebug("ESC", w, "    Posted by %s (%s)\n", XtName(cb), XdbRcType2String(type)));
 
 	if (type == XmMENU_BAR)
 	{
@@ -286,68 +314,6 @@ _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     	RC_MenuButtonPopdown(MGR_ActiveChild(rc), ev, &poppedUp);
     }
     return;
-
-#if 0
-#if 0
-    /* First guess */
-    cb = RC_MemWidget(rc);
-
-    /* Second guess */
-    if (!cb)
-    {
-	Widget p = RC_PopupPosted(rc);	/* Pane */
-
-	cb = RC_CascadeBtn(rc);
-	if (!cb && p)
-	{
-	    cb = RC_CascadeBtn(p);
-	}
-    }
-    else
-    {
-	DEBUGOUT(XdbDebug2(__FILE__, w, cb, "_XmMenuEscape() using memWidget\n"));
-    }
-
-    if (cb == 0)
-	_XmError(w, "_XmMenuEscape() cannot locate cascade button\n");
-#else
-    cb = RC_CascadeBtn(rc);
-#endif
-
-    DEBUGOUT(XdbDebug2(__FILE__, w, cb, "_XmMenuEscape()\n"));
-
-    if (cb && XmIsMenuBar(XtParent(cb)) || XmIsOptionMenu(XtParent(cb)))
-    {
-	RCClass_MenuProcs(XtClass(XtParent(cb)))(XmMENU_BAR_CLEANUP,
-						 XtParent(cb),
-						 NULL);
-    }
-    else
-    {
-#if 0
-	Widget pane = XmIsGadget(cb) ? CBG_Submenu(cb) : CB_Submenu(cb);
- 	Widget shell = XtParent(pane);
-
-        if (shell && XmIsMenuShell(shell) && MS_PrivateShell(shell))
-        {
-            XtCallActionProc(shell, "XtMenuPopdown",
-                             ev, params, *num_params);
-        }
-        else
-        {
-            XtUnmapWidget(pane);
-        }
-#else
-        if (cb)
-	{
-	Boolean poppedUp;
-
-	    RC_MenuShellPopdown(cb, ev, &poppedUp);
-	    _XmMenuArmItem(cb);
-	}
-#endif
-    }
-#endif
 }
 
 void
@@ -360,7 +326,21 @@ _XmMenuTraverseLeft(Widget w,
 
     if (!_XmIsEventUnique(ev)) return;
     _XmRecordEvent(ev);
-    _XmMenuTraversalHandler(XtParent(w), w, XmTRAVERSE_LEFT);
+    /* rws 27 Mar 1999
+       If we are in a menu that also has gadgets, and the gadget is the 
+       currently armed item, and the pointer is over a widget in the menu,
+       the widget will get the event that should be going to the gadget, as
+       it should.  In this case we need to traverse from the gadget.
+       (nedit Preferences menu)
+     */
+    if (MGR_ActiveChild(XtParent(w)) && XmIsGadget(MGR_ActiveChild(XtParent(w))))
+    {
+	_XmMenuTraversalHandler(XtParent(w), MGR_ActiveChild(XtParent(w)), XmTRAVERSE_LEFT);
+    }
+    else
+    {
+	_XmMenuTraversalHandler(XtParent(w), w, XmTRAVERSE_LEFT);
+    }
 }
 
 void
@@ -373,7 +353,14 @@ _XmMenuTraverseRight(Widget w,
 
     if (!_XmIsEventUnique(ev)) return;
     _XmRecordEvent(ev);
-    _XmMenuTraversalHandler(XtParent(w), w, XmTRAVERSE_RIGHT);
+    if (MGR_ActiveChild(XtParent(w)) && XmIsGadget(MGR_ActiveChild(XtParent(w))))
+    {
+	_XmMenuTraversalHandler(XtParent(w), MGR_ActiveChild(XtParent(w)), XmTRAVERSE_RIGHT);
+    }
+    else
+    {
+	_XmMenuTraversalHandler(XtParent(w), w, XmTRAVERSE_RIGHT);
+    }
 }
 
 void
@@ -386,7 +373,14 @@ _XmMenuTraverseUp(Widget w,
 
     if (!_XmIsEventUnique(ev)) return;
     _XmRecordEvent(ev);
-    _XmMenuTraversalHandler(XtParent(w), w, XmTRAVERSE_UP);
+    if (MGR_ActiveChild(XtParent(w)) && XmIsGadget(MGR_ActiveChild(XtParent(w))))
+    {
+	_XmMenuTraversalHandler(XtParent(w), MGR_ActiveChild(XtParent(w)), XmTRAVERSE_UP);
+    }
+    else
+    {
+	_XmMenuTraversalHandler(XtParent(w), w, XmTRAVERSE_UP);
+    }
 }
 
 void
@@ -396,10 +390,21 @@ _XmMenuTraverseDown(Widget w,
 		    Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmMenuTraverseDown()\n"));
+    DEBUGOUT(XdbDebug2("TRAV", XtParent(w), w, "_XmMenuTraverseDown() - %s %s\n",
+    	_XmIsEventUnique(ev) ? "unique" : "not-unique",
+    	MGR_ActiveChild(XtParent(w)) ? XtName(MGR_ActiveChild(XtParent(w))) : "no-active"
+    	));
 
     if (!_XmIsEventUnique(ev)) return;
     _XmRecordEvent(ev);
-    _XmMenuTraversalHandler(XtParent(w), w, XmTRAVERSE_DOWN);
+    if (MGR_ActiveChild(XtParent(w)) && XmIsGadget(MGR_ActiveChild(XtParent(w))))
+    {
+	_XmMenuTraversalHandler(XtParent(w), MGR_ActiveChild(XtParent(w)), XmTRAVERSE_DOWN);
+    }
+    else
+    {
+	_XmMenuTraversalHandler(XtParent(w), w, XmTRAVERSE_DOWN);
+    }
 }
 
 void
@@ -409,8 +414,14 @@ _XmRC_GadgetTraverseDown(Widget w,
 			 Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmRC_GadgetTraverseDown()\n"));
+    DEBUGOUT(XdbDebug("TRAV", w, "_XmRC_GadgetTraverseDown() - %s %s\n",
+    	_XmIsEventUnique(ev) ? "unique" : "not-unique",
+    	MGR_ActiveChild((w)) ? XtName(MGR_ActiveChild((w))) : "no-active"
+    	));
 
-    if (RC_Type(w) != XmMENU_BAR && XmIsGadget(MGR_ActiveChild(w)))
+    if (!_XmIsEventUnique(ev)) return;
+    _XmRecordEvent(ev);
+    if (RC_Type(w) != XmMENU_BAR && MGR_ActiveChild(w) /*&& XmIsGadget(MGR_ActiveChild(w))*/)
     {
 	_XmMenuTraversalHandler(w, MGR_ActiveChild(w), XmTRAVERSE_DOWN);
     }
@@ -424,7 +435,9 @@ _XmRC_GadgetTraverseUp(Widget w,
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmRC_GadgetTraverseUp()\n"));
 
-    if (RC_Type(w) != XmMENU_BAR && XmIsGadget(MGR_ActiveChild(w)))
+    if (!_XmIsEventUnique(ev)) return;
+    _XmRecordEvent(ev);
+    if (RC_Type(w) != XmMENU_BAR && MGR_ActiveChild(w) /*&& XmIsGadget(MGR_ActiveChild(w))*/)
     {
 	_XmMenuTraversalHandler(w, MGR_ActiveChild(w), XmTRAVERSE_UP);
     }
@@ -440,7 +453,7 @@ _XmRC_GadgetTraverseLeft(Widget w,
 
     if (!_XmIsEventUnique(ev)) return;
     _XmRecordEvent(ev);
-    if (XmIsGadget(MGR_ActiveChild(w)))
+    if (MGR_ActiveChild(w) /*&& XmIsGadget(MGR_ActiveChild(w))*/)
     {
 	_XmMenuTraversalHandler(w, MGR_ActiveChild(w), XmTRAVERSE_LEFT);
     }
@@ -456,7 +469,7 @@ _XmRC_GadgetTraverseRight(Widget w,
 
     if (!_XmIsEventUnique(ev)) return;
     _XmRecordEvent(ev);
-    if (XmIsGadget(MGR_ActiveChild(w)))
+    if (MGR_ActiveChild(w) /*&& XmIsGadget(MGR_ActiveChild(w))*/)
     {
 	_XmMenuTraversalHandler(w, MGR_ActiveChild(w), XmTRAVERSE_RIGHT);
     }
@@ -490,6 +503,7 @@ _XmMenuNextItem(Widget menu, Widget current_item)
     int current_item_index;
     int next_item_index;
 
+    /* printf("in XmMenuNextItem\n"); */
     for (current_item_index = 0;
          current_item_index < MGR_NumChildren(menu);
          current_item_index++)
@@ -508,15 +522,25 @@ _XmMenuNextItem(Widget menu, Widget current_item)
 	{
 	    for (valid_control = current_item_index + 1 < MGR_NumChildren(menu) ? current_item_index + 1 : 0; valid_control != current_item_index; valid_control = valid_control + 1 < MGR_NumChildren(menu) ? valid_control + 1 : 0)
 	    {
-		if (XmIsTraversable(MGR_Children(menu)[valid_control]))
+	      Widget w = MGR_Children(menu)[valid_control];
+	      /* printf("Examining %s (valid_control
+		 %d)\n",XtName(w),valid_control); */
+	      /* printf("Newmenus\n"); */
+	    if (w && XtIsSensitive(w) && XtIsManaged(w) &&
+		((XmIsLabel(w) && (((XmLabelClassRec *)(XtClass(w)))->label_class.menuProcs) && Prim_TraversalOn(w)) ||
+		 (XmIsLabelGadget(w) && (((XmLabelGadgetClassRec *)(XtClass(w)))->label_class.menuProcs) && G_TraversalOn(w))))
 		{
+		  /* printf("%s passed test\n",XtName(w)); */
+		  
 		    next_item_index = valid_control;
 		    break;
 		}
 	    }
 	}
     }
-
+    /*    printf("returning %s index %d\n",
+	   XtName(MGR_Children(menu)[next_item_index]),
+	   next_item_index); */
     return(MGR_Children(menu)[next_item_index]);
 }
 
@@ -541,7 +565,11 @@ _XmMenuPrevItem(Widget menu, Widget current_item)
 	{
 	    for (valid_control = current_item_index - 1 >= 0 ? current_item_index - 1 : MGR_NumChildren(menu) - 1; valid_control != current_item_index; valid_control = valid_control - 1 >= 0 ? valid_control - 1 : MGR_NumChildren(menu) - 1)
 	    {
-		if (XmIsTraversable(MGR_Children(menu)[valid_control]))
+	    Widget w = MGR_Children(menu)[valid_control];
+
+    if (w && XtIsSensitive(w) && XtIsManaged(w) &&
+	    ((XmIsLabel(w) && (((XmLabelClassRec *)(XtClass(w)))->label_class.menuProcs) && Prim_TraversalOn(w)) ||
+	     (XmIsLabelGadget(w) && (((XmLabelGadgetClassRec *)(XtClass(w)))->label_class.menuProcs) && G_TraversalOn(w))))
 		{
 			next_item_index = valid_control;
 			break;
@@ -556,20 +584,23 @@ _XmMenuPrevItem(Widget menu, Widget current_item)
 void
 _XmMenuDisarmItem(Widget w)
 {
-    if (w)
+    if (w &&
+	    ((XmIsLabel(w) && (((XmLabelClassRec *)(XtClass(w)))->label_class.menuProcs)) ||
+	     (XmIsLabelGadget(w) && (((XmLabelGadgetClassRec *)(XtClass(w)))->label_class.menuProcs))))
     {
-	    Lab_MenuDisarm(w);
+	Lab_MenuDisarm(w);
     }
 }
 
 void
 _XmMenuArmItem(Widget w)
 {
-    if (MGR_ActiveChild(XtParent(w)))
-    {
-	_XmMenuDisarmItem(MGR_ActiveChild(XtParent(w)));
-    }
-    if (!XmIsTraversable(w))
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmMenuArmItem()\n"));
+    DEBUGOUT(XdbDebug("MENU", w, "_XmMenuArmItem() - %s\n"
+    	));
+    if (w && (!XtIsSensitive(w) || !XtIsManaged(w) ||
+	    (!(XmIsLabel(w) && (((XmLabelClassRec *)(XtClass(w)))->label_class.menuProcs) && Prim_TraversalOn(w)) &&
+	     !(XmIsLabelGadget(w) && (((XmLabelGadgetClassRec *)(XtClass(w)))->label_class.menuProcs) && G_TraversalOn(w)))))
     {
     Widget next;
 
@@ -594,7 +625,6 @@ _XmMenuArmItem(Widget w)
 	    _XmMenuFocus(w, XmMENU_FOCUS_SET, CurrentTime);
 	}
 	Lab_MenuArm(w);
-        MGR_ActiveChild(XtParent(w)) = w;
     }
 }
 
@@ -609,26 +639,15 @@ _XmMenuTraversalHandler(Widget w, Widget pw, XmTraversalDirection direction)
     	{
 	    _XmWarning(pw, "%s(%i) - Traversal down in MENU_BAR not written yet!", __FILE__,__LINE__);
 	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Post %s\n", XtName(CB_Submenu(pw))));
-	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Disarm %s\n", XtName(MGR_ActiveChild(w))));
 	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Arm %s\n", XtName(MGR_Children(CB_Submenu(pw))[0])));
     	}
     	else if (RC_Type(w) == XmMENU_PULLDOWN || RC_Type(w) == XmMENU_POPUP)
     	{
-	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Disarm %s\n", XtName(MGR_ActiveChild(w))));
-	    _XmMenuDisarmItem(MGR_ActiveChild(w));
-
 	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Arm %s\n", XtName(_XmMenuNextItem(w, pw))));
+	    _XmMenuDisarmItem(pw);
 	    _XmMenuArmItem(_XmMenuNextItem(w,pw));
 
-	    if (XmIsGadget(MGR_ActiveChild(w)))
-	    {
-		_XmMenuFocus(XtParent(MGR_ActiveChild(w)), XmMENU_FOCUS_SET, CurrentTime);
-	    }
-	    else
-	    {
-		_XmMenuFocus(MGR_ActiveChild(w), XmMENU_FOCUS_SET, CurrentTime);
-	    }
-	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Active menu item is now %s\n", XtName(MGR_ActiveChild(w))));
+	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Active menu item is now %s\n", XtName(_XmMenuNextItem(w, pw))));
     	}
     	else
     	{
@@ -640,21 +659,11 @@ _XmMenuTraversalHandler(Widget w, Widget pw, XmTraversalDirection direction)
     case XmTRAVERSE_UP:
     	if (RC_Type(w) == XmMENU_PULLDOWN || RC_Type(w) == XmMENU_POPUP)
     	{
-	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Disarm %s\n", XtName(MGR_ActiveChild(w))));
-	    _XmMenuDisarmItem(MGR_ActiveChild(w));
-
 	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Arm %s\n", XtName(_XmMenuPrevItem(w, pw))));
+	    _XmMenuDisarmItem(pw);
 	    _XmMenuArmItem(_XmMenuPrevItem(w,pw));
 
-	    if (XmIsGadget(MGR_ActiveChild(w)))
-	    {
-		_XmMenuFocus(XtParent(MGR_ActiveChild(w)), XmMENU_FOCUS_SET, CurrentTime);
-	    }
-	    else
-	    {
-		_XmMenuFocus(MGR_ActiveChild(w), XmMENU_FOCUS_SET, CurrentTime);
-	    }
-	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Active menu item is now %s\n", XtName(MGR_ActiveChild(w))));
+	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Active menu item is now %s\n", XtName(_XmMenuPrevItem(w, pw))));
     	}
     	else
     	{
@@ -666,7 +675,7 @@ _XmMenuTraversalHandler(Widget w, Widget pw, XmTraversalDirection direction)
     case XmTRAVERSE_LEFT:
     	if (RC_Type(w) == XmMENU_BAR)
     	{
-    		int num_params = 0;
+    		Cardinal num_params = 0;
 
 		if (XmIsGadget(_XmMenuPrevItem(w,pw)))
 		{
@@ -679,18 +688,39 @@ _XmMenuTraversalHandler(Widget w, Widget pw, XmTraversalDirection direction)
     	}
     	else
     	{
-		DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    RC_LastSelectTopLevel %s %s\n", 
-			XtName(RC_LastSelectToplevel(w)),
-			XtName(MGR_ActiveChild(RC_LastSelectToplevel(w)))));
-			_XmMenuTraversalHandler( RC_LastSelectToplevel(w), 
-				MGR_ActiveChild(RC_LastSelectToplevel(w)),
-				direction);
+	    if (RC_CascadeBtn(w) && RC_Type(XtParent(RC_CascadeBtn(w))) != XmMENU_BAR)
+	    {
+	    Widget cb = RC_CascadeBtn(w);
+	    Boolean poppedUp;
+
+	    	RC_MenuShellPopdown(cb, NULL, &poppedUp);
+	    	_XmMenuArmItem(cb);
+	    }
+	    else if (RC_LastSelectToplevel(w))
+	    {
+		if (RC_CascadeBtn(w)  && !RC_TornOff(w))
+		{
+		    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    RC_LastSelectTopLevel %s %s\n", 
+			    XtName(RC_LastSelectToplevel(w)),
+			    /* XtName(MGR_ActiveChild(RC_LastSelectToplevel(w))), */
+			    XtName(RC_CascadeBtn(w))));
+		    _XmMenuTraversalHandler( RC_LastSelectToplevel(w), 
+				    RC_CascadeBtn(w),
+				    /* MGR_ActiveChild(RC_LastSelectToplevel(w)), */
+				    direction);
+		}
+	    }
+	    else
+	    {
+	    	_XmWarning(w, "%s:_XmMenuTraversalHandler(%d) - I have no idea what todo!",
+	    		__FILE__, __LINE__);
+	    }
     	}
     	break;
     case XmTRAVERSE_RIGHT:
     	if (RC_Type(w) == XmMENU_BAR)
     	{
-    		int num_params = 0;
+    		Cardinal num_params = 0;
 
 		if (XmIsGadget(_XmMenuNextItem(w,pw)))
 		{
@@ -703,32 +733,40 @@ _XmMenuTraversalHandler(Widget w, Widget pw, XmTraversalDirection direction)
     	}
     	else if (RC_Type(w) == XmMENU_PULLDOWN || RC_Type(w) == XmMENU_POPUP)
     	{
-    		if (XmIsCascadeButton(pw))
+    		if (XmIsCascadeButton(pw) && CB_Submenu(pw))
     		{
-    		int num_params = 0;
+    		Cardinal num_params = 0;
 
     			(*PrimC_ArmAndActivate(XtClass(pw)))(pw, NULL, NULL, &num_params);
-			_XmMenuArmItem(MGR_Children(CB_Submenu(pw))[0]);
-			MGR_ActiveChild(CB_Submenu(pw)) = MGR_Children(CB_Submenu(pw))[0];
+			RCClass_MenuTraverse(CB_Submenu(pw), XmTRAVERSE_HOME);
     		}
-    		else if (XmIsCascadeButtonGadget(pw))
+    		else if (XmIsCascadeButtonGadget(pw) && CBG_Submenu(pw))
     		{
-    		int num_params = 0;
+    		Cardinal num_params = 0;
 
     			(*GC_ArmAndActivate(XtClass(pw)))(pw, NULL, NULL, &num_params);
-			_XmMenuArmItem(MGR_Children(CBG_Submenu(pw))[0]);
-			MGR_ActiveChild(CB_Submenu(pw)) = MGR_Children(CBG_Submenu(pw))[0];
-
-			_XmMenuFocus(MGR_ActiveChild(CBG_Submenu(pw)), XmMENU_FOCUS_SET, CurrentTime);
+			RCClass_MenuTraverse(CBG_Submenu(pw), XmTRAVERSE_HOME);
     		}
     		else
     		{
-		    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    RC_LastSelectTopLevel %s %s\n", 
-			XtName(RC_LastSelectToplevel(w)),
-			XtName(MGR_ActiveChild(RC_LastSelectToplevel(w)))));
-			_XmMenuTraversalHandler( RC_LastSelectToplevel(w), 
-				MGR_ActiveChild(RC_LastSelectToplevel(w)),
-				direction);
+		    if (RC_CascadeBtn(w)  && !RC_TornOff(w))
+		    {
+			DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    RC_LastSelectTopLevel %s %s\n", 
+			    XtName(RC_LastSelectToplevel(w)),
+			    /* XtName(MGR_ActiveChild(RC_LastSelectToplevel(w))), */
+			    XtName(RC_CascadeBtn(w))));
+			if (RC_Type(RC_LastSelectToplevel(w)) == XmMENU_BAR)
+			{
+			    DEBUGOUT(XdbDebug2("TRAV", w, pw, "    RC_LastSelectTopLevel %s %s %s posted\n", 
+				XtName(RC_LastSelectToplevel(w)),
+				XtName(RC_CascadeBtn(w)),
+				RC_PopupPosted(w) ? XtName(RC_PopupPosted(w)) : "nothing"
+				));
+			    _XmMenuTraversalHandler( RC_LastSelectToplevel(w), 
+				    RC_CascadeBtn(RC_PopupPosted(RC_LastSelectToplevel(w))),
+				    direction);
+			}
+		    }
     		}
     	}
     	else
@@ -742,7 +780,7 @@ _XmMenuTraversalHandler(Widget w, Widget pw, XmTraversalDirection direction)
     }
 }
 
-int num_saved = 0;
+static int num_saved = 0;
 static String saved_trans[32];
 
 void
@@ -791,6 +829,13 @@ _XmGetMenuState(Widget widget)
     return (XmMenuState)info->menu_state;
 }
 
+#if XmVERSION > 1
+/* Use routines in PopupUtil.c if a 2.0 build */
+#else
+/* For the prototype police */
+extern void _XmPopup(Widget shell, XtGrabKind grab_kind);
+extern void _XmPopdown(Widget shell);
+
 void
 _XmPopup(Widget shell, XtGrabKind grab_kind)
 {
@@ -802,4 +847,4 @@ _XmPopdown(Widget shell)
 {
     XtPopdown(shell);
 }
-
+#endif
