@@ -1,7 +1,7 @@
 /*                   -*-Mode: c; c-file-style: "GNU"-*-              */
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.34 1998/08/27 17:39:48 pgw Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.46 2000/01/09 22:21:21 hennessy Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.34 1998/08/27 17:39:48 pgw Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.46 2000/01/09 22:21:21 hennessy Exp $";
 
 #include <LTconfig.h>
 #include <Xm/XmP.h>
@@ -41,90 +41,181 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v
 
 #include <XmI/DebugUtil.h>
 
-static void Activate();
-static void MoveBackwardChar();
-static void MoveBackwardParagraph();
-static void MoveBackwardWord();
-static void RingBell();
-static void MoveBeginningOfFile();
-static void MoveToLineStart();
-static void ClearSelection();
-static void CopyClipboard();
-static void CopyPrimary();
-static void ProcessCopy();
-static void CutClipboard();
-static void CutPrimary();
-static void DeleteForwardChar();
-static void DeleteBackwardChar();
-static void DeleteForwardWord();
-static void DeleteBackwardWord();
-static void DeleteCurrentSelection();
-static void DeleteToEndOfLine();
-static void DeleteToStartOfLine();
-static void DeselectAll();
-static void DoSelection();
-static void MoveEndOfFile();
-static void MoveToLineEnd();
-static void ExtendEnd();
-static void StartExtendSelection();
-static void FindWord();
-static void MoveForwardChar();
-static void MoveForwardParagraph();
-static void MoveForwardWord();
-static void DoGrabFocus();
-static void InsertString();
-static void KeySelection();
-static void KillForwardChar();
-static void KillForwardWord();
-static void KillBackwardChar();
-static void KillBackwardWord();
-static void KillCurrentSelection();
-static void KillToEndOfLine();
-static void KillToStartOfLine();
-static void MoveDestination();
-static void ProcessMove();
-static void InsertNewLine();
-static void InsertNewLineAndBackup();
-static void InsertNewLineAndIndent();
-static void MoveNextLine();
-static void MoveNextPage();
-static void TraverseNextTabGroup();
-static void MovePageLeft();
-static void MovePageRight();
-static void PasteClipboard();
-static void TraversePrevTabGroup();
-static void MovePreviousLine();
-static void MovePreviousPage();
-static void ProcessBDrag();
-static void ProcessCancel();
-static void ProcessDown();
-static void ProcessUp();
-static void ProcessHome();
-static void ProcessReturn();
-static void ProcessShiftDown();
-static void ProcessShiftUp();
-static void ProcessTab();
-static void RedrawDisplay();
-static void ScrollOneLineDown();
-static void ScrollOneLineUp();
-static void SecondaryAdjust();
-static void SecondaryNotify();
-static void ExtendSelection();
-static void SelectAll();
-static void StartExtendSelection();
-static void SelfInsert();
-static void SetAnchor();
-static void SetCursorPosition();
-static void SetSelectionHint();
-static void ToggleAddMode();
-static void TraverseHome();
-static void TraverseDown();
-static void TraverseUp();
-static void TextFocusIn();
-static void TextFocusOut();
-static void TextLeave();
-static void VoidAction();
-static void UnKill();
+static void Activate(Widget w, XEvent *ev, String *params,
+		     Cardinal *num_params);
+static void MoveBackwardChar(Widget w, XEvent *ev, String *params,
+			     Cardinal *num_params);
+static void MoveBackwardParagraph(Widget w, XEvent *ev, String *params,
+				  Cardinal *num_params);
+static void MoveBackwardWord(Widget w, XEvent *ev, String *params,
+			     Cardinal *num_params);
+static void RingBell(Widget w, XEvent *ev, String *params,
+		     Cardinal *num_params);
+static void MoveBeginningOfFile(Widget w, XEvent *ev, String *params,
+				Cardinal *num_params);
+static void MoveToLineStart(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void ClearSelection(Widget w, XEvent *ev, String *params,
+			   Cardinal *num_params);
+static void CopyClipboard(Widget w, XEvent *ev, String *params,
+			  Cardinal *num_params);
+static void CopyPrimary(Widget w, XEvent *ev, String *params,
+			Cardinal *num_params);
+static void ProcessCopy(Widget w, XEvent *ev, String *params,
+			Cardinal *num_params);
+static void CutClipboard(Widget w, XEvent *ev, String *params,
+			 Cardinal *num_params);
+static void CutPrimary(Widget w, XEvent *ev, String *params,
+		       Cardinal *num_params);
+static void DeleteForwardChar(Widget w, XEvent *ev, String *params,
+			      Cardinal *num_params);
+static void DeleteBackwardChar(Widget w, XEvent *ev, String *params,
+			       Cardinal *num_params);
+static void DeleteForwardWord(Widget w, XEvent *ev, String *params,
+			      Cardinal *num_params);
+static void DeleteBackwardWord(Widget w, XEvent *ev, String *params,
+			       Cardinal *num_params);
+static void DeleteCurrentSelection(Widget w, XEvent *ev, String *params,
+				   Cardinal *num_params);
+static void DeleteToEndOfLine(Widget w, XEvent *ev, String *params,
+			      Cardinal *num_params);
+static void DeleteToStartOfLine(Widget w, XEvent *ev, String *params,
+				Cardinal *num_params);
+static void DeselectAll(Widget w, XEvent *ev, String *params,
+			Cardinal *num_params);
+static void DoSelection(Widget w, XEvent *ev, String *params,
+			Cardinal *num_params);
+static void MoveEndOfFile(Widget w, XEvent *ev, String *params,
+			  Cardinal *num_params);
+static void MoveToLineEnd(Widget w, XEvent *ev, String *params,
+			  Cardinal *num_params);
+static void ExtendEnd(Widget w, XEvent *ev, String *params,
+		      Cardinal *num_params);
+static void StartExtendSelection(Widget w, XEvent *ev, String *params,
+				 Cardinal *num_params);
+static void FindWord(Widget w, XEvent *ev, String *params,
+		     Cardinal *num_params);
+static void MoveForwardChar(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void MoveForwardParagraph(Widget w, XEvent *ev, String *params,
+				 Cardinal *num_params);
+static void MoveForwardWord(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void DoGrabFocus(Widget w, XEvent *ev, String *params,
+			Cardinal *num_params);
+static void InsertString(Widget w, XEvent *ev, String *params,
+			 Cardinal *num_params);
+static void KeySelection(Widget w, XEvent *ev, String *params,
+			 Cardinal *num_params);
+static void KillForwardChar(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void KillForwardWord(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void KillBackwardChar(Widget w, XEvent *ev, String *params,
+			     Cardinal *num_params);
+static void KillBackwardWord(Widget w, XEvent *ev, String *params,
+			     Cardinal *num_params);
+static void KillCurrentSelection(Widget w, XEvent *ev, String *params,
+				 Cardinal *num_params);
+static void KillToEndOfLine(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void KillToStartOfLine(Widget w, XEvent *ev, String *params,
+			      Cardinal *num_params);
+static void MoveDestination(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void ProcessMove(Widget w, XEvent *ev, String *params,
+			Cardinal *num_params);
+static void InsertNewLine(Widget w, XEvent *ev, String *params,
+			  Cardinal *num_params);
+static void InsertNewLineAndBackup(Widget w, XEvent *ev, String *params,
+				   Cardinal *num_params);
+static void InsertNewLineAndIndent(Widget w, XEvent *ev, String *params,
+				   Cardinal *num_params);
+static void MoveNextLine(Widget aw, XEvent *ev, String *params,
+			 Cardinal *num_params);
+static void MoveNextPage(Widget aw, XEvent *ev, String *params,
+			 Cardinal *num_params);
+static void TraverseNextTabGroup(Widget w, XEvent *ev, String *params,
+				 Cardinal *num_params);
+static void MovePageLeft(Widget w, XEvent *ev, String *params,
+			 Cardinal *num_params);
+static void MovePageRight(Widget w, XEvent *ev, String *params,
+			  Cardinal *num_params);
+static void PasteClipboard(Widget w, XEvent *ev, String *params,
+			   Cardinal *num_params);
+static void TraversePrevTabGroup(Widget w, XEvent *ev, String *params,
+				 Cardinal *num_params);
+static void MovePreviousLine(Widget aw, XEvent *ev, String *params,
+			     Cardinal *num_params);
+static void MovePreviousPage(Widget aw, XEvent *ev, String *params,
+			     Cardinal *num_params);
+static void ProcessBDrag(Widget w, XEvent *ev, String *params,
+			 Cardinal *num_params);
+static void ProcessCancel(Widget w, XEvent *ev, String *params,
+			  Cardinal *num_params);
+static void ProcessDown(Widget w, XEvent *ev, String *params,
+			Cardinal *num_params);
+static void ProcessUp(Widget w, XEvent *ev, String *params,
+		      Cardinal *num_params);
+static void ProcessHome(Widget w, XEvent *ev, String *params,
+			Cardinal *num_params);
+static void ProcessReturn(Widget w, XEvent *ev, String *params,
+			  Cardinal *num_params);
+static void ProcessShiftDown(Widget w, XEvent *ev, String *params,
+			     Cardinal *num_params);
+static void ProcessShiftUp(Widget w, XEvent *ev, String *params,
+			   Cardinal *num_params);
+static void ProcessTab(Widget w, XEvent *ev, String *params,
+		       Cardinal *num_params);
+static void RedrawDisplay(Widget w, XEvent *ev, String *params,
+			  Cardinal *num_params);
+static void ScrollCursorVertically(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void ScrollOneLineDown(Widget w, XEvent *ev, String *params,
+			      Cardinal *num_params);
+static void ScrollOneLineUp(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void SecondaryAdjust(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void SecondaryNotify(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void SecondaryStart(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void ExtendSelection(Widget w, XEvent *ev, String *params,
+			    Cardinal *num_params);
+static void SelectAll(Widget w, XEvent *ev, String *params,
+		      Cardinal *num_params);
+static void StartExtendSelection(Widget w, XEvent *ev, String *params,
+				 Cardinal *num_params);
+static void SelfInsert(Widget w, XEvent *ev, String *params,
+		       Cardinal *num_params);
+static void SetAnchor(Widget w, XEvent *ev, String *params,
+		      Cardinal *num_params);
+static void SetCursorPosition(Widget w, XEvent *ev, String *params,
+			      Cardinal *num_params);
+static void SetSelectionHint(Widget w, XEvent *ev, String *params,
+			     Cardinal *num_params);
+static void ToggleAddMode(Widget w, XEvent *ev, String *params,
+			  Cardinal *num_params);
+static void ToggleOverstrike(Widget w, XEvent *ev, String *params,
+			  Cardinal *num_params);
+static void TraverseHome(Widget w, XEvent *ev, String *params,
+			 Cardinal *num_params);
+static void TraverseDown(Widget w, XEvent *ev, String *params,
+			 Cardinal *num_params);
+static void TraverseUp(Widget w, XEvent *ev, String *params,
+		       Cardinal *num_params);
+static void TextFocusIn(Widget w, XEvent *ev, String *params,
+			Cardinal *num_params);
+static void TextFocusOut(Widget w, XEvent *ev, String *params,
+			 Cardinal *num_params);
+static void TextLeave(Widget w, XEvent *ev, String *params,
+		      Cardinal *num_params);
+static void VoidAction(Widget w, XEvent *ev, String *params,
+		       Cardinal *num_params);
+static void UnKill(Widget w, XEvent *ev, String *params,
+		   Cardinal *num_params);
+
 extern void LineTable(XmTextWidget w);
 
 #define Offset(field) XtOffsetOf(XmTextInnerRec, inner.in.field)
@@ -230,10 +321,12 @@ static XtActionsRec ZdefaultTextActionsTable[] =
     {"quick-copy-set", VoidAction},
     {"quick-cut-set", VoidAction},
     {"redraw-display", RedrawDisplay},
+    {"scroll-cursor-vertically", ScrollCursorVertically},
     {"scroll-one-line-down", ScrollOneLineDown},
     {"scroll-one-line-up", ScrollOneLineUp},
     {"secondary-adjust", SecondaryAdjust},
     {"secondary-notify", SecondaryNotify},
+    {"secondary-start", SecondaryStart},
     {"select-adjust", DoSelection},
     {"select-all", SelectAll},
     {"select-end", DoSelection},
@@ -243,6 +336,7 @@ static XtActionsRec ZdefaultTextActionsTable[] =
     {"set-insertion-point", SetCursorPosition},
     {"set-selection-hint", SetSelectionHint},
     {"toggle-add-mode", ToggleAddMode},
+    {"toggle-overstrike", ToggleOverstrike},
     {"traverse-home", TraverseHome},
     {"traverse-next", TraverseDown},
     {"traverse-prev", TraverseUp},
@@ -261,6 +355,14 @@ static void
 Invalidate(XmTextWidget w, XmTextPosition start, XmTextPosition end,
 	   long delta)
 {
+    if ( delta == PASTENDPOS )
+	In_HighlightEnd(Text_InputData(w)) = 0 ;
+    else
+	{
+	/*CP:This works in some cases but not all !!
+	In_HighlightEnd(Text_InputData(w)) += delta ;
+	CP*/
+	}
 }
 
 static void
@@ -302,10 +404,11 @@ InputDestroy(Widget aw)
 {
     XmTextWidget w = (XmTextWidget)aw;
 
-    DEBUGOUT(XdbDebug(__FILE__, aw, "InputDestroy"));
+    DEBUGOUT(XdbDebug(__FILE__, aw, "InputDestroy\n"));
 
     (*Text_Source(w)->RemoveWidget) (Text_Source(w), w);
 
+    XtFree((XtPointer)In_SelArray(Text_Input(w)->data) );
     XtFree((char *)Text_Input(w));
 }
 
@@ -347,15 +450,25 @@ _XmTextInputCreate(Widget aw, ArgList args, Cardinal num_args)
     if (!Text_Source(w))
     {
 	Text_Source(w) = _XmStringSourceCreate(Text_Value(w), False);
+	Text_Source(w) -> data -> editable = Text_Editable(w);
     }
 
     (*Text_Source(w)->AddWidget) (Text_Source(w), w);
 
-    In_SelArray(i) = NULL;
+      /* CP :Thursday 13 May 1999
+	FIXME : is this correct for version 2.x ? */
+    In_SelArray(i) = (XmTextScanType*)XtMalloc ( sizeof(XmTextScanType) * 4 );
+    In_SelArray(i)[0] = XmSELECT_POSITION;
+    In_SelArray(i)[1] = XmSELECT_WORD;
+    In_SelArray(i)[2] = XmSELECT_LINE;
+    In_SelArray(i)[3] = XmSELECT_ALL;
+    In_SelArrayCount(i) = 4;
     In_SelArrayIndex(i) = 0;
 
     In_LastTime(i) = 0;
     In_HighlightStart(i) = 0;
+    i->extendDir = XmsdRight;
+    i->cancel = True;
 
     DEBUGOUT(XdbDebug(__FILE__, aw,
 		      "_XmTextInputCreate (selectionArrayCount %d)\n",
@@ -366,8 +479,6 @@ _XmTextInputCreate(Widget aw, ArgList args, Cardinal num_args)
 
 
 /* Scan type utilities ---------------------------------------------------
-
- * FIX ME - XmNselectionArray should be used.
  */
 static XmTextScanType
 ScanType(XmTextWidget w)
@@ -521,8 +632,10 @@ _XmTextDelete(XmTextWidget w, XEvent *ev, XmTextPosition start, XmTextPosition e
 	     * source.
 	     * Danny 8/5/1997
 	     */
+/*CP:17 May 1999: this does not seem to be needed
 	    _XmTextUpdateLineTable((Widget)w, cbs.startPos,
 				   cbs.endPos, &blockrec, True);
+*/
 	    _XmTextSetCursorPosition(aw, cbs.newInsert);
 	}
     }
@@ -659,9 +772,9 @@ DoScanType(XmTextWidget w, XEvent *ev, XmTextPosition pos)
 
     case XmSELECT_WORD:
 	right = (*Text_Source(w)->Scan) (Text_Source(w), pos, st,
-					 XmsdRight, -1, False);
+					 XmsdRight, 1, False);
 	left = (*Text_Source(w)->Scan) (Text_Source(w), right, st,
-					XmsdLeft, -1, False);
+					XmsdLeft, 1, False);
         In_HighlightPivot(i) = left;
 
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
@@ -674,16 +787,16 @@ DoScanType(XmTextWidget w, XEvent *ev, XmTextPosition pos)
 
     case XmSELECT_LINE:
 	right = (*Text_Source(w)->Scan) (Text_Source(w), pos, st,
-					 XmsdRight, -1, False);
+					 XmsdRight, 1, False);
 	left = (*Text_Source(w)->Scan) (Text_Source(w), right, st,
-					XmsdLeft, -1, False);
+					XmsdLeft, 1, False);
         In_HighlightPivot(i) = left;
 
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 		       "DoScanType: XmSELECT_LINE (left %d pos %d right %d)\n",
 			  left, pos, right));
 
-	XmTextSetSelection((Widget)w, left, right, ev->xbutton.time);
+	XmTextSetSelection((Widget)w, left, right + 1, ev->xbutton.time);
 	break;
 
 #if	XmVERSION > 1
@@ -751,7 +864,7 @@ Activate(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 static void
 MoveBackwardChar(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "MoveBackwardChar"));
+    DEBUGOUT(XdbDebug(__FILE__, w, "MoveBackwardChar\n"));
 
     if (Text_CursorPos(w) > 0)
     {
@@ -772,10 +885,10 @@ MoveBackwardParagraph(Widget w, XEvent *ev,
 	XmTextPosition pos;
 
 	pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
-				     XmSELECT_WHITESPACE, XmsdLeft, -1, False);
+				     XmSELECT_WHITESPACE, XmsdLeft, 1, False);
 
 	pos = (*Text_Source(w)->Scan) (Text_Source(w), pos,
-				       XmSELECT_PARAGRAPH, XmsdLeft, -1, False);
+				       XmSELECT_PARAGRAPH, XmsdLeft, 1, False);
 
 	(*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
 					       Text_CursorPos(w), off);
@@ -794,10 +907,10 @@ MoveBackwardWord(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	XmTextPosition pos;
 
 	pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
-				     XmSELECT_WHITESPACE, XmsdLeft, -1, False);
+				     XmSELECT_WHITESPACE, XmsdLeft, 1, False);
 
 	pos = (*Text_Source(w)->Scan) (Text_Source(w), pos,
-				       XmSELECT_WORD, XmsdLeft, -1, False);
+				       XmSELECT_WORD, XmsdLeft, 1, False);
 
 	(*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
 					       Text_CursorPos(w), off);
@@ -835,7 +948,7 @@ MoveToLineStart(Widget w, XEvent *ev, String *params,
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveToLineStart"));
 
     pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
-				   XmSELECT_LINE, XmsdLeft, -1, False);
+				   XmSELECT_LINE, XmsdLeft, 1, False);
 
     (*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
 					   Text_CursorPos(w), off);
@@ -878,27 +991,43 @@ _XmTextGetSelection(Widget w, XtPointer client, Atom *selection, Atom *type,
 
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmTextGetSelection\n"));
 
-    if (*len == 0)
+    if ((!value) || (*len == 0))
     {
-	return;
+    	if (*selection == XA_SECONDARY)
+    	{
+	    XtGetSelectionValue(w, XA_PRIMARY, XA_STRING, _XmTextGetSelection,
+			(XtPointer)ev, ev->xbutton.time);
+    	}
+    	else
+    	{
+	    XtFree(client);
+    	}
     }
-
-    if (value == NULL)
+    else
     {
-	return;
-    }
-
-    if (*type == XA_STRING)
-    {
-	char *s = (char *)value;
-
-	if (s)
+	if (*type == XA_STRING)
 	{
-	    DoInsert((XmTextWidget)w, ev, s, strlen(s));
+	    char *s = (char *)value;
+
+	    if (s)
+	    {
+		if (*selection == XA_SECONDARY)
+		{
+		    DoInsert((XmTextWidget)w, ev, s, *len);
+		}
+		else
+		{
+		XmTextPosition pos;
+
+		    pos = (*Text_Output(w)->XYToPos)((XmTextWidget)w,
+		    			ev->xbutton.x, ev->xbutton.y);
+		    XmTextSetCursorPosition(w, pos);
+		    DoInsert((XmTextWidget)w, ev, s, *len);
+		}
+	    }
 	}
-#if 0
-	XtFree(s);
-#endif
+	XtFree(value);
+	XtFree(client);
     }
 }
 
@@ -907,11 +1036,15 @@ ProcessCopy(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "ProcessCopy\n"));
 
-    if (! Text_Editable(w))
-	return;
+    if (Text_Editable(w))
+    {
+    XEvent *nev;
 
-    XtGetSelectionValue(w, XA_PRIMARY, XA_STRING, _XmTextGetSelection,
-			(XtPointer)ev, ev->xbutton.time);
+    	nev = XtNew(XEvent);
+    	*nev = *ev;
+	XtGetSelectionValue(w, XA_SECONDARY, XA_STRING, _XmTextGetSelection,
+			(XtPointer)nev, ev->xbutton.time);
+    }
 }
 
 static void
@@ -1030,10 +1163,10 @@ DeleteForwardWord(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     XmTextPosition pos;
 
         pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
-                                       XmSELECT_WORD, XmsdRight, -1, False);
+                                       XmSELECT_WORD, XmsdRight, 1, False);
 
         pos = (*Text_Source(w)->Scan) (Text_Source(w), pos,
-                                       XmSELECT_WHITESPACE, XmsdRight, -1, False);
+                                       XmSELECT_WHITESPACE, XmsdRight, 1, False);
         _XmTextDelete((XmTextWidget)w, ev,Text_CursorPos(w),pos);
     }
 }
@@ -1063,10 +1196,10 @@ DeleteBackwardWord(Widget w, XEvent *ev, String *params, Cardinal *num_params)
         XmTextPosition pos;
 
         pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
-                                     XmSELECT_WHITESPACE, XmsdLeft, -1, False);
+                                     XmSELECT_WHITESPACE, XmsdLeft, 1, False);
 
         pos = (*Text_Source(w)->Scan) (Text_Source(w), pos,
-                                       XmSELECT_WORD, XmsdLeft, -1, False);
+                                       XmSELECT_WORD, XmsdLeft, 1, False);
 
         _XmTextDelete((XmTextWidget)w,ev,pos,Text_CursorPos(w));
     }
@@ -1108,7 +1241,7 @@ DeleteToEndOfLine(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     }
 
     pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
-				   XmSELECT_LINE, XmsdRight, -1, False);
+				   XmSELECT_LINE, XmsdRight, 1, False);
     _XmTextDelete((XmTextWidget)w,ev,Text_CursorPos(w),pos);
 }
 
@@ -1125,7 +1258,7 @@ DeleteToStartOfLine(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     }
 
     pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
-				   XmSELECT_LINE, XmsdLeft, -1, False);
+				   XmSELECT_LINE, XmsdLeft, 1, False);
     _XmTextDelete((XmTextWidget)w,ev,pos,Text_CursorPos(w));
 }
 
@@ -1145,7 +1278,7 @@ MoveToLineEnd(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveToLineEnd"));
 
     pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
-				   XmSELECT_LINE, XmsdRight, -1, False);
+				   XmSELECT_LINE, XmsdRight, 1, False);
     (*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
 					   Text_CursorPos(w), off);
     _XmTextSetCursorPosition(w, pos);
@@ -1249,7 +1382,7 @@ ExtendSelection(Widget w, XEvent *ev, String *params, Cardinal *num_params)
         else
         {
             right = (*Text_Source(w)->Scan)(Text_Source(w), pos, st,
-                                            XmsdRight, -1, False);
+                                            XmsdRight, 1, False);
             left = In_HighlightPivot(i);
         }
         XmTextSetCursorPosition(w,right);
@@ -1264,9 +1397,9 @@ ExtendSelection(Widget w, XEvent *ev, String *params, Cardinal *num_params)
         else
         {
             left = (*Text_Source(w)->Scan)(Text_Source(w), pos, st,
-                                           XmsdLeft, -1, False);
+                                           XmsdLeft, 1, False);
             right = (*Text_Source(w)->Scan)(Text_Source(w),In_HighlightPivot(i),
-                                            st, XmsdRight, -1, False);
+                                            st, XmsdRight, 1, False);
         }
         XmTextSetCursorPosition(w,left);
     }
@@ -1284,7 +1417,7 @@ FindWord(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 static void
 MoveForwardChar(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "MoveForwardChar"));
+    DEBUGOUT(XdbDebug(__FILE__, w, "MoveForwardChar\n"));
 
     if (Text_CursorPos(w) < Text_LastPos(w))
     {
@@ -1303,10 +1436,10 @@ MoveForwardParagraph(Widget w, XEvent *ev, String *params,
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveForwardParagraph"));
 
     pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
-				   XmSELECT_PARAGRAPH, XmsdRight, -1, False);
+				   XmSELECT_PARAGRAPH, XmsdRight, 1, False);
 
     pos = (*Text_Source(w)->Scan) (Text_Source(w), pos,
-				   XmSELECT_WHITESPACE, XmsdRight, -1, False);
+				   XmSELECT_WHITESPACE, XmsdRight, 1, False);
 
     (*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
 					   Text_CursorPos(w), off);
@@ -1324,10 +1457,10 @@ MoveForwardWord(Widget w, XEvent *ev, String *params,
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveForwardWord"));
 
     pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
-				   XmSELECT_WORD, XmsdRight, -1, False);
+				   XmSELECT_WORD, XmsdRight, 1, False);
 
     pos = (*Text_Source(w)->Scan) (Text_Source(w), pos,
-				   XmSELECT_WHITESPACE, XmsdRight, -1, False);
+				   XmSELECT_WHITESPACE, XmsdRight, 1, False);
 
     (*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
 					   Text_CursorPos(w), off);
@@ -1368,6 +1501,8 @@ static void
 InsertString(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "InsertString\n"));
+    if ( *num_params == 1 )
+           XmTextInsert ( w, XmTextGetCursorPosition(w), params[0] );
 }
 
 static void
@@ -1638,6 +1773,7 @@ KillToStartOfLine(Widget w, XEvent *ev, String *params,
 		  Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "KillToStartOfLine\n"));
+    DeleteToStartOfLine(w, ev, params, num_params);
 }
 
 static void
@@ -1674,7 +1810,7 @@ MoveNextLine(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
     int index, maxindex;
     XmTextPosition pos = Text_CursorPos(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, aw, "MoveNextLine"));
+    DEBUGOUT(XdbDebug(__FILE__, aw, "MoveNextLine\n"));
 
     index = _XmTextGetTableIndex(w, pos);
     if (index == (maxindex = Text_TotalLines(w) - 1))
@@ -1700,9 +1836,8 @@ MoveNextLine(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
     if (pos != Text_CursorPos(w))
     {
 	(*Text_Output(w)->DrawInsertionPoint) (w, Text_CursorPos(w), off);
+	_XmTextSetCursorPosition((Widget)w, pos);
     }
-
-    _XmTextSetCursorPosition((Widget)w, pos);
 }
 
 
@@ -1712,7 +1847,11 @@ MoveNextPage(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
     XmTextWidget w = (XmTextWidget)aw;
     int index, maxindex;
     XmTextPosition pos = Text_CursorPos(w);
+#ifndef BUG45
+    int delta = Out_Rows(Text_OutputData(w));
+#else
     int delta = Text_LineCount(w) - 1;
+#endif
 
     DEBUGOUT(XdbDebug(__FILE__, aw, "MoveNextPage"));
 
@@ -1789,8 +1928,23 @@ InsertNewLineAndIndent(Widget w, XEvent *ev, String *params,
 	/* if we can't edit, or we are in single line only mode. */
 	return;
     }
+    else
+    {
+       XmTextBlockRec  block;
+       XmTextPosition beginingOfLine, right;
+       beginingOfLine = (*Text_Source(w)->Scan) (Text_Source(w),
+                               Text_CursorPos(w), XmSELECT_LINE,  XmsdLeft, 1, False);
+       right = (*Text_Source(w)->Scan) (Text_Source(w), beginingOfLine,
+                               XmSELECT_WHITESPACE, XmsdRight,
+                               Text_CursorPos(w) - beginingOfLine, /* so we do not select more
+                                                                      whitespace than just this line */
+                               False);
 
-    DoInsert((XmTextWidget)w, ev, "\n", 1);
+               /* passing "beginingOfLine - 1" should mean we also get the newline :-) */
+       (*Text_Source(w)->ReadSource) (Text_Source(w), beginingOfLine - 1, right, &block);
+       DoInsert((XmTextWidget)w, ev, block.ptr , block.length );
+       XtFree(block.ptr);
+    }
 }
 
 static void
@@ -1828,7 +1982,7 @@ MovePreviousLine(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
     unsigned int index;
     XmTextPosition pos = Text_CursorPos(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, aw, "MovePreviousLine"));
+    DEBUGOUT(XdbDebug(__FILE__, aw, "MovePreviousLine\n"));
 
     index = _XmTextGetTableIndex(w, pos);
     if (index == 0)
@@ -1860,7 +2014,11 @@ MovePreviousPage(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
     XmTextWidget w = (XmTextWidget)aw;
     unsigned int index;
     XmTextPosition pos = Text_CursorPos(w);
+#ifndef BUG45
+    int delta = Out_Rows(Text_OutputData(w));
+#else
     int delta = Text_LineCount(w) - 1;
+#endif
 
     DEBUGOUT(XdbDebug(__FILE__, aw, "MovePreviousPage"));
 
@@ -1950,6 +2108,7 @@ static void
 ScrollOneLineUp(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "ScrollOneLineUp\n"));
+    XmTextScroll(w, -1);
 }
 
 static void
@@ -1964,6 +2123,11 @@ SecondaryNotify(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     DEBUGOUT(XdbDebug(__FILE__, w, "SecondaryNotify\n"));
 }
 
+static void
+SecondaryStart(Widget w, XEvent *ev, String *params, Cardinal *num_params)
+{
+    DEBUGOUT(XdbDebug(__FILE__, w, "SecondaryStart\n"));
+}
 
 static void
 SelectAll(Widget w, XEvent *ev, String *params, Cardinal *num_params)
@@ -2052,16 +2216,33 @@ SetSelectionHint(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     DEBUGOUT(XdbDebug(__FILE__, w, "SetSelectionHint\n"));
 }
 
+static void
+ScrollCursorVertically(Widget w, XEvent *ev, String *params, Cardinal *num_params)
+{
+    DEBUGOUT(XdbDebug(__FILE__, w, "ScrollCursorVertically\n"));
+}
+
+
 static void
 ScrollOneLineDown(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "ScrollOneLineDown\n"));
+    XmTextScroll(w, 1);
 }
 
 static void
 ToggleAddMode(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "ToggleAddMode\n"));
+    Text_AddMode(w) = ! Text_AddMode(w);
+}
+
+
+static void
+ToggleOverstrike(Widget w, XEvent *ev, String *params, Cardinal *num_params)
+{
+    DEBUGOUT(XdbDebug(__FILE__, w, "ToggleOverstrike\n"));
+    Text_InputData(w) -> overstrike = ! Text_InputData(w) -> overstrike ;
 }
 
 static void
@@ -2137,7 +2318,7 @@ TextFocusOut(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     XmTextVerifyCallbackStruct cbs;
     OutputData o = Text_OutputData(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "TextFocusOut"));
+    DEBUGOUT(XdbDebug(__FILE__, w, "TextFocusOut\n"));
 
     if (Text_Editable(w))
     {
@@ -2227,8 +2408,19 @@ _XmTextSetSel2(XmTextWidget tw,
 	       XmTextPosition right,
 	       Time set_time)
 {
-    /* FIX ME */
-    return False;
+    /*CP: first attempt */
+    InputData inp = Text_InputData(tw);
+
+    if ( left < 0 ) 
+		/*CP: it seems Motif does not do this check ! 
+		|| right > Text_LastPos(tw) ) */
+	return False;
+
+    inp -> sel2Left = left;
+    inp -> sel2Right = right;
+    inp -> hasSel2 = True;
+    inp -> sec_time = set_time;
+    return True;
 }
 
 Boolean
@@ -2236,6 +2428,11 @@ _XmTextGetSel2(XmTextWidget tw,
 	       XmTextPosition *left,
 	       XmTextPosition *right)
 {
-    /* FIX ME */
-    return False;
+    InputData inp = Text_InputData(tw);
+    if ( ! inp -> hasSel2 )
+	return False;
+
+    *left = inp -> sel2Left ;
+    *right = inp -> sel2Right ;
+    return True;
 }
