@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextStrSo.c,v 1.19 1998/08/24 21:37:02 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextStrSo.c,v 1.38 2000/01/09 22:34:54 hennessy Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: TextStrSo.c,v 1.19 1998/08/24 21:37:02 rwscott Exp $";
+static const char rcsid[] = "$Id: TextStrSo.c,v 1.38 2000/01/09 22:34:54 hennessy Exp $";
 
 /*
  * Empirical evidence (fooling around with the structures found in Motif
@@ -119,12 +119,13 @@ AddWidget(XmTextSource source, XmTextWidget w)
 	else
 	{
 	    d->value = XtNewString(d->value);
+/*CP:wrong - the "old_length" is the lengh in chars of the longest line
 	    d->old_length = (d->value) ? strlen(d->value) : 0;
+*/
 	}
-
-	d->maxlength = d->old_length + SOURCE_ALLOC_SIZE;
+	d->length = (d->value) ? strlen(d->value) : 0;
+	d->maxlength = d->length + SOURCE_ALLOC_SIZE;
 	d->ptr = XtMalloc(d->maxlength + 1);
-	d->length = d->old_length;
 	strcpy(d->ptr, d->value);
 
 	d->gap_start = d->ptr;
@@ -188,10 +189,11 @@ CountLines(XmTextSource source, XmTextPosition start, unsigned long len)
 
     d = source->data;
     ptr = StrPtr(d, start);
-    count = 1;
+
+    count = 0;
     while (len-- > 0)
     {
-	if (*ptr == '\n')
+	if (ptr[len] == '\n')
 	{
 	    count++;
 	}
@@ -241,7 +243,13 @@ ReadSource(XmTextSource source,
 
     block->ptr[len] = '\0';
 
-    return pos;
+    if ( d->old_length < ( last - pos ) )
+    	{
+    	if ( ( pos == 0 || d->ptr[pos-1] == '\n' ) &&
+    		( last == d->length || d->ptr[last] == '\n' ) && len > d->old_length )
+	    d->old_length = len;
+	}
+    return last;
 }
 
 /* Replace ------------------------------------------------------------------
@@ -342,7 +350,8 @@ Replace(XmTextWidget w,
 	return EditError;
     }
 
-    if (d->length + (end - start) + block->length > d->maxallowed)
+    if ( ( start >= end ) &&
+               d->length + (end - start) + block->length > d->maxallowed)
     {
 	return EditError;
     }
@@ -393,13 +402,45 @@ Replace(XmTextWidget w,
 	}
     }
 
-    /* Simplistic treatment of selection (FIX ME)
-     * What this does is eliminate the selection once the text is edited. */
-    d->hasselection = False;
-    for (i = 0; i < d->numwidgets; i++)
-    {
-	Text_Highlight(d->widgets[i]).number = 0;
-    }
+   if ( d -> hasselection )
+	{
+	if ( start == d->left && end == d-> right )
+	    {
+		/* CP:15 May 1999: If the text we are replacing is the
+		** selected text then unselect it. */
+	    d->hasselection = False;
+	    for (i = 0; i < d->numwidgets; i++)
+	      {
+#if 0
+		_XmHighlightRecNode* p;
+		p = Text_Highlight(d->widgets[i]).list;
+		while (p) {
+		  _XmHighlightRecNode* k=p;
+		  p = p->next;
+		  XtFree((XtPointer)k);
+		}
+		Text_Highlight(d->widgets[i]).list = NULL;
+#else
+		Text_Highlight(d->widgets[i]).number = 0;
+#endif
+	      }
+	    }
+	/*CP:15 May 1999:
+		at this point we do not want to overwrite the selection
+		but just to move its coordinates if we are inserting
+		text before the selection. */
+
+	else if ( start <  d-> left )
+		{
+		d-> left += block -> length;
+		d-> right += block -> length;
+		for (i = 0; i < d->numwidgets; i++)
+			{
+			XmTextSetHighlight((Widget)d->widgets[i],
+				d-> left, d->right, XmHIGHLIGHT_SELECTED);
+			}
+		}
+	}
     /* End selection */
 
     for (i = 0; i < d->numwidgets; i++)
@@ -413,6 +454,22 @@ Replace(XmTextWidget w,
 			  block->length - end + start);
     }
 
+#if 0
+    /* Update the insertion position */
+    for (i = 0; i < d->numwidgets; i++)
+    {
+	XmTextPosition	p = Text_CursorPos(d->widgets[i]);
+
+	if (p < *startret) {
+		/* no action */
+	} else if (p < *endret) {
+		_XmTextSetCursorPosition(d->widgets[i], *startret);
+	} else {
+		_XmTextSetCursorPosition(d->widgets[i], p + block->length);
+	}
+    }
+#endif
+
     for (i = 0; i < d->numwidgets; i++)
     {
 	_XmTextEnableRedisplay(d->widgets[i]);
@@ -448,8 +505,8 @@ Scan(XmTextSource source, XmTextPosition pos, XmTextScanType type,
 
     if (pos > d->length)
     {
-	return(pos);
 	pos = d->length;
+	return(pos);
     }
 
     if (count < 0)
@@ -470,7 +527,6 @@ Scan(XmTextSource source, XmTextPosition pos, XmTextScanType type,
 	ptr = StrPtr(d, pos - 1);
 	while (count > 0 && pos > 0 && InWhiteSpace(*ptr))
 	{
-	    count--;
 	    pos--;
 	    ptr = StrPtr(d, pos - 1);
 	}
@@ -480,7 +536,6 @@ Scan(XmTextSource source, XmTextPosition pos, XmTextScanType type,
 	ptr = StrPtr(d, pos);
 	while (count > 0 && pos < d->length && InWhiteSpace(*ptr))
 	{
-	    count--;
 	    pos++;
 	    ptr = StrPtr(d, pos);
 	}
@@ -523,6 +578,12 @@ Scan(XmTextSource source, XmTextPosition pos, XmTextScanType type,
 		}
 		break;
 
+	    case XmSELECT_ALL:
+		/*CP: probably a bit too simple */
+		found = True;
+		pos = 0;
+		break;
+
 	    default:
 		found = True;
 		break;
@@ -534,11 +595,13 @@ Scan(XmTextSource source, XmTextPosition pos, XmTextScanType type,
 		DEBUGOUT(XdbDebug(__FILE__, (Widget)d->widgets[0],
 				  "Scan(%d)=>%d\n", ipos, pos));
 #endif
-		return pos;
+		count--;
+		if ( count == 0 )
+		    return ( inc ? pos - 1 : pos );
+		found = False;    
 	    }
 
 	    pos--;
-	    count--;
 	}
     }
     else
@@ -576,6 +639,13 @@ Scan(XmTextSource source, XmTextPosition pos, XmTextScanType type,
 		}
 		break;
 
+	    case XmSELECT_ALL:
+		if (! *ptr )
+		{
+		    found = True;
+		}
+		break;
+
 	    default:
 		found = True;
 		break;
@@ -587,11 +657,19 @@ Scan(XmTextSource source, XmTextPosition pos, XmTextScanType type,
 		DEBUGOUT(XdbDebug(__FILE__, (Widget)d->widgets[0],
 				  "Scan(%d)=>%d\n", ipos, pos));
 #endif
-		return pos;
+		count--;
+		if ( count == 0 )
+		    return ( inc ? pos+1 : pos );
+		found = False;    
+		if ( type == XmSELECT_WORD )
+		    while ( InWhiteSpace(*ptr ) )
+			{
+			pos++;
+			ptr = StrPtr(d, pos);
+			}
 	    }
 
 	    pos++;
-	    count--;
 	}
     }
 
@@ -599,6 +677,10 @@ Scan(XmTextSource source, XmTextPosition pos, XmTextScanType type,
     DEBUGOUT(XdbDebug(__FILE__, (Widget)d->widgets[0],
 		      "Scan(%d)=>%d\n", ipos, pos));
 #endif
+    if ( dir == XmsdRight && pos != d->length )
+	return ( inc ? pos+1 : pos );
+    if ( dir == XmsdLeft && pos != 0 )
+	return ( inc ? pos - 1 : pos );
     return pos;
 }
 
@@ -670,12 +752,25 @@ _XmTextConvertSelection(Widget w, Atom *selection, Atom *target, Atom *type,
 static void
 _XmTextLoseSelection(Widget w, Atom *selection)
 {
+    XmAnyCallbackStruct	cbs;
+    XmTextWidget	tw = (XmTextWidget)w;
+
     if (*selection == XA_PRIMARY)
     {
 	DEBUGOUT(XdbDebug(__FILE__, w, "_XmTextLoseSelection(PRIMARY)\n"));
 
 	XmTextSetHighlight(w, 0, XmTextGetLastPosition(w),
 			   XmHIGHLIGHT_NORMAL);
+
+	/*
+	 * FIX ME
+	 * Need to call losePrimaryCallback here ?
+	 */
+	cbs.reason = XmCR_LOSE_PRIMARY;
+	cbs.event = NULL;	/* ??? */
+	XtCallCallbackList(w,
+                           tw->text.lose_primary_callback,
+                           (XtPointer)&cbs);
     }
     else
     {
@@ -798,9 +893,13 @@ _XmStringSourceCreate(char *value, Boolean is_wchar)
     d->numwidgets = 0;
     d->value = value;
     d->ptr = d->gap_start = d->gap_end = NULL;
-    d->length = d->old_length = d->maxlength = d->maxallowed = 0;
+    d->length = d->maxlength = d->maxallowed = 0;
+    d->old_length = 1;
+    d->right = d->left = 0;
     d->hasselection = False;
-
+    d->prim_time = 0;
+	/*CP: How do I set the character used for new lines ?? : FIXME */
+    d->PSWC_NWLN= XtNewString("\n");
     source->data = d;
 
     return source;
@@ -818,14 +917,25 @@ _XmStringSourceDestroy(XmTextSource source)
 {
     XmSourceData d = source->data;
 
+    /* rws 1 Apr 1999
+       Even if these are == 0, we have still allocates space with XtNewString(0),
+       and therefore it must be freed.
+     */
+    /*
     if (d->old_length > 0)
+    */
     {
 	XtFree(d->value);
     }
+    /*
     if (d->length > 0)
+    */
     {
 	XtFree(d->ptr);
     }
+	/* tills 12 apr 1999 free the widget list also (memleak) */
+	XtFree((char*)d->widgets);
+    XtFree(d->PSWC_NWLN);
     XtFree((char *)d);
     XtFree((char *)source);
 }
@@ -858,37 +968,65 @@ _XmStringSourceGetValue(XmTextSource source, Boolean want_wchar)
 void
 _XmStringSourceSetValue(XmTextWidget w, char *value)
 {
-    XmTextSource source = Text_Source(w);
-    XmSourceData d = source->data;
+    XmTextVerifyCallbackStruct cbs;
+    XmTextBlockRec tb;
     int len;
-    XmAnyCallbackStruct cbs;
+
+    SetSelection(Text_Source(w),-1,-1,CurrentTime);
 
     len = value ? strlen(value) : 0;
-    d->length = 0;
-    d->gap_start = d->ptr;
-    d->gap_end = d->ptr;
-    CheckSize(d, len);
 
-    if (len)
+    cbs.reason = XmCR_MODIFYING_TEXT_VALUE;
+    cbs.doit = True;
+    cbs.event = NULL;
+    cbs.startPos = 0;
+    cbs.endPos = Text_Source(w)->data->length;
+    cbs.currInsert = Text_CursorPos(w);
+    cbs.newInsert = len;
+
+    if (value)
     {
-	Insert(d, 0, value, len);
+        tb.ptr = XtMalloc(len + 1);	/* NULL-terminate */
+        tb.length = len;
+        tb.format = XmFMT_8_BIT;
+        strncpy(tb.ptr, value, len);
+        tb.ptr[len] = '\0';             /* NULL-terminate */
     }
     else
     {
-	/* This may not be necessary.
-	   d->ptr may still contain something that confuses us later.
-	   Get rid of it.
-	 */
-	if (d->length && d->ptr)
-	    d->ptr[0] = '\0';
+	/* Motif sets ptr to NULL for delete, not to NULL string */
+        tb.ptr = NULL;
+        tb.length = 0;
+        tb.format = XmFMT_8_BIT;
     }
-
-    /* Need to call XmNvalueChangedCallback here; not possible a level lower. */
-    if (Text_ValueChangedCallback(w))
+    cbs.text = &tb;
+    if (Text_ModifyVerifyCallback(w))
     {
-	cbs.reason = XmCR_VALUE_CHANGED;
-	cbs.event = NULL;
-	XtCallCallbacks((Widget)w, XmNvalueChangedCallback, &cbs);
+        XtCallCallbacks((Widget)w, XmNmodifyVerifyCallback, &cbs);
+    }
+#if 0
+    /* FIXME ? */
+    /* SG 23/08/1998 this is wrong as the callback structure is NOT
+       the same for wide characters, given that the rest of wide character
+       support is missing - I think it preferable to skip this for now.
+     */
+    if (TextF_WcsModifyVerifyCallback(w))
+    {
+	XtCallCallbacks((Widget)w, XmNmodifyVerifyCallbackWcs, &cbs);
+    }
+#endif
+    if (cbs.doit)
+    {
+	Replace(w, NULL, &cbs.startPos, &cbs.endPos, &tb, True);
+    }
+#if 0
+    else {
+	VerifyBell(w);
+    }
+#endif
+    if (tb.ptr)
+    {
+	XtFree(tb.ptr);
     }
 }
 
