@@ -1,6 +1,6 @@
 /**
  *
- * $Id: VirtKeys.c,v 1.9 1998/09/12 15:05:50 rwscott Exp $
+ * $Id: VirtKeys.c,v 1.12 1999/11/13 14:10:34 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: VirtKeys.c,v 1.9 1998/09/12 15:05:50 rwscott Exp $";
+static const char rcsid[] = "$Id: VirtKeys.c,v 1.12 1999/11/13 14:10:34 rwscott Exp $";
 
 #include <LTconfig.h>
 
@@ -44,6 +44,7 @@ static const char rcsid[] = "$Id: VirtKeys.c,v 1.9 1998/09/12 15:05:50 rwscott E
 #include <Xm/TransltnsP.h>
 #include <X11/keysym.h>
 #include <X11/Xfuncs.h>
+#include <Xm/XmosP.h>
 
 #include <XmI/DebugUtil.h>
 
@@ -1024,7 +1025,7 @@ FindXmBindAliasBindings(Display *Dsp, String *bindingSpec,
 static void
 VirtKeysInitialize(Widget w)
 {
-    String HOME = getenv("HOME");
+    String HOME = _XmOSGetHomeDirName();	/* getenv("HOME"); */
     String bindingSpec = NULL;
     String motifbind, XMBINDDIR;
     XmDisplayRec *wd;
@@ -1345,6 +1346,9 @@ XmTranslateKey(Display *Dsp, KeyCode Keycode,
     DEBUGOUT(XdbDebug(__FILE__, NULL,
 	"XmTranslateKey (KeyCode %d, modifiers 0x%X), event %d\n",
 	Keycode, modifiers, d->display.lastKeyEvent->state));
+    DEBUGOUT(XdbDebug("NMEM", NULL,
+	"XmTranslateKey (KeyCode %d, modifiers 0x%X), event %d\n",
+	Keycode, modifiers, d->display.lastKeyEvent->state));
 
     XtTranslateKey(Dsp, Keycode, modifiers, modifiers_return, Keysym_return);
 
@@ -1386,6 +1390,7 @@ _XmVirtKeysHandler(Widget w, XtPointer Data, XEvent *Event,
     XmDisplay	d;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmVirtKeysHandler\n"));
+    DEBUGOUT(XdbDebug("NMEM", w, "_XmVirtKeysHandler\n"));
 
     if (!w->core.being_destroyed && (Event->xany.type == KeyPress))
     {
