@@ -1,6 +1,6 @@
 /*
 
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.32 1998/10/01 21:10:10 danny Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.45 1999/12/13 15:59:10 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.32 1998/10/01 21:10:10 danny Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.45 1999/12/13 15:59:10 rwscott Exp $";
 
 /*
  * ** Danny had added a flag here, but I think the behavior he added is always
@@ -83,6 +83,11 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,
 
 #define DEBUG	1
 
+/* For the prototype police */
+extern XmFontList XmStringCreateFontList( XFontStruct *font,
+					  XmStringCharSet charset);
+extern XmString XmStringConcatAndFree(XmString str1, XmString str2);
+
 /**************************** PRIVATE FUNCTIONS ****************************/
 #ifdef DEBUG
 static int
@@ -256,7 +261,7 @@ asn1_dump(unsigned char *string)
     }
 }
 
-void
+extern void
 _Xm_dump_fontlist(XmFontList fontlist)
 {
     int i;
@@ -272,19 +277,19 @@ _Xm_dump_fontlist(XmFontList fontlist)
     printf("\n");
 }
 
-void
+extern void
 _Xm_dump_fontlist_cache(void)
 {
 }
 
-void
+extern void
 _Xm_dump_external(XmString str)
 {
     asn1_dump(str);
     printf("\n");
 }
 
-void
+extern void
 _Xm_dump_internal(_XmString str)
 {
     int i;
@@ -930,6 +935,11 @@ __XmStringSegmentExtent(XmFontList flist, _XmStringComponent comp,
 
 	return False;
     }
+    if (!flist)
+    {
+	_XmWarning(NULL, "__XmStringSegmentExtent: got NULL FontList");
+	return False;
+    }
 
     switch (flist[comp->font].type)
     {
@@ -961,27 +971,7 @@ __XmStringSegmentExtent(XmFontList flist, _XmStringComponent comp,
 			comp->data, comp->length));
 	    }
 
-	    /*
-	     * MLM: I know I added this for a reason, but I can't remember why
-	     * right now.
-	     * Now I do.  It's because of nedit.  The height of a menu item is
-	     * is set to 0 (well, really 1); however, without this code,
-	     * XmCalcLabelDimensions returns a different value, even though
-	     * the true height of the label is 0.  Do not comment this code
-	     * (if part).
-	     */
-	    if (*width == 0)
-	    {
-		DEBUGOUT(XdbDebug(__FILE__, NULL,
-		   "_XmStringSegmentExtent: nedit hack, height (was %d) = 0\n",
-		    *height));
-
-		*height = 0;
-	    }
-	    else
-	    {
-		*height = amax + dmax;
-	    }
+	    *height = amax + dmax;
 	    *ascent = amax;
 	    *descent = dmax;
 	}
@@ -1059,23 +1049,21 @@ _XmFontListGetDefaultFont(XmFontList fontlist,
 	}
     }
 
+    /* Default to first entry */
     if (fontlist[i].tag == NULL)
-    {
-	*font_struct = NULL;
-	return False;
-    }
+	i = 0;
 
     if (fontlist[i].type == XmFONT_IS_FONT)
     {
        if ((*font_struct = (XFontStruct *)fontlist[i].font))
-         return True;
-         else return False; /* Codematic 11.10.1998 */
+	 return True;
+	 else return False; /* Codematic 11.10.1998 */
     }
     else if (fontlist[i].type == XmFONT_IS_FONTSET)
     {
        if ((*font_struct = _XmGetFirstFont(&fontlist[i])))
-         return True;
-         else return False; /* Codematic 11.10.1998 */
+	 return True;
+	 else return False; /* Codematic 11.10.1998 */
     }
 
     *font_struct = NULL;
@@ -1130,14 +1118,14 @@ _XmFontListSearch(XmFontList fontlist,
     if (fontlist[i].type == XmFONT_IS_FONT)
     {
        if ((*font_struct = (XFontStruct *)fontlist[i].font))
-        return True;
-         else return False; /* Codematic 11.10.1998 */
+	return True;
+	 else return False; /* Codematic 11.10.1998 */
     }
     else if (fontlist[i].type == XmFONT_IS_FONTSET)
     {
        if ((*font_struct = _XmGetFirstFont(&fontlist[i])))
-        return True;
-         else return False; /* Codematic 11.10.1998 */          
+	return True;
+	 else return False; /* Codematic 11.10.1998 */
     }
 
     *font_struct = NULL;
@@ -1604,7 +1592,7 @@ _XmStringUpdateWMShellTitle(XmString xmstr, Widget shell)
 
     if (freeit)
     {
-        XtFree(title);
+	XtFree(title);
     }
 }
 
@@ -1995,7 +1983,7 @@ _XmStringDraw(Display *d, Window w, XmFontList fontlist, _XmString string,
     _XmStringComponent comp;
     Position current_y;
     Position current_x = 0;
-    Boolean have_seg, have_line_height;
+    Boolean have_seg, have_line_height, clipped;
     Dimension default_line_height;
     XRectangle ink, log;
     int pending_newlines;
@@ -2004,9 +1992,14 @@ _XmStringDraw(Display *d, Window w, XmFontList fontlist, _XmString string,
     {
 	return;
     }
+    if (!fontlist)
+    {
+	_XmWarning(NULL, "_XmDrawString: got NULL FontList");
+	return;
+    }
 
     DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(d, w),
-		      "XmStringDraw x %d y %d wid %d\n",
+		      "_XmStringDraw x %d y %d wid %d\n",
 		      x, y, width));
 
     current_y = y;
@@ -2017,6 +2010,7 @@ _XmStringDraw(Display *d, Window w, XmFontList fontlist, _XmString string,
 
     pending_newlines = 0;
     have_line_height = False;
+    clipped = False;
     default_line_height = 0;
 
     /* repeat while there's at least one thing in the string */
@@ -2095,6 +2089,23 @@ _XmStringDraw(Display *d, Window w, XmFontList fontlist, _XmString string,
 	    current_y += line_ascent;
 	}
 
+	/* Set clipping if necessary, or perhaps disregard the line entirely */
+	if (clip)
+	{
+	    if (current_y + line_descent <= clip->y ||
+		current_y - line_ascent >= clip->y + clip->height)
+	    {
+		current_y += line_descent;
+		continue;
+	    }
+	    if (!clipped && (current_y - line_ascent < clip->y ||
+		current_y + line_descent > clip->y + clip->height))
+	    {
+		XSetClipRectangles(d, gc, 0, 0, clip, 1, YXBanded);
+		clipped = True;
+	    }
+	}
+
 	context->current_component = start;
 
 	/* now we handle the starting x of this line based on the alignment */
@@ -2129,50 +2140,74 @@ _XmStringDraw(Display *d, Window w, XmFontList fontlist, _XmString string,
 	    switch (fontlist[comp->font].type)
 	    {
 	    case XmFONT_IS_FONT:
+		seg_width =
+		    (((XFontStruct *)fontlist[comp->font].font)->min_byte1 |
+		     ((XFontStruct *)fontlist[comp->font].font)->max_byte1)
+		    ? XTextWidth16(((XFontStruct *)fontlist[comp->font].font),
+				   (XChar2b *)comp->data, comp->length >> 1)
+		    : XTextWidth(((XFontStruct *)fontlist[comp->font].font),
+				 comp->data, comp->length);
+		break;
+
+	    case XmFONT_IS_FONTSET:
+		XmbTextExtents((XFontSet)fontlist[comp->font].font,
+			       comp->data, comp->length, &ink, &log);
+		/* need to do something with the extents of the font set here. */
+		seg_width = log.width;
+		break;
+	    }
+
+	    /* Set clipping if necessary, or perhaps skip this segment */
+	    if (clip)
+	    {
+		if (current_x + seg_width <= clip->x ||
+		    current_x >= clip->x + clip->width)
 		{
-		    XFontStruct *fontstruct =
-		    (XFontStruct *)fontlist[comp->font].font;
-
-                   if (!fontstruct) break; 
-                    /* modified by Codematic 11.01.1998 */
-
-
-		    XSetFont(d, gc, fontstruct->fid);
-
-		    if (fontstruct->min_byte1 || fontstruct->max_byte1)
-		    {
-			XDrawString16(d, w, gc, current_x, current_y,
-				      (XChar2b *)comp->data, comp->length / 2);
-			current_x += XTextWidth16(fontstruct,
-				      (XChar2b *)comp->data, comp->length / 2);
-		    }
-		    else
-		    {
-			XDrawString(d, w, gc, current_x, current_y,
-				    comp->data, comp->length);
-			current_x += XTextWidth(fontstruct,
-						comp->data, comp->length);
-		    }
+		    current_x += seg_width;
+		    continue;
+		}
+		if (!clipped && (current_x < clip->x ||
+		    current_x + seg_width > clip->x + clip->width))
+		{
+		    XSetClipRectangles(d, gc, 0, 0, clip, 1, YXBanded);
+		    clipped = True;
+		}
+	    }
 
+	    switch (fontlist[comp->font].type)
+	    {
+	    case XmFONT_IS_FONT:
+		XSetFont(d, gc,
+			 ((XFontStruct *)fontlist[comp->font].font)->fid);
+		if (((XFontStruct *)fontlist[comp->font].font)->min_byte1 |
+		    ((XFontStruct *)fontlist[comp->font].font)->max_byte1)
+		{
+		    XDrawString16(d, w, gc, current_x, current_y,
+				  (XChar2b *)comp->data, comp->length >> 1);
+		}
+		else
+		{
+		    XDrawString(d, w, gc, current_x, current_y,
+				comp->data, comp->length);
 		}
 		break;
 
 	    case XmFONT_IS_FONTSET:
 		XmbDrawString(d, w, (XFontSet)fontlist[comp->font].font, gc,
 			      current_x, current_y, comp->data, comp->length);
-
-		XmbTextExtents((XFontSet)fontlist[comp->font].font,
-			       comp->data, comp->length, &ink, &log);
-
-		/* need to do something with the extents of the font set here. */
-		current_x += log.width;
 		break;
 	    }
+
+	    current_x += seg_width;
 	}
 
 	current_y += line_descent;
     }
     _XmStringFreeContext(context);
+    if (clipped)
+    {
+	XSetClipMask(d, gc, None);
+    }
 }
 
 void
@@ -2186,7 +2221,7 @@ _XmStringDrawImage(Display *d, Window w,
     _XmStringComponent comp;
     Position current_y;
     Position current_x = 0;
-    Boolean have_seg, have_line_height;
+    Boolean have_seg, have_line_height, clipped;
     Dimension default_line_height;
     XRectangle ink, log;
     int pending_newlines;
@@ -2199,6 +2234,7 @@ _XmStringDrawImage(Display *d, Window w,
 
     pending_newlines = 0;
     have_line_height = False;
+    clipped = False;
     default_line_height = 0;
 
     /* repeat while there's at least one thing in the string */
@@ -2279,6 +2315,23 @@ _XmStringDrawImage(Display *d, Window w,
 	    current_y += line_ascent;
 	}
 
+	/* Set clipping if necessary, or perhaps disregard the line entirely */
+	if (clip)
+	{
+	    if (current_y + line_descent <= clip->y ||
+		current_y - line_ascent >= clip->y + clip->height)
+	    {
+		current_y += line_descent;
+		continue;
+	    }
+	    if (!clipped && (current_y - line_ascent < clip->y ||
+		current_y + line_descent > clip->y + clip->height))
+	    {
+		XSetClipRectangles(d, gc, 0, 0, clip, 1, YXBanded);
+		clipped = True;
+	    }
+	}
+
 	context->current_component = start;
 
 	/* now we handle the starting x of this line based on the alignment */
@@ -2313,56 +2366,77 @@ _XmStringDrawImage(Display *d, Window w,
 	    switch (fontlist[comp->font].type)
 	    {
 	    case XmFONT_IS_FONT:
-		{
-		    XFontStruct *fontstruct =
-		    ((XFontStruct *)fontlist[comp->font].font);
-
-                   if (!fontstruct) break; 
-                    /* modified by Codematic 11.01.1998 */
-
-		    XSetFont(d, gc, fontstruct->fid);
-
-
-		    if (fontstruct->min_byte1 || fontstruct->max_byte1)
-		    {
-			XDrawImageString16(d, w, gc, current_x, current_y,
-				      (XChar2b *)comp->data, comp->length / 2);
-			current_x += XTextWidth16(fontstruct,
-				      (XChar2b *)comp->data, comp->length / 2);
-		    }
-		    else
-		    {
-			XDrawImageString(d, w, gc, current_x, current_y,
-					 comp->data, comp->length);
-			current_x += XTextWidth(fontstruct,
-						comp->data, comp->length);
-		    }
-
-		}
+		seg_width =
+		    (((XFontStruct *)fontlist[comp->font].font)->min_byte1 |
+		     ((XFontStruct *)fontlist[comp->font].font)->max_byte1)
+		    ? XTextWidth16(((XFontStruct *)fontlist[comp->font].font),
+				   (XChar2b *)comp->data, comp->length >> 1)
+		    : XTextWidth(((XFontStruct *)fontlist[comp->font].font),
+				 comp->data, comp->length);
 		break;
 
+	    case XmFONT_IS_FONTSET:
+		XmbTextExtents((XFontSet)fontlist[comp->font].font,
+			       comp->data, comp->length, &ink, &log);
+		/* need to do something with the extents of the font set here. */
+		seg_width = log.width;
+		break;
+	    }
+
+	    /* Set clipping if necessary, or perhaps skip this segment */
+	    if (clip)
+	    {
+		if (current_x + seg_width <= clip->x ||
+		    current_x >= clip->x + clip->width)
+		{
+		    current_x += seg_width;
+		    continue;
+		}
+		if (!clipped && (current_x < clip->x ||
+		    current_x + seg_width > clip->x + clip->width))
+		{
+		    XSetClipRectangles(d, gc, 0, 0, clip, 1, YXBanded);
+		    clipped = True;
+		}
+	    }
+
+	    switch (fontlist[comp->font].type)
+	    {
+	    case XmFONT_IS_FONT:
+		XSetFont(d, gc,
+			 ((XFontStruct *)fontlist[comp->font].font)->fid);
+		if (((XFontStruct *)fontlist[comp->font].font)->min_byte1 |
+		    ((XFontStruct *)fontlist[comp->font].font)->max_byte1)
+		{
+		    XDrawImageString16(d, w, gc, current_x, current_y,
+				  (XChar2b *)comp->data, comp->length >> 1);
+		}
+		else
+		{
+		    XDrawImageString(d, w, gc, current_x, current_y,
+				     comp->data, comp->length);
+ 		}
+ 		break;
+
 	    case XmFONT_IS_FONTSET:
 		XmbDrawImageString(d, w, (XFontSet)fontlist[comp->font].font,
 				   gc, current_x, current_y,
 				   comp->data, comp->length);
-
-		XmbTextExtents((XFontSet)fontlist[comp->font].font,
-			       comp->data, comp->length, &ink, &log);
-
-		/* need to do something with the extents of the font set here. */
-                current_x += log.width;
 		break;
 	    }
+
+	    current_x += seg_width;
 	}
 
 	current_y += line_descent;
     }
     _XmStringFreeContext(context);
+    if (clipped)
+    {
+	XSetClipMask(d, gc, None);
+    }
 }
 
-
-
-
 void
 _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
 		       _XmString string, GC gc,
@@ -2376,7 +2450,7 @@ _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
     Position current_x = 0;
     int xo;
     char *p, *u;
-    Boolean have_seg, have_line_height;
+    Boolean have_seg, have_line_height, clipped;
     Dimension default_line_height;
     XRectangle ink, log;
     int pending_newlines;
@@ -2425,6 +2499,7 @@ _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
 
     pending_newlines = 0;
     have_line_height = False;
+    clipped = False;
     default_line_height = 0;
 
     /* repeat while there's at least one thing in the string */
@@ -2505,6 +2580,23 @@ _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
 	    current_y += line_ascent;
 	}
 
+	/* Set clipping if necessary, or perhaps disregard the line entirely */
+	if (clip)
+	{
+	    if (current_y + line_descent < clip->y ||
+		current_y - line_ascent >= clip->y + clip->height)
+	    {
+		current_y += line_descent;
+		continue;
+	    }
+	    if (!clipped && (current_y - line_ascent < clip->y ||
+		current_y + line_descent >= clip->y + clip->height))
+	    {
+		XSetClipRectangles(d, gc, 0, 0, clip, 1, YXBanded);
+		clipped = True;
+	    }
+	}
+
 	context->current_component = start;
 
 	/* now we handle the starting x of this line based on the alignment */
@@ -2536,20 +2628,57 @@ _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
 		continue;
 	    }
 
+	    switch (fontlist[comp->font].type)
+	    {
+	    case XmFONT_IS_FONT:
+		seg_width =
+		    (((XFontStruct *)fontlist[comp->font].font)->min_byte1 |
+		     ((XFontStruct *)fontlist[comp->font].font)->max_byte1)
+		    ? XTextWidth16(((XFontStruct *)fontlist[comp->font].font),
+				   (XChar2b *)comp->data, comp->length >> 1)
+		    : XTextWidth(((XFontStruct *)fontlist[comp->font].font),
+				 comp->data, comp->length);
+		break;
+
+	    case XmFONT_IS_FONTSET:
+		XmbTextExtents((XFontSet)fontlist[comp->font].font,
+			       comp->data, comp->length, &ink, &log);
+		/* need to do something with the extents of the font set here. */
+		seg_width = log.width;
+		break;
+	    }
+
+	    /* Set clipping if necessary, or perhaps skip this segment */
+	    if (clip)
+	    {
+		if (current_x + seg_width <= clip->x ||
+		    current_x >= clip->x + clip->width)
+		{
+		    current_x += seg_width;
+		    continue;
+		}
+		if (!clipped && (current_x < clip->x ||
+		    current_x + seg_width > clip->x + clip->width))
+		{
+		    XSetClipRectangles(d, gc, 0, 0, clip, 1, YXBanded);
+		    clipped = True;
+		}
+	    }
+
 	    switch (fontlist[comp->font].type)
 	    {
 	    case XmFONT_IS_FONT:
 		{
 		    XFontStruct *fontstruct =
 		    (XFontStruct *)fontlist[comp->font].font;
-                   
-                   Boolean is_wide;
-                   
-                   if (!fontstruct) break; 
-                    /* modified by Codematic 11.01.1998 */                 
-                   
-                   is_wide =
-                    fontstruct->min_byte1 || fontstruct->max_byte1;
+
+		   Boolean is_wide;
+
+		   if (!fontstruct) break;
+		    /* modified by Codematic 11.01.1998 */
+
+		   is_wide =
+		    fontstruct->min_byte1 || fontstruct->max_byte1;
 
 		    XSetFont(d, gc, fontstruct->fid);
 
@@ -2569,7 +2698,6 @@ _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
 
 		    if (u && (p = strstr(comp->data, u)))
 		    {		/* Underline */
-
 			int ww, llen = p - comp->data;
 			char *f = XtMalloc(llen + 1);
 
@@ -2597,7 +2725,6 @@ _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
 			    ww = XTextWidth(fontstruct, u, strlen(u));
 			}
 
-
 			/*
 			 * Subtracted 1 from the width to take the space
 			 * between characters into account. Is there a
@@ -2609,17 +2736,6 @@ _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
 				  current_x + xo + ww - 1,
 				  current_y + line_descent);
 		    }
-
-		    if (is_wide)
-		    {
-			current_x += XTextWidth16(fontstruct,
-				      (XChar2b *)comp->data, comp->length / 2);
-		    }
-		    else
-		    {
-			current_x += XTextWidth(fontstruct,
-						comp->data, comp->length);
-		    }
 		}
 		break;
 
@@ -2628,48 +2744,47 @@ _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
 				   gc, current_x, current_y,
 				   comp->data, comp->length);
 
-                /* Added by SG 08/08/1998
-                 * Now *first* underline, then update current_x
-                 */
+		/* Added by SG 08/08/1998
+		 * Now *first* underline, then update current_x
+		 */
 
-                if (u && (p = strstr(comp->data, u)))
-                {           /* Underline */
-                int ww, llen = p - comp->data;
+		if (u && (p = strstr(comp->data, u)))
+		{           /* Underline */
+		    int ww, llen = p - comp->data;
 
 		    XmbTextExtents((XFontSet)fontlist[comp->font].font,
-			       comp->data, llen, &ink, &log);
+				   comp->data, llen, &ink, &log);
 		    xo = log.width;
 
-                    /* ww = width of underlined text */
+		    /* ww = width of underlined text */
 		    XmbTextExtents((XFontSet)fontlist[comp->font].font,
 			       u, strlen(u), &ink, &log);
-                    ww = log.width;
+		    ww = log.width;
 
-                    /*
-                     * Subtracted 1 from the width to take the space
-                     * between characters into account. Is there a
-                     * decent way to do this ??  Danny 17/4/96
-                     */
-                    XDrawLine(d, w, gc,
-                              current_x + xo,
-                              current_y + line_descent,
-                              current_x + xo + ww - 1,
-                              current_y + line_descent);
-                }
-
-		
-		XmbTextExtents((XFontSet)fontlist[comp->font].font,
-			       comp->data, comp->length, &ink, &log);
-		/* need to do something with the extents of the font set here. */
-		current_x += log.width; /* Try this ! */
-		break;
+		    /*
+		     * Subtracted 1 from the width to take the space
+		     * between characters into account. Is there a
+		     * decent way to do this ??  Danny 17/4/96
+		     */
+		    XDrawLine(d, w, gc,
+			      current_x + xo,
+			      current_y + line_descent,
+			      current_x + xo + ww - 1,
+			      current_y + line_descent);
+		}
 	    }
+
+	    current_x += seg_width;
 	}
 
 	current_y += line_descent;
     }
     _XmStringFreeContext(und_c);
     _XmStringFreeContext(context);
+    if (clipped)
+    {
+	XSetClipMask(d, gc, None);
+    }
 }
 
 void
@@ -2712,6 +2827,118 @@ _XmStringDrawMnemonic(Display *d,
     _XmStringFree(_und);
 }
 
+/* This is essentially _XmStringDraw without the drawing part.
+ * It expects baselines to point to memory already allocated.
+ */
+void
+_XmStringBaselines(XmFontList fontlist, _XmString string, Position y,
+		   Dimension *baselines)
+{
+    _XmStringContext context = NULL;
+    _XmStringComponent comp = NULL;
+    Boolean have_seg, have_line_height;
+    Dimension default_line_height, default_line_ascent;
+    int pending_newlines;
+
+    _XmStringUpdate(fontlist, string);
+
+    _XmStringInitContext(&context, string);
+
+    pending_newlines = 0;
+    have_line_height = False;
+    default_line_height = 0;
+    default_line_ascent = 0;
+
+    while (__XmStringPeekNextComponent(context) != NULL)
+    {
+	Dimension line_height, line_ascent, line_descent;
+	Dimension seg_height, seg_width, seg_ascent, seg_descent;
+	int start;
+
+	start = context->current_component;
+
+	line_height = 0;
+	line_ascent = line_descent = 0;
+	have_seg = False;
+
+	while ((comp = __XmStringGetNextComponent(context)) != NULL)
+	{
+	    if ((comp->type == XmSTRING_COMPONENT_TEXT ||
+		 comp->type == XmSTRING_COMPONENT_LOCALE_TEXT) &&
+		comp->font != XmUNSPECIFIED)
+	    {
+		__XmStringSegmentExtent(fontlist, comp,
+					&seg_width, &seg_height,
+					&seg_ascent, &seg_descent);
+
+		if (seg_height > line_height)
+		{
+		    line_height = seg_height;
+		}
+		if (seg_ascent > line_ascent)
+		{
+		    line_ascent = seg_ascent;
+		}
+		if (seg_descent > line_descent)
+		{
+		    line_descent = seg_descent;
+		}
+
+		have_seg = True;
+	    }
+	    else if (comp->type == XmSTRING_COMPONENT_SEPARATOR)
+	    {
+		if (!have_seg && !have_line_height)
+		{
+		    pending_newlines++;
+		}
+		else if (have_seg && !have_line_height)
+		{
+		    default_line_height = line_ascent + line_descent;
+		    default_line_ascent = line_ascent;
+		    have_line_height = True;
+		    while (pending_newlines--)
+		    {
+			*baselines++ = y + default_line_ascent;
+			y += default_line_height;
+		    }
+		}
+		else if (have_seg)
+		{
+		    default_line_height = line_ascent + line_descent;
+		    default_line_ascent = line_ascent;
+		}
+		break;
+	    }
+	}
+
+	if (!have_seg && !have_line_height)
+	{
+	    continue;
+	}
+	else if (!have_seg && have_line_height)
+	{
+	    *baselines++ = y + default_line_ascent;
+	    y += default_line_height;
+	    continue;
+	}
+	else
+	{
+	    y += line_ascent;
+	}
+
+	*baselines++ = y;
+	y += line_descent;
+    }
+    if (comp)
+    {
+	*baselines = have_line_height
+	    ? y + default_line_ascent
+	    : y;
+    }
+    _XmStringFreeContext(context);
+}
+
 /************************* PUBLIC FUNCTIONS *****************************/
 Dimension
 XmStringBaseline(XmFontList fontlist,
@@ -2755,50 +2982,69 @@ XmStringByteCompare(XmString s1,
     return ret;
 }
 
-Boolean 
+Boolean
 XmStringCompare(XmString s1, XmString s2)
 {
  XmStringContext context1 = NULL, context2 = NULL;
  char *text1, *text2, *tag1, *tag2;
  Boolean separator1, separator2, compare_tags;
  XmStringDirection direction1, direction2;
- 
- if (!_XmStringIsXmString(s1) || !_XmStringIsXmString(s2))
- 	return False;
- 
- if (!XmStringInitContext(&context1, s1))
- 	return False;
- if (!XmStringInitContext(&context2, s2))
- 	{ XmStringFreeContext(context1); return False; }
 
- while (XmStringGetNextSegment(context1, 
-         &text1, &tag1, &direction1, &separator1)) {
- 	if (!XmStringGetNextSegment(context2, 
-              &text2, &tag2, &direction2, &separator2)) {
-        	XtFree(text1);
-        	XmStringFreeContext(context1);
-        	return False;      
-        }
-        compare_tags = (strcmp(tag1, XmFONTLIST_DEFAULT_TAG) != 0 &&
-                        strcmp(tag2, XmFONTLIST_DEFAULT_TAG) != 0);
-         /* Don't compare tags when one of them = XmFONTLIST_DEFAULT_TAG */
-        if (strcmp(text1, text2) != 0 /* diferent text */
-            || direction1 != direction2
-            || (compare_tags && strcmp(tag1, tag2) != 0)
-            || separator1 != separator2) {
-        	XtFree(text1);
+ if (!_XmStringIsXmString(s1) || !_XmStringIsXmString(s2))
+	return False;
+
+ if (!XmStringInitContext(&context1, s1))
+	return False;
+ if (!XmStringInitContext(&context2, s2))
+	{ XmStringFreeContext(context1); return False; }
+
+ while (XmStringGetNextSegment(context1,
+	 &text1, &tag1, &direction1, &separator1)) {
+	if (!XmStringGetNextSegment(context2,
+	      &text2, &tag2, &direction2, &separator2)) {
+		XtFree(text1);
+		XtFree(tag1);
+		XmStringFreeContext(context1);
+		XmStringFreeContext(context2);
+		return False;
+	}
+	compare_tags = (strcmp(tag1, XmFONTLIST_DEFAULT_TAG) != 0 &&
+			strcmp(tag2, XmFONTLIST_DEFAULT_TAG) != 0);
+	 /* Don't compare tags when one of them = XmFONTLIST_DEFAULT_TAG */
+	/*
+	printf("%p %p\n>%s< >%s<\n%i %i\n%i %i\n", text1, text2, text1, text2, direction1, direction2, separator1, separator2);
+	*/
+	if (
+	    ((text1 == NULL || text2 == NULL) && text1 != text2) 
+	    || (text1 != text2 && strcmp(text1, text2) != 0) /* diferent text */
+	    || direction1 != direction2
+	    || (compare_tags && strcmp(tag1, tag2) != 0)
+	    || separator1 != separator2
+	   )
+	{
+		/*
+		printf("Different\n");
+		*/
+		XtFree(text1);
 		XtFree(text2);
-        	XmStringFreeContext(context1);
-        	XmStringFreeContext(context2);
-        	return False;
-        }
+		XtFree(tag1);
+		XtFree(tag2);
+		XmStringFreeContext(context1);
+		XmStringFreeContext(context2);
+		return False;
+	}
+		/*
+		printf("Equal\n");
+		*/
  }
 
  XtFree(text1);
  XtFree(text2);
+ XtFree(tag1);
+ XtFree(tag2);
  XmStringFreeContext(context1);
  XmStringFreeContext(context2);
- return True;  
+ return True;
 }
 
 XmString
@@ -3603,6 +3849,7 @@ XmStringGetLtoR(XmString string,
     Boolean Found = False;
     XmStringContext context = NULL;
 
+    *text = NULL;
     if (!_XmStringIsXmString(string))
     {
 	return False;
@@ -3786,7 +4033,7 @@ XmStringGetNextSegment(XmStringContext context,
 	return False;
     }
 
-    if (text && context->text)
+    if (text)
     {
 	*text = XtNewString(context->text);
     }
@@ -3937,3 +4184,151 @@ XmStringConcatAndFree(XmString str1, XmString str2)
   XmStringFree(str2);
   return tmp;
 }
+
+#if XmVERSION > 1
+/*
+ * MxFTP uses this as
+ *	item = (char *)XmStringUnparse(tempstr,NULL,XmCHARSET_TEXT,
+ *		XmCHARSET_TEXT,NULL,0,XmOUTPUT_ALL);
+ */
+XtPointer
+XmStringUnparse(XmString string, XmStringTag tag, XmTextType tag_type,
+	XmTextType output_type, XmParseTable parse_table,
+	Cardinal parse_count, XmParseModel parse_model)
+{
+    _XmStringContext	context = NULL;
+    _XmStringComponent	comp = NULL;
+    _XmString		_string;
+    Boolean		have_seg, have_line_height;
+    Dimension		default_line_height, default_line_ascent;
+    int			pending_newlines;
+    char		*res = NULL;
+
+    if (tag != NULL) {
+	_XmWarning(NULL, "XmStringUnparse currently only handles tag == NULL case");
+    }
+
+    _string = _XmStringCreate(string);
+    _XmStringInitContext(&context, _string);
+
+    pending_newlines = 0;
+    have_line_height = False;
+    default_line_height = 0;
+    default_line_ascent = 0;
+
+#if 0
+    fprintf(stderr, "XmStringUnparse[internal] _string %p context %p\n",
+			_string, context);
+#endif
+
+    while (__XmStringPeekNextComponent(context) != NULL)
+    {
+	Dimension line_height, line_ascent, line_descent;
+	int start;
+
+	start = context->current_component;
+
+	line_height = 0;
+	line_ascent = line_descent = 0;
+	have_seg = False;
+
+	while ((comp = __XmStringGetNextComponent(context)) != NULL)
+	{
+	    if ((comp->type == XmSTRING_COMPONENT_TEXT ||
+		 comp->type == XmSTRING_COMPONENT_LOCALE_TEXT) &&
+		comp->font != XmUNSPECIFIED)
+	    {
+#if 0
+		fprintf(stderr, "XmStringUnparse[internal] len %d '%s' font %d\n",
+			comp->length, comp->data ? comp->data : "(null)", comp->font);
+#endif
+		if (comp->data) {
+			if (res) {
+				res = realloc(res, strlen(res) + 1 + strlen(comp->data));
+				strcat(res, comp->data);
+			} else {
+				res = strdup(comp->data);
+			}
+		}
+			
+	    }
+	    else if (comp->type == XmSTRING_COMPONENT_SEPARATOR)
+	    {
+	    }
+	}
+
+    }
+    _XmStringFreeContext(context);
+    _XmStringFree(_string);
+
+    return res;
+}
+
+XmString
+XmStringParseText(XtPointer text, XtPointer *text_end, XmStringTag tag,
+	XmTextType type, XmParseTable parse_table, Cardinal parse_count,
+	XtPointer call_data)
+{
+	return NULL;
+}
+
+Cardinal
+XmStringToXmStringTable(XmString string, XmString break_comp, XmStringTable *table)
+{
+	return 0;
+}
+
+XmString
+XmStringTableToXmString(XmStringTable table, Cardinal count, XmString break_component)
+{
+	return NULL;
+}
+
+XtPointer *
+XmStringTableUnparse(XmStringTable table, Cardinal count, XmStringTag tag,
+	XmTextType tag_type, XmTextType output_type, XmParseTable parse,
+	Cardinal parse_count, XmParseModel parse_model)
+{
+	return NULL;
+}
+
+XmStringTable
+XmStringTableParseStringArray(XtPointer *strings, Cardinal count,
+	XmStringTag tag, XmTextType type, XmParseTable parse,
+	Cardinal parse_count, XtPointer call_data)
+{
+	return NULL;
+}
+
+XmString
+XmStringPutRendition(XmString string, XmStringTag rendition)
+{
+	return NULL;
+}
+
+XmParseMapping
+XmParseMappingCreate(ArgList  arg_list, Cardinal arg_count)
+{
+	return NULL;
+}
+
+void
+XmParseMappingSetValues(XmParseMapping parse_mapping, ArgList arg_list, Cardinal arg_count)
+{
+}
+
+void
+XmParseMappingGetValues(XmParseMapping parse_mapping, ArgList arg_list, Cardinal arg_count)
+{
+}
+
+void
+XmParseMappingFree(XmParseMapping parse_mapping)
+{
+}
+
+void
+XmParseTableFree(XmParseTable parse_table, Cardinal parse_count)
+{
+}
+#endif
